// EventRepository.cpp
//
// Version	1.0
// Date		08.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../GeneralInclude/Include.h"
#include "../TelemetryManagement/Telemeterable.h"
#include "../BasicObjects/Include.h"
#include "../TelemetryManagement/TelemetryStream.h"
#include "../FailureRecovery/RecoveryAction.h"
#include "../FailureDetection/ConsistencyCheckable.h"
#include "../AocsEvent/Include.h"

// -------------------------------------------------------------------
// Define static variables
int EventRepository::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------
// Default constructor.

EventRepository::EventRepository()
{
	setClassId(ID_EVENTREPOSITORY);			// assign class identifier

	tmImageLength[shortTm]  = 0;	
	tmImageLength[normalTm] = 8*sizeof(short)+8*sizeof(unsigned char);
	tmImageLength[longTm]   = tmImageLength[normalTm]+8*sizeof(InstanceId);
	tmImageLength[debugTm]  = tmImageLength[normalTm]+8*sizeof(InstanceId);

	list = NULL;
	listSize = 0;
	
	localReset();							// reset the local object's state
	
	localResetConfiguration();				// reset the local object's configuration

}

// -------------------------------------------------------------------
// Allocate memory for list. This method can only be called once.

void EventRepository::setRepositorySize(int n)
{
	if (list != NULL)			// method was already called!
	{	reportConfigurationError(this,MEMORY_ALREADY_ALLOCATED,this);
		return;
	}

	list = new AocsEvent*[n];
	listSize = n;
}

// -------------------------------------------------------------------
// Insert a new event and return a pointer to it
// NB There is no protection against attempting to insert an event in 
//    a list for which memory has not yet been allocated

AocsEvent* EventRepository::insert()
{
	counter++;

	if (counter == 1)	
		latestEvent = 0;		// first event in the list	
	else	
		if (latestEvent<(listSize-1))
			latestEvent++;
		else
			latestEvent=0;
	
	return list[latestEvent];
}

// -------------------------------------------------------------------
// Insert a new event and return a pointer to it

void EventRepository::setList(int i,AocsEvent* evt)
{
	list[i]=evt;
}

// -------------------------------------------------------------------
// Return pointer to most recently stored event.
// After the list has been initialized and memory for it has been 
// allocated, list[latestEvent] points to the last event to have
// been inserted in the event buffer.
// list[iteratorPointer] is the element currently being retrieved by the
// iterator operations.
// iterationCounter is the number of elements to be retrieved in a full
// iteration cycle.

AocsEvent* EventRepository::latest()
{
	if (counter == 0)				// there are no events
		return NULL;

	iteratorPointer = latestEvent;

	if (counter<listSize)	
		iterationCounter = counter;	// list has not yet filled up
	else
		iterationCounter = listSize;
	
	return list[iteratorPointer];
}

// -------------------------------------------------------------------
// Method to iterate through the events in the list

AocsEvent* EventRepository::previous()
{
	iterationCounter--;

	if (iteratorPointer == 0)
		iteratorPointer = listSize-1;
	else
		iteratorPointer--;

	return list[iteratorPointer];
}

// -------------------------------------------------------------------
// Termination method for list iterator.
// Returns true at the end of an iteration through all events inserted in
// repository.

bool EventRepository::isIterationEnded()
{
	return (iterationCounter == 0);
}

// -------------------------------------------------------------------
// Reset any on-going iterations and the event counter

void EventRepository::localReset()
{
	latestEvent = 0;
	iteratorPointer = latestEvent;
	iterationCounter = 0;
	counter = 0;
	counterOld = 0;

	for (AocsEvent* evt=latest(); !isIterationEnded(); previous())
		evt->reset();
}


// -------------------------------------------------------------------
// Reset the global repository state

void EventRepository::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------
// Reset the local repository configuration

void EventRepository::localResetConfiguration()
{
	recoveryAction = NULL;

	for (AocsEvent* evt=latest(); !isIterationEnded(); previous())
		evt->resetConfiguration();
}

// -------------------------------------------------------------------
// Reset the global repository configuration

void EventRepository::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------
// Check if repository is configured

bool EventRepository::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (list == NULL)
		return false;

	for (AocsEvent* evt=latest(); !isIterationEnded(); previous())
		if (!evt->isConfigured()) 
			return false;

	return true;
}

// -------------------------------------------------------------------
// Write events' state to telemetry

void EventRepository::writeToTelemetry(TelemetryStream* stream)
{
	AocsObject::writeToTelemetry(stream);

	switch (getTelemetryFormat()) 
	{
		case debugTm :							// fall-through is intended!
		case longTm :
			if (recoveryAction!=NULL)
				stream->write(recoveryAction->getInstanceId());
			else
				stream->write((InstanceId)NULL);
		case normalTm :
			stream->write((unsigned short)counter);
			stream->write((unsigned char)listSize);
	}

	for (AocsEvent* evt=latest(); !isIterationEnded(); evt=previous())
		evt->writeToTelemetry(stream);
}

// -------------------------------------------------------------------
// The telemetry format of the repository is imposed on all events in 
// the repository

void EventRepository::setTelemetryFormat(TelemetryFormat newFormat)
{
	AocsObject::setTelemetryFormat(newFormat);

	for (AocsEvent* evt=latest(); !isIterationEnded(); evt=previous())
		evt->setTelemetryFormat(newFormat);
}

// -------------------------------------------------------------------

int EventRepository::getTelemetryImageLength()
{
	int temp=0;
	for (AocsEvent* evt=latest(); !isIterationEnded(); previous())
		temp+=evt->getTelemetryImageLength();

	return (temp+tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// Check fails if more than listSize events are created in between calls
// doConsistencyCheck
// NB This version of the program assumes that counter never overflows

bool EventRepository::doConsistencyCheck()
{
	if ( (counter-counterOld)>listSize )	// some events were lost
	{	counterOld = counter;
		return false;
	}

	counterOld = counter;
	return true;
}

// -------------------------------------------------------------------

RecoveryAction* EventRepository::getRecoveryAction()
{
	return recoveryAction;
}

// -------------------------------------------------------------------

void EventRepository::setRecoveryAction(RecoveryAction* r)
{
	recoveryAction = r;
}