// AocsUnitFactory.h
//
// Version	1.0
// Date		11.07.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Factory Class for Aocs Units.
// AOCS units are created and partly configured. Only the part of the
// configuration that does not depend on outputs from other AOCS
// factories is done here. In particular, units are not linked to their
// destination and source buffers and no recovery action is plugged in.
// All units are embedded in reconfiguration managers.
// -------------------------------------------------------------------

#ifndef AocsUnitFactoryH
#define AocsUnitFactoryH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../ReconfigurationManagement/BasicUnitReconfigurer.h"
#include "../AocsUnit/FunctionalTriggerList.h"
#include "../AocsUnit/AocsUnit.h"
#include "../ReconfigurationManagement/RwSet.h"
#include "../AocsUnit/MacsTcController.h"
#include "../AocsUnit/FollowerUnitTriggerModeManager.h"
#include "../AocsUnit/TorquingThrusters.h"

// -------------------------------------------------------------------

class AocsUnitFactory {

	AocsUnit* fssA;
	AocsUnit* fssB;
	AocsUnit* gyrA;
	AocsUnit* gyrB;
	AocsUnit* sapA;
	AocsUnit* sapB;
	AocsUnit* rw[4];

	TorquingThrusters* thu;

	BasicUnitReconfigurer* fss;
	BasicUnitReconfigurer* gyr;
	BasicUnitReconfigurer* sap;
	RwSet* rwSet;

	MacsTcController* unitHw;

	FunctionalTriggerList* triggerListAcquisition;
	FunctionalTriggerList* cspTriggerListSend;
	FunctionalTriggerList* fspTriggerListSend;

	FollowerUnitTriggerModeManager* unitTriggerModeMngAcquisition;
	FollowerUnitTriggerModeManager* unitTriggerModeMngSend;

	UnitTrigger* unitTriggerAcquisition;
	UnitTrigger* unitTriggerSend;

public:

	AocsUnitFactory();	

	// Construct the interface to the external units
	virtual void makeAocsUnitHardware(
		short* macsCsReg,short* MacsInstrReg,short* MacsDataReg);

	// Construct the units
	virtual void makeFss(AocsData* dataDestBuf, 
							RecoveryAction* hwFailureA,
								RecoveryAction* hwFailureB,
									RecoveryAction* reconfigError);

	virtual void makeGyr(AocsData* dataDestBuf, 
							RecoveryAction* hwFailureA,
								RecoveryAction* hwFailureB,
									RecoveryAction* reconfigError);

	virtual void makeRw(int wheelNumber,
							DataItemWrite dataDestBuf, 
								DataItemWrite dataSourceBuf, 
									RecoveryAction* hwFailure);

	virtual void makeRwSet(AocsData* dataDestBuf, 
								AocsData* dataSourceBuf, 
									RecoveryAction* reconfigError);

	virtual void makeSap(AocsData* sas1, 
							AocsData* sas2, 
								AocsData* sas3,
									AocsData* thuOnTime,
										AocsData* thuDelayTime,
											RecoveryAction* hwFailureA,
												RecoveryAction* hwFailureB,
													RecoveryAction* reconfigError);

	virtual void makeThu(AocsData* dataSourceBuf);

	// Construct the unit trigger lists
	virtual void makeUnitTriggerLists();

	// Construct the mode managers for the unit triggers
	virtual void makeUnitTriggerModeManagers(
			ModeManager* masterModeManager, ModeChangeAction* mca);

	// Construct the unit triggers
	virtual void makeUnitTriggers();

	// Getter methods for unit objects
	AocsUnitFunctional* getFss()
		{ return fss; }
	AocsUnit* getFssA()
		{ return fssA; }
	AocsUnit* getFssB()
		{ return fssB; }

	AocsUnitFunctional* getGyr()
		{ return gyr; }
	AocsUnit* getGyrA()
		{ return gyrA; }
	AocsUnit* getGyrB()
		{ return gyrB; }

	AocsUnitFunctional* getSap()
		{ return sap; }
	AocsUnit* getSapA()
		{ return sapA; }
	AocsUnit* getSapB()
		{ return sapB; }

	AocsUnit* getRw(int i)
		{ return rw[i]; }

	// Getter methods for fictitious AOCS units
	AocsUnitFunctional* getRwSet()
		{ return rwSet; }

	AocsUnitFunctional* getThu()
		{ return thu; }

	// Getter methods for reconfiguration managers
	Reconfigurable* getFssReconfigurer()
		{ return fss; }

	Reconfigurable* getGyrReconfigurer()
		{ return gyr; }

	Reconfigurable* getSapReconfigurer()
		{ return sap; }

	Reconfigurable* getRwReconfigurer()
		{ return rwSet; }

	// Getter methods for the trigger lists
	TriggerList* getTriggerListAcquisition()
		{ return triggerListAcquisition; }
	TriggerList* getCspTriggerListSend()
		{ return cspTriggerListSend; }
	TriggerList* getFspTriggerListSend()
		{ return fspTriggerListSend; }

	// Getter methods for the unit triggers
	UnitTrigger* getUnitTriggerAcquisition()
		{ return unitTriggerAcquisition; }
	UnitTrigger* getUnitTriggerSend()
		{ return unitTriggerSend; }
};

#endif
