// ControllerFactory.cpp 
//
// Version	1.0
// Date		10.11.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Controller Management Framelet Factory Class 
// -------------------------------------------------------------------

// Include framework header files
#include "../ControllerManagement/Controller.h"
#include "../ControllerManagement/ControllerManager.h"
#include "../ControllerManagement/ControllerModeManager.h"
#include "../ControllerManagement/FollowerControllerModeManager.h"
#include "../ObjectList/ObjectListTemplate.h"
#include "../AocsEvent/ConfigurationEventRepository.h"
#include "../SequentialDataProcessing/P_Block.h"
#include "../SequentialDataProcessing/D_Block.h"
#include "../SequentialDataProcessing/I_Block.h"
#include "../SequentialDataProcessing/AdderBlock.h"
#include "../SequentialDataProcessing/SplitterBlock.h"
#include "../Xmath/Include.h"
#include "../SequentialDataProcessing/XmathUcbPidBlock.h"
#include "../AocsEvent/EventTypeId.h"

// Include the AOCS configuration file
#include "../AocsFactory/AocsConfiguration.h"

// Include the AOCS factory files
#include "../AocsFactory/ControllerFactory.h"

// -------------------------------------------------------------------

ControllerFactory::ControllerFactory()
{
	manager = NULL;
	modeManager = NULL;

	makeControllerList();
}

// -------------------------------------------------------------------
// Create and configure the controller mode manager

void ControllerFactory::makeControllerModeManager(
						ModeManager* master, ModeChangeAction* mca)
{
	if (modeManager == NULL)				// create only once
	{	FollowerControllerModeManager* fcmm = 
			new FollowerControllerModeManager(
					N_CONT_MODES,MAX_CONT_MONITORS,master);
		
		// Plug in the mode change action
		fcmm->setModeChangeAction(mca);

		// Plug in controller lists
		for (int i=0; i<N_CONT_MODES; i++)
			fcmm->loadControllerList(i,controllerList[i]);

		// Assign internal variable
		modeManager = fcmm;
	}
}
	
// -------------------------------------------------------------------
// Create and configure the controller manager.

void ControllerFactory::makeControllerManager()
{
	if (manager == NULL)					// create only once
	{	manager = new ControllerManager();
		if (modeManager == NULL)
			AocsObject::getConfigurationEventRepository()->
				create(NULL,MISSING_CONT_MM,NULL);
		else
			manager->setModeManager(modeManager);
	}

}

// -------------------------------------------------------------------
// Return the controller list for the i-th mode.

ObjectListTemplate<Controller>* 
						ControllerFactory::getControllerList(int mode)
{
	if (mode >= N_CONT_MODES)
	{	AocsObject::getConfigurationEventRepository()->
			create(NULL,ILLEGAL_FACTORY_PAR,NULL);
		return NULL;
	}
	else
		return controllerList[mode];
}

// -------------------------------------------------------------------
// Create and configure the list of controllers.
// controllerList[i] contains the list of controllers for the 
// i-th mode. The controller component has N_CONT_MODES modes and
// it manages N_OF_CONTROLLERS controllers.

void ControllerFactory::makeControllerList()
{
	if (controllerList==NULL)				// do only once
	{	controllerList = new 
				ObjectListTemplate<Controller>*[N_CONT_MODES];
		for (int i=0; i<N_CONT_MODES; i++)
		{	ObjectListTemplate<Controller>* ccList = 
					new ObjectListTemplate<Controller>();
			ccList->allocateMemory(N_OF_CONTROLLERS);
			controllerList[i] = ccList;
		}
	}
}

// -------------------------------------------------------------------
// Construct a 'minimal' concrete controller

Controller* ControllerFactory::makeController(DataItemRead measurementInput,
								DataItemWrite actuatorCommand,
								AbstractControlChannel* compensator,
								RecoveryAction* instabilityRecoveryAction)
{
	// Create the controller
	Controller* c = new Controller();

	// Load the configuration parameters
	c->linkSensorMeasurement(measurementInput);
	c->linkActuatorCommand(actuatorCommand);
	c->setCompensator(compensator);
	c->setInstabilityRecoveryAction(instabilityRecoveryAction);

	return c;
}

// -------------------------------------------------------------------
// Construct a full concrete controller. Some of the method
// parameters may be undefined or unconfigured in which case
// they are not loaded into the controller.

Controller* ControllerFactory::makeController(DataItemRead measurementInput,
								DataItemRead referenceInput,
								DataItemRead feedforwardInput,
								DataItemWrite actuatorCommand,
								DataItemWrite compensatorOutput,
								DataItemWrite controlError,
								AbstractControlChannel* compensator,
								AbstractControlChannel* measurementFilter,
								AbstractControlChannel* referenceInputFilter,
								AbstractControlChannel* feedforwardFilter,
								RecoveryAction* instabilityRecoveryAction)
{
	// Create the controller
	Controller* c = new Controller();

	// Load the mandatory configuration parameters
	c->linkSensorMeasurement(measurementInput);
	c->linkActuatorCommand(actuatorCommand);
	c->setCompensator(compensator);
	c->setInstabilityRecoveryAction(instabilityRecoveryAction);

	// If not empty, load the optional configuration parameters
	if (referenceInput.isDataItemReadConfigured())
	{	c->linkReferenceSignal(referenceInput);
		if (referenceInputFilter!=NULL)
			c->setReferenceSignalFilter(referenceInputFilter);
	}

	if (feedforwardInput.isDataItemReadConfigured())
	{	c->linkFeedforwardSignal(feedforwardInput);
		if (feedforwardFilter!=NULL)
			c->setFeedforwardFilter(feedforwardFilter);
	}

	if (compensatorOutput.isDataItemWriteConfigured())
		c->linkCompensatorOutput(compensatorOutput);

	if (controlError.isDataItemWriteConfigured())
		c->linkControllerError(controlError);

	if (measurementFilter!=NULL)
		c->setMeasurementFilter(measurementFilter);

	return c;
}

// -------------------------------------------------------------------
// Construct a proportional controller.

AbstractControlChannel* ControllerFactory::makePController(Real kp)
{
	P_Block* p = new P_Block();
	p->setGain(kp);
	return p;
}

// -------------------------------------------------------------------
// Construct a derivative controller.

AbstractControlChannel* ControllerFactory::makeDController(Real kd, 
												Real maximumDerivative)
{
	D_Block* d = new D_Block();
	d->setGain(kd);
	d->setMaximumDerivative(maximumDerivative);
	return d;
}

// -------------------------------------------------------------------
// Construct a proportional-derivative controller. The controller
// is obtained by combining a P and a D controller.

AbstractControlChannel* ControllerFactory::makePDController(
							Real kp, Real kd, Real maximumDerivative)
{
	// Create and configure proportional block
	P_Block* p = new P_Block();
	p->setGain(kp);

	// Create and configure derivative block
	D_Block* d = new D_Block();
	d->setGain(kd);
	d->setMaximumDerivative(maximumDerivative);

	// Create and adder block and link the P- and D-blocks to it
	AdderBlock* adder = new AdderBlock;
	adder->linkInput(d,0,0);
	adder->linkInput(p,0,1);

	// Create and configure the superblock where the P- and D-blocks
	// will be embedded 
	ControlChannelSuperBlock* pd = new ControlChannelSuperBlock(1,1);
	SplitterBlock* splitter = new SplitterBlock(2);
	d->linkInput(splitter,0,0);
	p->linkInput(splitter,1,0);
	pd->linkInputInternal(splitter,0,0);
	pd->linkOutputInternal(adder,0,0);

	return (AbstractControlChannel*)pd;
}

// -------------------------------------------------------------------
// Construct an integral controller.

AbstractControlChannel* ControllerFactory::makeIController(Real ki)
{
	I_Block* i = new I_Block();
	i->setGain(ki);
	return i;
}

// -------------------------------------------------------------------
// Construct a PID controller. The controller implementation is
// a wrapper for an Xmath-generated subroutine. Note that the 
// controller parameter are hard-coded in the Xmath-generated
// subroutine.

AbstractControlChannel* ControllerFactory::makePIDController(
												AocsTime samplingTime)
{
	XmathUcbPidBlock* pid = new XmathUcbPidBlock((AocsTime)samplingTime);
	return pid;
}
