// ControllerFactory.h
//
// Version	1.0
// Date		10.11.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Factory Class for Controller Management Framelet.
//
// The factory constructs the following components:
//	- a controller manager, 
//	- a controller mode manager,
//	- a list of concrete controllers (one for each mode),
//	- several types of concrete control channels implementing useful
//	  transfer functions like PID, PD, etc.
//
// This class should be customized (by inheritance) for each AOCS
// application.
// -------------------------------------------------------------------

#ifndef ControllerFactoryH
#define ControllerFactoryH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../AocsData/DataItemRead.h"
#include "../AocsData/DataItemWrite.h"

// -------------------------------------------------------------------

class ControllerFactory {

	ControllerManager* manager;
	ControllerModeManager* modeManager;

	// controllerList[i] contains the list of controllers for the 
	// i-th mode. 
	ObjectListTemplate<Controller>** controllerList;

protected:

	virtual void makeControllerList();

public:

	ControllerFactory();	

	// Construct and configure the manoeuvre manager
	virtual void makeControllerManager();

	// Construct the controller mode manager
	virtual void makeControllerModeManager(
				ModeManager* masterModeManager, ModeChangeAction* mca);

	// Construct a 'minimal' concrete controller
	Controller* makeController(	DataItemRead measurementInput,
								DataItemWrite actuatorCommand,
								AbstractControlChannel* compensator,
								RecoveryAction* instabilityRecoveryAction);

	// Construct a full concrete controller. Some of the method
	// parameters may be undefined or unconfigured in which case
	// they are not loaded into the controller.
	Controller* makeController(	DataItemRead measurementInput,
								DataItemRead referenceInput,
								DataItemRead feedforwardInput,
								DataItemWrite actuatorCommand,
								DataItemWrite compensatorOutput,
								DataItemWrite controlError,
								AbstractControlChannel* compensator,
								AbstractControlChannel* measurementFilter,
								AbstractControlChannel* referenceInputFilter,
								AbstractControlChannel* feedforwardFilter,
								RecoveryAction* instabilityRecoveryAction);

	// Contruct concrete controllers as instances of abstract control 
	// channels. The PID controller is built using an Xmath-generate
	// routine.
	// (NB: since the controllers are returned as instances of
	// AbstractControlChannel, the client of the factory cannot change 
	// their parameters without performing a downcast!)
	AbstractControlChannel* makePController(Real kp);
	AbstractControlChannel* makeDController(Real kd, 
												Real maximumDerivative);
	AbstractControlChannel* makeIController(Real ki);
	AbstractControlChannel* makePDController(Real kp, Real kd, 
												Real maximumDerivative);
	AbstractControlChannel* makePIDController(AocsTime samplingTime);

	// Return the controller list for the i-th mode
	ObjectListTemplate<Controller>* getControllerList(int mode);

	// Getter methods 
	ControllerManager* getControllerManager()
		{ return manager; }

	ControllerModeManager* getControllerModeManager()
		{ return modeManager; }
};

#endif
