// SystemFactory.cpp 
//
// Version	1.0
// Date		14.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include framework header files
#include "../AocsEvent/FailureEventRepository.h"
#include "../AocsEvent/ReconfigurationEventRepository.h"
#include "../AocsEvent/ConfigurationEventRepository.h"
#include "../AocsEvent/SystemEventRepository.h"
#include "../AocsEvent/ChangeEventRepository.h"
#include "../AocsEvent/ModeEventRepository.h"
#include "../AocsEvent/TelecommandEventRepository.h"
#include "../AocsEvent/RecoveryEventRepository.h"
#include "../AocsEvent/ManoeuvreEventRepository.h"
#include "../SystemManagement/SystemManager.h"
#include "../OperatingSystemObjects/RtemsAocsClock.h"
#include "../AocsData/AttitudeDataPool.h"
#include "../AocsData/DataItemWrite.h"
#include "../ModeManagement/AocsMissionModeManager.h"
#include "../ModeManagement/ModeManager.h"
#include "../ReconfigurationManagement/ReconfigurerHelper.h"
#include "../FailureRecovery/RecoveryStrategy.h"
#include "../BasicObjects/AocsObject.h"
#include "../TelecommandManagement/TelecommandManager.h"
#include "../ManoeuvreManagement/Manoeuvre.h"
#include "../ManoeuvreManagement/ManoeuvreManager.h"

// Include the factory declaration files
#include "../AocsFactory/SystemFactory.h"

// Include the AOCS configuration file
#include "../AocsFactory/AocsConfiguration.h"

// -------------------------------------------------------------------

SystemFactory::SystemFactory()
{
	// Reset the static configuration
	AocsObject::resetStaticConfiguration();
	ModeManager::resetStaticConfiguration();
	ReconfigurerHelper::resetStaticConfiguration();
	RecoveryStrategy::resetStaticConfiguration();
	TelecommandManager::resetStaticConfiguration();
	Manoeuvre::resetStaticConfiguration();
	ManoeuvreManager::resetStaticConfiguration();

	// Reset local variables
	systemEvtRep = NULL;
	failureEvtRep = NULL;
	configEvtRep = NULL;
	modeEvtRep = NULL;
	changeEvtRep = NULL;
	reconfigurationEvtRep = NULL;
	recoveryEvtRep = NULL;
	telecommandEvtRep = NULL;
	manoeuvreEvtRep = NULL;
	systemManager = NULL;
	attitudeDataPool = NULL;
	missionModeManager = NULL;

	// Construct the system manager and the system event repository
	makeSystemManager();

	// Create and configure the event repositories and load all
	// static references to event repositories
	makeEventRepositories();
}

// -------------------------------------------------------------------

void SystemFactory::makeSystemManager()
{
	if (systemManager == NULL)		// only do once
	{	
		systemEvtRep = new SystemEventRepository();
		systemEvtRep->setRepositorySize(SYSTEM_EVENT_REPOSITORY_SIZE);	
		systemManager = new SystemManager(MAX_N_AOCS_OBJECTS,
											MAX_N_RECONFIGURER,
											  systemEvtRep);

		// Plug system manager into AocsObject
		AocsObject::setSystemManager(systemManager);
	}
}

// -------------------------------------------------------------------

void SystemFactory::makeEventRepositories()
{
	if (AocsObject::getFailureEventRepository() == NULL)	// only do once
	{
		// Create event repositories (but the system event repository 
		// is created in method makeSystemManager
		failureEvtRep = new FailureEventRepository();
		configEvtRep = new ConfigurationEventRepository();
		modeEvtRep = new ModeEventRepository();
		changeEvtRep = new ChangeEventRepository();
		reconfigurationEvtRep = new ReconfigurationEventRepository();
		recoveryEvtRep = new RecoveryEventRepository();
		telecommandEvtRep = new TelecommandEventRepository();
		manoeuvreEvtRep = new ManoeuvreEventRepository();

		// Allocate memory to event repositories
		failureEvtRep->setRepositorySize(FAILURE_EVENT_REPOSITORY_SIZE);	
		configEvtRep->setRepositorySize(CONFIGURATION_EVENT_REPOSITORY_SIZE);	
		modeEvtRep->setRepositorySize(MODE_EVENT_REPOSITORY_SIZE);	
		changeEvtRep->setRepositorySize(CHANGE_EVENT_REPOSITORY_SIZE);	
		reconfigurationEvtRep->setRepositorySize(RECONFIGURATION_EVENT_REPOSITORY_SIZE);	
		recoveryEvtRep->setRepositorySize(RECOVERY_EVENT_REPOSITORY_SIZE);	
		telecommandEvtRep->setRepositorySize(TELECOMMAND_EVENT_REPOSITORY_SIZE);	
		manoeuvreEvtRep->setRepositorySize(MANOEUVRE_EVENT_REPOSITORY_SIZE);	

		// Fill in static references to event repositories 
		AocsObject::setFailureEventRepository(failureEvtRep);		
		AocsObject::setConfigurationEventRepository(configEvtRep);
		ModeManager::setChangeEventRepository(changeEvtRep);
		ModeManager::setModeEventRepository(modeEvtRep);
		ReconfigurerHelper::setChangeEventRepository(changeEvtRep);
		ReconfigurerHelper::setReconfigurationEventRepository(reconfigurationEvtRep);
		RecoveryStrategy::setRecoveryEventRepository(recoveryEvtRep);
		TelecommandManager::setTelecommandEventRepository(telecommandEvtRep);
		Manoeuvre::setChangeEventRepository(changeEvtRep);
		Manoeuvre::setManoeuvreEventRepository(manoeuvreEvtRep);
		ManoeuvreManager::setManoeuvreEventRepository(manoeuvreEvtRep);
	}
}

// -------------------------------------------------------------------

void SystemFactory::makeAocsClock()
{
	if (AocsObject::getAocsClock() == NULL)			// only do once
	{	
		AocsClock* aocsClock = new RtemsAocsClock(TICKS_PER_CYCLE);

		// Plug AOCS clock into AocsObject and DataItemWrite
		AocsObject::setAocsClock(aocsClock);
		DataItemWrite::setAocsClock(aocsClock);		
	}												
}

// -------------------------------------------------------------------
// Only one data pool - the attitude data pool - is created.
// The same recovery action is associated to all data in the data
// pool and to the data pool itself

void SystemFactory::makeDataPools(RecoveryAction* consistencyCheckFailure)
{
	if (attitudeDataPool == NULL)		// only do once
	{	attitudeDataPool = new AttitudeDataPool();
		attitudeDataPool->setRecoveryAction(consistencyCheckFailure);

		// Configure the data in the attitude data pool
		for (AocsData* d = getAttitudeDataPool()->first();
				!getAttitudeDataPool()->isLast();
					d = getAttitudeDataPool()->next())
			d->setRecoveryAction(consistencyCheckFailure);
	}
}

// -------------------------------------------------------------------

void SystemFactory::makeMissionModeManager(ModeChangeAction* mca)
{
	if (missionModeManager == NULL)		// only do once
	{	missionModeManager = new AocsMissionModeManager(2,0,MAX_N_MODE_MANAGERS);
		missionModeManager->setModeChangeAction(mca);
	}
}

