// AocsPrototypeTest.cpp 
//
// Version	1.0
// Date		16.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Test program for the AOCS prototype
// -------------------------------------------------------------------

// Include system header files
#include <stdio.h>  

// Include framework header files
#include "../FailureDetection/FailureDetectionManager.h"
#include "../FailureRecovery/FailureRecoveryManager.h"
#include "../AocsUnit/UnitTrigger.h"
#include "../TelemetryManagement/TelemetryManager.h"
#include "../TelecommandManagement/TelecommandManager.h"
#include "../TelecommandManagement/TelecommandLoader.h"
#include "../ManoeuvreManagement/ManoeuvreManager.h"
#include "../ControllerManagement/ControllerManager.h"

// Include the AOCS factory and AOCS configuration headers
#include "../AocsFactory/AocsConfiguration.h"
#include "../AocsFactory/SystemFactory.h"
#include "../AocsFactory/TelemetryFactory.h"
#include "../AocsFactory/TelecommandFactory.h"
#include "../AocsFactory/AocsUnitFactory.h"
#include "../AocsFactory/FailureRecoveryFactory.h"
#include "../AocsFactory/FailureDetectionFactory.h"
#include "../AocsFactory/MonitoringFactory.h"
#include "../AocsFactory/ModeFactory.h"
#include "../AocsFactory/ManoeuvreFactory.h"
#include "../AocsFactory/ControllerFactory.h"

// Include test header files
#include "../AocsPrototypeTest/AocsPrototypeTest.h"
#include "../AocsPrototypeTest/AocsTestHarness_2.h"
#include "../GeneralUtilities/FlushTelemetryBuffers.h"
#include "../GeneralUtilities/FlushMacsBuffers.h"
#include "../GeneralUtilities/TestUtilities.h"

// Define factories
SystemFactory systemFactory;
TelemetryFactory telemetryFactory;
TelecommandFactory telecommandFactory;
AocsUnitFactory aocsUnitFactory;
FailureRecoveryFactory failureRecoveryFactory;
FailureDetectionFactory failureDetectionFactory;
MonitoringFactory monitoringFactory;
ModeFactory modeFactory;
ManoeuvreFactory manoeuvreFactory;
ControllerFactory controllerFactory;

// -------------------------------------------------------------------

int main(int argc, char* argv[])
{
	// Open output file 
	FILE* out;
    if( (out  = fopen("AocsPrototypeTestOut.txt","w")) == NULL )
      printf( "\nThe file 'AocsPrototypeTestOut.txt' could not be opened\n" );   
	else
      printf( "\nThe file 'AocsPrototypeTestOut.txt' was successfully opened\n" );
	
	// Print header message to output file
	printTestHeader(out,"AocsPrototypeTest");

	// Initialize factory structure
	struct FrameworkFactories fwFact = {&systemFactory, 
										&telemetryFactory,
										&telecommandFactory,
										&aocsUnitFactory,
										&failureRecoveryFactory,
										&failureDetectionFactory,
										&monitoringFactory,
										&modeFactory,
										&manoeuvreFactory,
										&controllerFactory};

	// Initialize AOCS software
	instantiateAocs(out, fwFact);

	// Instantiate a test harness 
	AocsTestHarness* aocsTestHarness = new AocsTestHarness_2(fwFact, out);

	// Define and load the structure holding the AOCS tasks
	Runnable* aocsTasks[N_AOCS_TASKS];

	for (int k=0; k<N_AOCS_TASKS; k++)
		aocsTasks[k] = NULL;

	aocsTasks[0] = aocsUnitFactory.getUnitTriggerAcquisition();
	aocsTasks[1] = telemetryFactory.getTelemetryManager();
	aocsTasks[2] = telecommandFactory.getTelecommandLoader();
	aocsTasks[3] = telecommandFactory.getTelecommandManager();
	aocsTasks[4] = failureDetectionFactory.getFailureDetectionManager();
	aocsTasks[5] = failureRecoveryFactory.getFailureRecoveryManager();
	aocsTasks[6] = manoeuvreFactory.getManoeuvreManager();
	aocsTasks[7] = controllerFactory.getControllerManager();
	aocsTasks[N_AOCS_TASKS-2] = aocsUnitFactory.getUnitTriggerSend();
	aocsTasks[N_AOCS_TASKS-1] = aocsTestHarness;

	// Simulate running a small number of AOCS cycles
	for (int i=0; i<N_AOCS_CYCLES; i++)
		for (int j=0; j<N_AOCS_TASKS; j++)
			if (aocsTasks[j]!=NULL)
				aocsTasks[j]->run(0);

	// Flush telemetry buffer
	flushTelemetryBuffers(out, aocsTestHarness);

	// Flush MACS buffers
	flushMacsBuffers(out, aocsTestHarness);

	// close output file
	fclose(out);
	return 0;
}

