// AocsTestHarness_1.cpp
//
// Version	1.0
// Date		16.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include system header files
#include <stdio.h>  

// Include framework header files
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/CompilerSwitches.h"
#include "../TelemetryManagement/DmaTelemetryStream.h"
#include "../AocsUnit/AocsUnit.h"
#include "../AocsData/AttitudeDataPool.h"
#include "../TelecommandManagement/ModeChangeTransactionTelecommand.h"
#include "../TelecommandManagement/ReconfigureTransactionTelecommand.h"
#include "../TelecommandManagement/ReconfigureTelecommand.h"
#include "../OperatingSystemObjects/AocsClock.h"

// Include the AOCS factory and AOCS configuration headers
#include "../AocsFactory/AocsUnitFactory.h"
#include "../AocsFactory/TelemetryFactory.h"
#include "../AocsFactory/SystemFactory.h"
#include "../AocsFactory/AocsConfiguration.h"

// Include test header files
#include "../AocsPrototypeTest/AocsTestHarness_1.h"
#include "../GeneralUtilities/TestUtilities.h"


// -------------------------------------------------------------------

AocsTestHarness_1::AocsTestHarness_1(FrameworkFactories f, FILE* out):
					AocsTestHarness(f,out)
{
	macsToggle = 0;
	cmdToggle = 0;
	tcCycleCounter = 0;
}

// -------------------------------------------------------------------
// Load the hardware buffers for incoming MACS data.
// A toggle switch is used to update the loaded values across 
// activations. 

void AocsTestHarness_1::loadMacs()
{
	// Recover the AOCS units that send data to the AOCS computer
	AocsUnitFactory* unitFactory = fwFact.aocsUnitFactory;

	AocsUnit* fssA = unitFactory->getFssA();
	AocsUnit* fssB = unitFactory->getFssB();

	AocsUnit* gyrA = unitFactory->getGyrA();
	AocsUnit* gyrB = unitFactory->getGyrB();

	AocsUnit* sapA = unitFactory->getSapA();
	AocsUnit* sapB = unitFactory->getSapB();

	AocsUnit* rw[4];
	for (int i=0; i<4; i++)
		rw[i] = unitFactory->getRw(i);

	// Set the base value
	if (macsToggle == 0)
	{	macsBase = 0x8400;		// macsBase = 1024
		rwBase = 0x8002;		// rwBase = 2
		macsToggle = 1;
	}
	else
	{	macsBase = 0x0400;		// macsBase = -1024
		rwBase = 0x0002;		// rwBase = -2
		macsToggle = 0;
	}

	// Load the hardware buffers for incoming MACS data
	fssA->setFcHwInpBuff(macsBase,0);
	fssA->setFcHwInpBuff(macsBase,1);
	fssB->setFcHwInpBuff(macsBase,0);
	fssB->setFcHwInpBuff(macsBase,1);

	gyrA->setFcHwInpBuff(macsBase+0x0400,0);
	gyrB->setFcHwInpBuff(macsBase+0x0400,0);

	for (i=0; i<6; i++)
	{	sapA->setFcHwInpBuff(macsBase+0x0800,i);
		sapB->setFcHwInpBuff(macsBase+0x0800,i);
	}

	for (i=0; i<4; i++)
		rw[i]->setFcHwInpBuff(rwBase+2*i,0);
}

// -------------------------------------------------------------------
// Assemble TC buffer representing TC send to the AOCS sw. 
// Telecommands are sent at the fourth and sixth cycle.

void AocsTestHarness_1::loadTelecommands()
{
	tcCycleCounter++;

	if (tcCycleCounter==6)		
		loadTelecommandTransaction();

	if (tcCycleCounter==8)		
		loadSimpleTelecommand();
}

// -------------------------------------------------------------------
// Assemble TC transaction. 
// The TC consists of a TC transaction with two transaction TC's.
// TC1: mode change for the AOCS mission mode manager
// TC2: reconfiguration for FSS
// The TC carries a time tag that should cause it to be executed in 
// the cycle in which it is loaded

void AocsTestHarness_1::loadTelecommandTransaction()
{
	// Recovery the TC buffer
	TelecommandFactory* tcFactory = fwFact.telecommandFactory;
	char* tcBuffer = (char*)tcFactory->getTelecommandBuffer();

	// Load the AOCS mission mode manager address in a 4-byte array
	SystemFactory* systemFactory = fwFact.systemFactory;
	int mmAddressInt = (int)(systemFactory->getMissionModeManager());
	int* mmAddressPtr = &mmAddressInt;
	char* mmAddress = (char*)mmAddressPtr;

	// Create a dummy mode change transaction TC. It is needed to recover  
	// the VTable pointer for class ModeChangeTransactionTelecommand.
	ModeChangeTransactionTelecommand mct(0,NULL,1);

	// Load the VTable pointer for class ModeChangeTransactionTelecommand 
	// into a 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int mctVTableAddress = *((int*)&mct);
#else
	int mctVTableAddress = *(((int*)&mct+3));
#endif
	int* mctVTablePtrInt = &mctVTableAddress;
	char* mctVTablePtr = (char*)mctVTablePtrInt;

	// Load the FSS reconfiguration manager address in a 4-byte array
	AocsUnitFactory* aocsUnitFactory = fwFact.aocsUnitFactory;
	int frAddressInt = (int)(aocsUnitFactory->getFssReconfigurer());
	int* frAddressPtr = &frAddressInt;
	char* frAddress = (char*)frAddressPtr;

	// Create a dummy reconfigure transaction TC. It is needed to recover  
	// the VTable pointer for class ModeChangeTransactionTelecommand.
	ReconfigureTransactionTelecommand rtt(0,NULL);

	// Load the VTable pointer for class ReconfigureTransactionTelecommand 
	// into a 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int rttVTableAddress = *((int*)&rtt);
#else
	int rttVTableAddress = *(((int*)&rtt+3));
#endif
	int* rttVTablePtrInt = &rttVTableAddress;
	char* rttVTablePtr = (char*)rttVTablePtrInt;

	// Load a 4-byte array with the time-tag for the TC
	AocsTime timeTag = systemFactory->getAocsClock()->getTime()+1;
	int* timeTagPtrInt = (int*)&timeTag;
	char* timeTagPtr = (char*)timeTagPtrInt;

	// Prepare the TC buffer. 
	// The buffer consists of one packet.
	// The packet contains two blocks.
	// The first block is a mode change transaction TC.
	// The second block is a reconfigure transaction TC.

	tcBuffer[0]=1;	// start of TC buffer -- 1 packet in TC buffer	
	tcBuffer[1]=2;	// start of TC packet -- 2 blocks in TC packet

	tcBuffer[2]=14;	// start of first TC block -- 14 bytes in TC data section
	tcBuffer[3]=0;	// no VTable section 

	tcBuffer[4]=mctVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[5]=mctVTablePtr[1];
	tcBuffer[6]=mctVTablePtr[2];
	tcBuffer[7]=mctVTablePtr[3];

	tcBuffer[8]=timeTagPtr[0];		// time tag
	tcBuffer[9]=timeTagPtr[1];
	tcBuffer[10]=timeTagPtr[2];
	tcBuffer[11]=timeTagPtr[3];

	tcBuffer[12]=mct.getClassId();	// class identifier

	tcBuffer[13]=mmAddress[0];		// mode manager address
	tcBuffer[14]=mmAddress[1];
	tcBuffer[15]=mmAddress[2];
	tcBuffer[16]=mmAddress[3];

	tcBuffer[17]=FSP;				// new mode

	tcBuffer[18]=9;		// start of second TC block -- 9 bytes in TC data section

	tcBuffer[20]=rttVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[21]=rttVTablePtr[1];
	tcBuffer[22]=rttVTablePtr[2];
	tcBuffer[23]=rttVTablePtr[3];

	tcBuffer[24]=rtt.getClassId();	// class identifier

	tcBuffer[25]=frAddress[0];		// FSS reconfigurer address
	tcBuffer[26]=frAddress[1];
	tcBuffer[27]=frAddress[2];
	tcBuffer[28]=frAddress[3];

}

// -------------------------------------------------------------------
// Assemble simple TC. 
// The TC consists of a simple TC to reconfigure the FSS reconfigurer.
// The TC carries a time tag that should cause it to be executed in 
// the cycle in which it is loaded

void AocsTestHarness_1::loadSimpleTelecommand()
{
	// Recovery the TC buffer
	TelecommandFactory* tcFactory = fwFact.telecommandFactory;
	char* tcBuffer = (char*)tcFactory->getTelecommandBuffer();

	// Load the FSS reconfiguration manager address in a 4-byte array
	AocsUnitFactory* aocsUnitFactory = fwFact.aocsUnitFactory;
	int frAddressInt = (int)(aocsUnitFactory->getFssReconfigurer());
	int* frAddressPtr = &frAddressInt;
	char* frAddress = (char*)frAddressPtr;

	// Create a dummy reconfigure transaction TC. It is needed to recover  
	// the VTable pointer for class ModeChangeTransactionTelecommand.
	ReconfigureTelecommand rtt(0,NULL);

	// Load the VTable pointer for class ReconfigureTransactionTelecommand 
	// into a 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int rttVTableAddress = *((int*)&rtt);
#else
	int rttVTableAddress = *(((int*)&rtt+3));
#endif
	int* rttVTablePtrInt = &rttVTableAddress;
	char* rttVTablePtr = (char*)rttVTablePtrInt;

	// Load a 4-byte array with the time-tag for the TC
	SystemFactory* systemFactory = fwFact.systemFactory;
	AocsTime timeTag = systemFactory->getAocsClock()->getTime()+1;
	int* timeTagPtrInt = (int*)&timeTag;
	char* timeTagPtr = (char*)timeTagPtrInt;

	// Prepare the TC buffer. 
	// The buffer consists of one packet.
	// The packet contains two blocks.
	// The first block is a mode change transaction TC.
	// The second block is a reconfigure transaction TC.

	tcBuffer[0]=1;	// start of TC buffer -- 1 packet in TC buffer	
	tcBuffer[1]=1;	// start of TC packet -- 1 block in TC packet

	tcBuffer[2]=13;	// start of first TC block -- 13 bytes in TC data section
	tcBuffer[3]=0;	// no VTable section 

	tcBuffer[4]=rttVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[5]=rttVTablePtr[1];
	tcBuffer[6]=rttVTablePtr[2];
	tcBuffer[7]=rttVTablePtr[3];

	tcBuffer[8]=timeTagPtr[0];		// time tag
	tcBuffer[9]=timeTagPtr[1];
	tcBuffer[10]=timeTagPtr[2];
	tcBuffer[11]=timeTagPtr[3];

	tcBuffer[12]=rtt.getClassId();	// class identifier

	tcBuffer[13]=frAddress[0];		// FSS reconfigurer address
	tcBuffer[14]=frAddress[1];
	tcBuffer[15]=frAddress[2];
	tcBuffer[16]=frAddress[3];

}

// -------------------------------------------------------------------
// Set the spacecraft torque. 

void AocsTestHarness_1::generateActuatorCommands()
{
	// Recover the spacecraft torque data
	SystemFactory* systemFactory = fwFact.systemFactory;
	AocsData* scTor = systemFactory->getAttitudeDataPool()->getScTorque();

	// Set the spacecraft torque
	if (cmdToggle==0)
	{
		scTor->getDataItemWrite(0).set(1.0);
		scTor->getDataItemWrite(1).set(2.0);
		scTor->getDataItemWrite(2).set(3.0);
		cmdToggle = 1;
	}
	else
	{
		scTor->getDataItemWrite(0).set(-1.0);
		scTor->getDataItemWrite(1).set(-2.0);
		scTor->getDataItemWrite(2).set(-3.0);
		cmdToggle = 0;
	}

}