// AocsUnit.cpp
//
// Version	1.0
// Date		28.05.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include framework header files
#include "../AocsUnit/AocsUnit.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../BasicObjects/AocsObject.h"
#include "../AocsData/DataItemWrite.h"
#include "../SequentialDataProcessing/AbstractControlChannel.h"
#include "../ObjectMonitoring/Property.h"	
#include "../AocsEvent/EventTypeId.h"	
#include "../AocsData/AocsData.h"	

// -------------------------------------------------------------------
// Define static variables
int AocsUnit::tmImageLength[NTMFORMATS];				

// -------------------------------------------------------------------

AocsUnit::AocsUnit(int n, int m, int p, int q, int r, int s)
{
	tmImageLength[shortTm]  = 0;	// Not implemented in fw prototype	
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	monitorList.allocateMemory(0);		// Monitoring is not implemented
	changeObjectList.allocateMemory(0);	// in fw prototype

	nHkInpBuf = n;
	nHkDdBuf = m;
	nFcInpBuf = p;
	nFcDdBuf = q;
	nFcOutBuf = r;
	nFcDsBuf = s;

	// Allocate memory to hardware buffers.
	if (nHkInpBuf>0)
		hkHwInpBuf = new Real[nHkInpBuf];
	else
		hkHwInpBuf = NULL;

	if (nFcInpBuf>0)
		fcHwInpBuf = new Real[nFcInpBuf];
	else
		fcHwInpBuf = NULL;

	if (nFcOutBuf>0)
		fcHwOutBuf = new Real[nFcOutBuf];
	else
		fcHwOutBuf = NULL;

	// Allocate memory to destination and source buffers
	if (nHkDdBuf>0)
		hkDdBuf = new DataItemWrite[nHkDdBuf];
	else
		hkDdBuf = NULL;

	if (nFcDdBuf>0)
		fcDdBuf = new DataItemWrite[nFcDdBuf];
	else
		fcDdBuf = NULL;

	if (nFcDsBuf>0)
		fcDsBuf = new DataItemWrite[nFcDsBuf];
	else
		fcDsBuf = NULL;

	localResetConfiguration();
	localReset();
}

// -------------------------------------------------------------------

void AocsUnit::localReset()
{
	// Set the content of all internal buffers to 0
	for (int i=0; i<nHkInpBuf; i++)
		hkHwInpBuf[i]=0;

	for (i=0; i<nFcInpBuf; i++)
		fcHwInpBuf[i]=0;

	for (i=0; i<nFcOutBuf; i++)
		fcHwOutBuf[i]=0;

	// Reset the converter (if they exist)
	if (hkInConv!=NULL)
		hkInConv->reset();
	if (fcInConv!=NULL)
		fcInConv->reset();
	if (fcOutConv!=NULL)
		fcOutConv->reset();
}
	
// -------------------------------------------------------------------

void AocsUnit::localResetConfiguration()
{
	// Create an empty data item write
	DataItemWrite dw((AocsTime*)NULL,(Real*)NULL);

	// Remove the links to all external buffers,
	for (int i=0; i<nHkDdBuf; i++)
		hkDdBuf[i]=dw;

	for (i=0; i<nFcDdBuf; i++)
		fcDdBuf[i]=dw;

	for (i=0; i<nFcDsBuf; i++)
		fcDsBuf[i]=dw;

	// Remove link to hardware unit object
	hwUnit = NULL;

	// Remove link to recovery action associated to hardware failures
	hwFailureRecAct = NULL;

	// Remove link to data converters
	hkInConv = NULL;
	fcInConv = NULL;
	fcOutConv = NULL;
}	

// -------------------------------------------------------------------

void AocsUnit::reset()
{
	AocsObject::reset();
	localReset();
}


// -------------------------------------------------------------------

void AocsUnit::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool AocsUnit::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (hwUnit == NULL)
		return false;

	if ( (nHkInpBuf!=0) && (hkInConv==NULL) )
		return false;

	if ( (nFcInpBuf!=0) && (fcInConv==NULL) )
		return false;

	if ( (nFcOutBuf!=0) && (fcOutConv==NULL) )
		return false;

	for (int i=0; i<nHkDdBuf; i++)
		if (!hkDdBuf[i].isDataItemWriteConfigured())
			return false;

	for (i=0; i<nFcDdBuf; i++)
		if (!fcDdBuf[i].isDataItemWriteConfigured())
			return false;

	for (i=0; i<nFcDsBuf; i++)
		if (!fcDsBuf[i].isDataItemWriteConfigured())
			return false;

	return true;
}

// -------------------------------------------------------------------

void AocsUnit::setHousekeepingInConverter(AbstractControlChannel* c)
{
	if ( (nHkInpBuf==0) || (nHkDdBuf==0) )
	{	reportConfigurationError(this,ILLEGAL_HK_CONVERTER,this);
		return;
	}

	if ((c->getInputOrder()!=nHkInpBuf) || (c->getOutputOrder()!=nHkDdBuf))
	{	reportConfigurationError(this,ILLEGAL_HK_CONVERTER,this);
		return;
	}

	hkInConv = c;

	// Link the inputs of the converter to the input hw buffer
	for (int i=0; i<nHkInpBuf; i++)
		c->linkInput(DataItemRead(&hkHwInpBuf[i]),i);
}

// -------------------------------------------------------------------

void AocsUnit::setFunctionalInConverter(AbstractControlChannel* c)
{
	if ( (nFcInpBuf==0) || (nFcDdBuf==0) )
	{	reportConfigurationError(this,ILLEGAL_FC_CONVERTER,this);
		return;
	}

	if ((c->getInputOrder()!=nFcInpBuf) || (c->getOutputOrder()!=nFcDdBuf))
	{	reportConfigurationError(this,ILLEGAL_FC_CONVERTER,this);
		return;
	}

	fcInConv = c;

	// Link the inputs of the converter to the input hw buffer
	for (int i=0; i<nFcInpBuf; i++)
		c->linkInput(DataItemRead(&fcHwInpBuf[i]),i);
}

// -------------------------------------------------------------------

void AocsUnit::setFunctionalOutConverter(AbstractControlChannel* c)
{
	if ( (nFcOutBuf==0) || (nFcDsBuf==0) )
	{	reportConfigurationError(this,ILLEGAL_FC_CONVERTER,this);
		return;
	}

	if ((c->getInputOrder()!=nFcDsBuf) || (c->getOutputOrder()!=nFcOutBuf))
	{	reportConfigurationError(this,ILLEGAL_FC_CONVERTER,this);
		return;
	}

	fcOutConv = c;

	// Link the inputs of the converter to the data source buffers
	for (int i=0; i<nFcDsBuf; i++)
		fcOutConv->linkInput(fcDsBuf[i],i);
}

// -------------------------------------------------------------------

void AocsUnit::setHousekeepingInLink(DataItemWrite d, int i)
{
	if ( (i<0) || (i>=nHkDdBuf) )
	{	reportConfigurationError(this,ILLEGAL_HK_DEST_BUF,this);
		return;
	}

	hkDdBuf[i] = d;
}

// -------------------------------------------------------------------

void AocsUnit::setHousekeepingInLink(AocsData* d)
{
	if ( d->getNumberOfItems()!=nHkDdBuf )
	{	reportConfigurationError(this,ILLEGAL_HK_DEST_BUF,this);
		return;
	}

	for (int i=0; i<nHkDdBuf; i++)
		setHousekeepingInLink(d->getDataItemWrite(i),i);
}

// -------------------------------------------------------------------

DataItemWrite AocsUnit::getHousekeepingInLink(int i)
{
	if ( (i<0) || (i>=nHkDdBuf) )
	{	reportConfigurationError(this,ILLEGAL_HK_DEST_BUF,this);
		return DataItemWrite((AocsTime*)NULL,(Real*)NULL);
	}

	return hkDdBuf[i];
}

// -------------------------------------------------------------------

void AocsUnit::setFunctionalInLink(DataItemWrite d, int i)
{
	if ( (i<0) || (i>=nFcDdBuf) )
	{	reportConfigurationError(this,ILLEGAL_FC_DEST_BUF,this);
		return;
	}

	fcDdBuf[i] = d;
}

// -------------------------------------------------------------------

void AocsUnit::setFunctionalInLink(AocsData* d)
{
	if ( d->getNumberOfItems()!=nFcDdBuf )
	{	reportConfigurationError(this,ILLEGAL_FC_DEST_BUF,this);
		return;
	}

	for (int i=0; i<nFcDdBuf; i++)
		setFunctionalInLink(d->getDataItemWrite(i),i);
}

// -------------------------------------------------------------------

void AocsUnit::setFunctionalOutLink(DataItemWrite d, int i)
{
	if ( (i<0) || (i>=nFcDsBuf) )
	{	reportConfigurationError(this,ILLEGAL_FC_SOURCE_BUF,this);
		return;
	}

	fcDsBuf[i] = d;

	// Link the inputs of the converter to the data source buffers
	if (fcOutConv!=NULL)
		fcOutConv->linkInput(fcDsBuf[i],i);
	else
		reportConfigurationError(this,MISSING_FC_OUT_CONV,this);
}

// -------------------------------------------------------------------

void AocsUnit::setFunctionalOutLink(AocsData* d)
{
	if ( d->getNumberOfItems()!=nFcDsBuf )
	{	reportConfigurationError(this,ILLEGAL_FC_SOURCE_BUF,this);
		return;
	}

	for (int i=0; i<nFcDsBuf; i++)
		setFunctionalOutLink(d->getDataItemWrite(i),i);
}

// -------------------------------------------------------------------

DataItemWrite AocsUnit::getFunctionalOutLink(int i)
{
	if ( (i<0) || (i>=nFcDsBuf) )
	{	reportConfigurationError(this,ILLEGAL_FC_SOURCE_BUF,this);
		return DataItemWrite((AocsTime*)NULL,(Real*)NULL);
	}

	return fcDsBuf[i];
}

// -------------------------------------------------------------------

DataItemWrite AocsUnit::getFunctionalInLink(int i)
{
	if ( (i<0) || (i>=nFcDdBuf) )
	{	reportConfigurationError(this,ILLEGAL_FC_DEST_BUF,this);
		return DataItemWrite((AocsTime*)NULL,(Real*)NULL);
	}

	return fcDdBuf[i];
}

// -------------------------------------------------------------------

void AocsUnit::refreshHousekeepingIn()
{
	hkInConv->propagate(getTime());
	for (int i=0; i<nHkDdBuf; i++)
		hkDdBuf[i].set(hkInConv->getOutput(i).get());
}	

// -------------------------------------------------------------------

void AocsUnit::refreshFunctionalIn()
{
	fcInConv->propagate(getTime());
	for (int i=0; i<nFcDdBuf; i++)
		fcDdBuf[i].set(fcInConv->getOutput(i).get());
}	

// -------------------------------------------------------------------

void AocsUnit::refreshFunctionalOut()
{
	fcOutConv->propagate(getTime());
	for (int i=0; i<nFcOutBuf; i++)
		fcHwOutBuf[i]=(fcOutConv->getOutput(i).get());
}	

// -------------------------------------------------------------------

void AocsUnit::setFcHwInpBuff(unsigned short d, int i)
{
	if ((i<0) || (i>=nFcInpBuf))
		reportConfigurationError(this,ILLEGAL_HW_BUF,this);

	Real tempReal=0;
	unsigned short* tempUs = (unsigned short*)(&tempReal);
	*tempUs = d;
	fcHwInpBuf[i] = tempReal; 
}

// -------------------------------------------------------------------

unsigned short AocsUnit::getFcHwOutBuff(int i)
{
	if ((i<0) || (i>=nFcOutBuf))
		reportConfigurationError(this,ILLEGAL_HW_BUF,this);
	
	unsigned short* tempUs = (unsigned short*)(&fcHwOutBuf[i]);
	return (*tempUs);
}


// -------------------------------------------------------------------

void AocsUnit::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// not implemented in framework prototype
}

// -------------------------------------------------------------------

int AocsUnit::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}
