// AocsUnit.h
//
// Version	1.0
// Date		28.05.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Abstract base class for AOCS units 
// -------------------------------------------------------------------

#ifndef AocsUnitH
#define AocsUnitH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../BasicObjects/AocsObject.h"
#include "../AocsUnit/AocsUnitFunctional.h"
#include "../AocsUnit/AocsUnitHousekeeping.h"
#include "../AocsData/DataItemWrite.h"
#include "../AocsUnit/AocsUnitHardware.h"
#include "../SequentialDataProcessing/AbstractControlChannel.h"
#include "../ObjectMonitoring/Property.h"
#include "../ObjectList/ObjectListTemplate.h"

// -------------------------------------------------------------------

class AocsUnit : public AocsObject, 
					public AocsUnitFunctional,  
					   public AocsUnitHousekeeping {

	static int tmImageLength[NTMFORMATS];

protected :
	
	int nHkInpBuf;						// number of hk hw input buffers
	int nHkDdBuf;						// number of hk destination buffers
	int nFcInpBuf;						// number of funct. hw input buffers
	int nFcDdBuf;						// number of funct. destination buffers
	int nFcOutBuf;						// number of funct. hw output buffers
	int nFcDsBuf;						// number of funct. data source buffers

	Real* hkHwInpBuf;					// hw input buffers for hk data
	Real* fcHwInpBuf;					// hw input buffers for funct. data
	Real* fcHwOutBuf;					// hw output buffers for funct. data

	DataItemWrite* hkDdBuf;				// hk data destination buffers
	DataItemWrite* fcDdBuf;				// fc data destination buffers
	DataItemWrite* fcDsBuf;				// fc data source buffers

	AocsUnitHardware* hwUnit;
	
	AbstractControlChannel* hkInConv;	// converter for incoming hk data
	AbstractControlChannel* fcInConv;	// data converter for incoming funct. data
	AbstractControlChannel* fcOutConv;	// data converter for outgoing funct. data
	
	bool isHealthy;

	RecoveryAction* hwFailureRecAct;	// recovery acttion for hw failures

	// Local reset methods
	void localReset();
	void localResetConfiguration();		

	// List of registered health monitors and their associated change objects
	ObjectListTemplate<PropertyMonitor> monitorList;
	ObjectListTemplate<ChangeObject> changeObjectList;

public:

	AocsUnit(int nHkInpBuf, int nHkDdBuf, int nFcInpBuf, int nFcDdBuf,
					int nFcOutBuf, int nFcDsBuf);

	virtual void initialize()
		{ return; }
	virtual bool isInitialized()
		{ return hwUnit->isTransactionFinished(); }

	virtual void selfTest()
		{ return; }
	virtual bool isSelfTestFinished()
		{ return true; }
	virtual int getSelfTestResults()
		{ return 0; }

	virtual void resetUnit()
		{ return; }
	virtual void resetTransaction()
		{ return; }

	virtual void acquireHousekeepingData()
		{ return; }
	virtual bool isHousekeepingTransactionFinished()
		{ return hwUnit->isTransactionFinished(); }

	virtual void acquireFunctionalData()
		{ return; }
	virtual void sendFunctionalData()
		{ return; }
	virtual bool isAcquireTransactionFinished()
		{ return hwUnit->isTransactionFinished(); }
	virtual bool isSendTransactionFinished()
		{ return hwUnit->isTransactionFinished(); }

	virtual void refreshHousekeepingIn();
	virtual void refreshFunctionalIn();
	virtual void refreshFunctionalOut();

	virtual void setHousekeepingInConverter(AbstractControlChannel* c);
	virtual void setFunctionalInConverter(AbstractControlChannel* c);
	virtual void setFunctionalOutConverter(AbstractControlChannel* c);

	virtual void setHousekeepingInLink(DataItemWrite d, int i);
	virtual void setFunctionalInLink(DataItemWrite d, int i);
	virtual void setFunctionalOutLink(DataItemWrite d, int i);
	virtual void setHousekeepingInLink(AocsData* d);
	virtual void setFunctionalInLink(AocsData* d);
	virtual void setFunctionalOutLink(AocsData* d);
	virtual DataItemWrite getHousekeepingInLink(int i);
	virtual DataItemWrite getFunctionalInLink(int i);
	virtual DataItemWrite getFunctionalOutLink(int i);

	virtual void synchronizeHousekeeping()
		{ return; }
	virtual bool isSynchronizeHousekeepingFinished()
		{ return hwUnit->isTransactionFinished(); }

	virtual void synchronizeFunctional()
		{ return; }
	virtual bool isSynchronizeFunctionalFinished()
		{ return hwUnit->isTransactionFinished(); }

	virtual void switchOn()
		{ return; }
	virtual void switchOff()
		{ return; }
	virtual bool isSwitchOn()
		{ return true; }

	virtual void setMode(int newMode)
		{ return; }

	virtual bool isUnitHealthy()
		{ return true; }

	// Methods to set and get the input hw buffers - useful during testing when
	// no real hardware is available
	void setFcHwInpBuff(unsigned short d, int i);
	unsigned short getFcHwOutBuff(int i);

	// Health property management is not implemented in the fw prototype
	virtual Property getHealthProperty()
		{ return Property(); }
	virtual void addHealthMonitor(PropertyMonitor* monitor, ChangeObject* changeObject)
		{ return; };
	virtual void removeHealthMonitor(PropertyMonitor* monitor)
		{ return; }

	// pure virtual methods from interface Resettable
	virtual void reset();

	// pure virtual methods from interface Configurable
	virtual void resetConfiguration();
	virtual bool isConfigured();
	
	// pure virtual methods from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

	// Straightforward getter and setter methods
	AocsUnitHardware* getAocsUnitHardware()
		{ return hwUnit; }
	void setAocsUnitHardware(AocsUnitHardware* u)
		{ hwUnit=u; }

	void setHwFailureRecoveryAction(RecoveryAction* r)
		{ hwFailureRecAct = r; }
	RecoveryAction* getHwFailureRecoveryAction()
		{ return hwFailureRecAct; }
};

// -------------------------------------------------------------------
#endif