// TestCaseAocsUnit_2.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseAocsUnit_2.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../AocsEvent/FailureEventRepository.h"
#include "../../AocsEvent/ConfigurationEventRepository.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../SequentialDataProcessing/TwoByTwoMatrixBlock.h"
#include "../../AocsUnit/MacsTcController.h"
#include "../../AocsUnit/FssPrototype.h"
#include "../../FailureRecovery/NullRecoveryAction.h"

// -------------------------------------------------------------------
TestCaseAocsUnit_2::TestCaseAocsUnit_2():
	TestCaseGenericSetUp(ID_UNITTRIGGER*10+2,
									"TestCaseAocsUnit_2") {
		return;
}

// -------------------------------------------------------------------

void TestCaseAocsUnit_2::runTestCase() {

	// Retrieve the number of failure and configuration events
	int fen = AocsObject::getFailureEventRepository()->getCounter();
	int cen = AocsObject::getConfigurationEventRepository()->getCounter();

	// The following locations represent the MACS registers
	short macsCs=0;
	short macsInstr=0;
	short macsData=0;

	// Create and configure the MACS TC object
	MacsTcController macsTcCont(&macsCs, &macsInstr, &macsData);
	macsTcCont.setMaxPolling(2);

	// Create and configure a converter for the FSS
	TwoByTwoMatrixBlock conv;
	conv.setMatrixElement(0,0,1);
	conv.setMatrixElement(0,1,0);
	conv.setMatrixElement(1,0,0);
	conv.setMatrixElement(1,1,1);

	// Create a dummy recovery action
	NullRecoveryAction recAct;

	// Create the destination buffer for the FSS measurements and
	// their time tag
	Real fss_x, fss_y;
	AocsTime fss_x_tag, fss_y_tag;
	DataItemWrite fss_x_diw(&fss_x_tag, &fss_x);
	DataItemWrite fss_y_diw(&fss_y_tag, &fss_y);	

	// Create and configure the FSS object
	FssPrototype fss(10,11,12);
	fss.setAocsUnitHardware(&macsTcCont);
	fss.setFunctionalInConverter(&conv);
	fss.setFunctionalInLink(fss_x_diw,0);
	fss.setFunctionalInLink(fss_y_diw,1);
	fss.setHwFailureRecoveryAction(&recAct);

	// Perform some illegal configuration operation and check the 
	// corresponding configuration events are generated
	fss.setFunctionalOutConverter(&conv);
	fss.setHousekeepingInLink(fss_x_diw,0);
	fss.setFunctionalInLink(fss_x_diw,2);
	if (AocsObject::getConfigurationEventRepository()->getCounter() != cen+3)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 2 failed");
		return;
	}
	if (AocsObject::getConfigurationEventRepository()->latestConfigurationEvent()->
			getEventType() != ILLEGAL_FC_DEST_BUF)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 2 failed");
		return;
	}

	// Check that the FSS object is configured
	if (!fss.isConfigured())
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 2 failed");
		return;
	}

	// Initialize the FSS and check that the correct instruction is 
	// loaded in the MACS control register
	fss.initialize();
	if (macsInstr!=0x0156)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 2 failed");
		return;
	}

	// Since there is no real external unit, a bus error should have been reported
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+1)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 2 failed");
		return;
	}
	if (AocsObject::getFailureEventRepository()->latestFailureEvent()->
			getEventType() != NO_BUS_TRANSMISSION)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 2 failed");
		return;
	}

	// Acquire the FSS measurements. Since there is no real external unit, two
	// bus errors should be generated (one for each channel) and the MACS control
	// register should reflect the last instruction put on the bus
	fss.acquireFunctionalData();
	if (macsInstr!=-29610)		// -29610=0x8C56
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 2 failed");
		return;
	}
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+3)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 2 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}




