// TestCaseAocsUnit_7.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseAocsUnit_7.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../AocsEvent/FailureEventRepository.h"
#include "../../AocsEvent/ConfigurationEventRepository.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../SequentialDataProcessing/TwoByTwoMatrixBlock.h"
#include "../../SequentialDataProcessing/RawBusDataConverterIn.h"
#include "../../SequentialDataProcessing/RawBusDataConverterOut.h"
#include "../../SequentialDataProcessing/BiasScalingCompensator.h"
#include "../../SequentialDataProcessing/ControlChannelSuperBlock.h"
#include "../../SequentialDataProcessing/TorqueToThrusterPrototype.h"
#include "../../AocsUnit/MacsTcController.h"
#include "../../AocsUnit/SapPrototype.h"
#include "../../AocsUnit/TorquingThrusters.h"
#include "../../FailureRecovery/NullRecoveryAction.h"

// -------------------------------------------------------------------
TestCaseAocsUnit_7::TestCaseAocsUnit_7():
	TestCaseGenericSetUp(ID_UNITTRIGGER*10+7,
									"TestCaseAocsUnit_7") {
		return;
}

// -------------------------------------------------------------------

void TestCaseAocsUnit_7::runTestCase() {

	// Retrieve the number of failure and configuration events
	int fen = AocsObject::getFailureEventRepository()->getCounter();
	int cen = AocsObject::getConfigurationEventRepository()->getCounter();

	// The following locations represent the MACS registers
	short macsCs=0;
	short macsInstr=0;
	short macsData=0;

	// Create and configure the MACS TC object
	MacsTcController macsTcCont(&macsCs, &macsInstr, &macsData);
	macsTcCont.setMaxPolling(2);

	// Create the raw data converter for the SAS measurements
	RawBusDataConverterIn convIn(6,0.2);

	// Create the bias and scaling factor converter for the SAS rate measurement
	BiasScalingCompensator bsc1(1.0,2.0);
	BiasScalingCompensator bsc2(2.0,2.0);
	BiasScalingCompensator bsc3(3.0,2.0);
	BiasScalingCompensator bsc4(4.0,2.0);
	BiasScalingCompensator bsc5(5.0,2.0);
	BiasScalingCompensator bsc6(6.0,2.0);

	// Link the raw data converter and the bias/scaling compensator
	bsc1.linkInput(&convIn,0,0);
	bsc2.linkInput(&convIn,1,0);
	bsc3.linkInput(&convIn,2,0);
	bsc4.linkInput(&convIn,3,0);
	bsc5.linkInput(&convIn,4,0);
	bsc6.linkInput(&convIn,5,0);

	// Create the raw data converter for the THU commands.
	// The first six inputs are reserved to the delay-time commands and 
	// the last six inputs are reserved to the on-time commands.
	// It is assumed that there is no bias/scaling_factor compensation 
	// for the THU commands.
	RawBusDataConverterOut convOut(12,10);

	// Create and configure the converter superblock for the SAS outputs
	ControlChannelSuperBlock sasConv(6,6);
	for (int i=0; i<6; i++)
		sasConv.linkInputInternal(&convIn,i,i);
	sasConv.linkOutputInternal(&bsc1,0,0);
	sasConv.linkOutputInternal(&bsc2,0,1);
	sasConv.linkOutputInternal(&bsc3,0,2);
	sasConv.linkOutputInternal(&bsc4,0,3);
	sasConv.linkOutputInternal(&bsc5,0,4);
	sasConv.linkOutputInternal(&bsc6,0,5);

	// Create a dummy recovery action
	NullRecoveryAction recAct;

	// Create the destination buffers for SAS measurements and their time tag
	Real sas[6];
	AocsTime sasTTag[6];
	DataItemWrite sasDiw[6];
	for (i=0; i<6; i++)
		sasDiw[i].initialize(&sasTTag[i],&sas[i]);

	// Create the source buffers for the torques 
	Real tor[3];

	// Create the on-time and delay-time buffers (with their time tag buffers)
	Real onTime[6];
	AocsTime onTimeTT[6];
	Real delayTime[6];
	AocsTime delayTimeTT[6];

	// Create and configure the SAP object
	SapPrototype sap(10,1,1);
	sap.setAocsUnitHardware(&macsTcCont);
	sap.setFunctionalInConverter(&sasConv);
	sap.setFunctionalOutConverter(&convOut);
	for (i=0; i<6; i++)
	{	sap.setFunctionalInLink(sasDiw[i],i);
		sap.setFunctionalOutLink(DataItemWrite(&delayTimeTT[i],&delayTime[i]),i);
		sap.setFunctionalOutLink(DataItemWrite(&onTimeTT[i],&onTime[i]),i+6);
	}
	sap.setHwFailureRecoveryAction(&recAct);

	// Create the torque-to-thruster converter
	TorqueToThrusterPrototype torToThuConv(10,500);

	// Create and configure the torquingThruster fictitious unit
	TorquingThrusters torThu(6);
	AocsTime torThuTT[6];
	torThu.setFunctionalOutConverter(&torToThuConv);
	for (i=0; i<3; i++)
		torThu.setFunctionalOutLink(DataItemWrite(&torThuTT[6],&tor[i]),i);
	torThu.setLowLevelThrusterUnit(&sap);

	// Check that the SAP and torquingThrusters objects are configured
	if (!sap.isConfigured())
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 7 failed");
		return;
	}

	if (!torThu.isConfigured())
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 7 failed");
		return;
	}

	// Send a torque request to the thruster. 
	tor[0]=5;
	tor[1]=-1;
	tor[2]=11.0;		// this should saturate the thruster
	
	torThu.refreshFunctionalOut();

	for (i=0; i<6; i++)
	{	if (sap.getFcHwOutBuff(i)!=0)
		{	setTestResult(TEST_FAILURE, "Aocs Unit test 7 failed");
			return;
		}
	}

	if (sap.getFcHwOutBuff(6)!=(2500+32768))
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 7 failed");
		return;
	}
	if (sap.getFcHwOutBuff(7)!=0)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 7 failed");
		return;
	}
	if (sap.getFcHwOutBuff(8)!=0)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 7 failed");
		return;
	}
	if (sap.getFcHwOutBuff(9)!=32768+500)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 7 failed");
		return;
	}
	if (sap.getFcHwOutBuff(10)!=(5000+32768))
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 7 failed");
		return;
	}
	if (sap.getFcHwOutBuff(11)!=0)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 7 failed");
		return;
	}

	// Send the on-time and delay-time requests to the thrusters.
	// Since there is no real external unit, 12 bus errors should have been 
	// generated. The MACS control register should reflect  the last 
	// instruction put on the bus
	torThu.sendFunctionalData();
	if (macsInstr!=0x4C56)			
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 7 failed");
		return;
	}
	if (AocsObject::getFailureEventRepository()->getCounter() != fen+12)
	{	setTestResult(TEST_FAILURE, "Aocs Unit test 7 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
