// FullTriggerList.h
//
// Version	1.0
// Date		02.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Encapsulation of full trigger list objects
// -------------------------------------------------------------------

#ifndef FullTriggerListH
#define FullTriggerListH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../BasicObjects/AocsObject.h"
#include "../AocsUnit/TriggerList.h"
#include "../ObjectList/ObjectListTemplate.h"
#include "../AocsUnit/AocsUnitFunctional.h"
#include "../AocsUnit/AocsUnitHousekeeping.h"

// -------------------------------------------------------------------

class FullTriggerList : public AocsObject, public TriggerList {

	static int tmImageLength[NTMFORMATS];

	// List of units to which hk synch message must be sent
	ObjectListTemplate<AocsUnitHousekeeping> hkSynch;

	// List of units from which hk data must be acquired
	ObjectListTemplate<AocsUnitHousekeeping> hkAcquire;

	// List of units to which fc synch message must be sent
	ObjectListTemplate<AocsUnitFunctional> fcSynch;

	// List of units from which functional data must be acquired
	ObjectListTemplate<AocsUnitFunctional> fcAcquire;

	// List of units to which functional data must be send
	ObjectListTemplate<AocsUnitFunctional> fcSend;

	// List of units to be initialized/shutdown
	ObjectListTemplate<AocsUnitHousekeeping> hkInit;

	// Local reset methods
	void localReset();
	void localResetConfiguration();		

public:

	FullTriggerList(int hkSynchListSize,
						int hkAcquireListSize, 
							int fcSynchListSize,
								int fcAcquireListSize,
									int fcSendListSize,
										int hkInitSize);

	// Add/Remove items from the lists of units to be triggered
	void addHousekeepingSynchronize(AocsUnitHousekeeping* unitToBeSynchronized)
		{ hkSynch.addObject(unitToBeSynchronized); }

	void addHousekeepingAcquire(AocsUnitHousekeeping* unitToBeTriggered)
		{ hkAcquire.addObject(unitToBeTriggered); }
	
	void addFunctionalSynchronize(AocsUnitFunctional* unitToBeSynchronized)
		{ fcSynch.addObject(unitToBeSynchronized); }

	void addFunctionalAcquire(AocsUnitFunctional* unitToBeTriggered)
		{ fcAcquire.addObject(unitToBeTriggered); }

	void addFunctionalSend(AocsUnitFunctional* unitToBeTriggered)
		{ fcSend.addObject(unitToBeTriggered); }

	void removeHousekeepingSynchronize(AocsUnitHousekeeping* unitToBeSynchronized)
		{ hkSynch.removeObject(unitToBeSynchronized); }

	void removeHousekeepingAcquire(AocsUnitHousekeeping* unitToBeTriggered)
		{ hkAcquire.removeObject(unitToBeTriggered); }
	
	void removeFunctionalSynchronize(AocsUnitFunctional* unitToBeSynchronized)
		{ fcSynch.removeObject(unitToBeSynchronized); }

	void removeFunctionalAcquire(AocsUnitFunctional* unitToBeTriggered)
		{ fcAcquire.removeObject(unitToBeTriggered); }

	void removeFunctionalSend(AocsUnitFunctional* unitToBeTriggered)
		{ fcSend.removeObject(unitToBeTriggered); }

	// Iterate through units to which hk synchronization message must be sent
	AocsUnitHousekeeping* firstHousekeepingSynchronize()
		{ return hkSynch.first(); }
	AocsUnitHousekeeping* nextHousekeepingSynchronize()
		{ return hkSynch.next(); }
	bool isLastHousekeepingSynchronize()
		{ return hkSynch.isLast(); }

	// Iterate through units from which hk data must be acquired
	AocsUnitHousekeeping* firstHousekeepingAcquire()
		{ return hkAcquire.first(); }
	AocsUnitHousekeeping* nextHousekeepingAcquire()
		{ return hkAcquire.next(); }
	bool isLastHousekeepingAcquire()
		{ return hkAcquire.isLast(); }

	// Iterate through units to which fc synchronization messages must be sent
	AocsUnitFunctional* firstFunctionalSynchronize()
		{ return fcSynch.first(); }
	AocsUnitFunctional* nextFunctionalSynchronize()
		{ return fcSynch.next(); }
	bool isLastFunctionalSynchronize()
		{ return fcSynch.isLast(); }

	// Iterate through units from which fc data must be acquired
	AocsUnitFunctional* firstAcquireFunctional()
		{ return fcAcquire.first(); }
	AocsUnitFunctional* nextAcquireFunctional()
		{ return fcAcquire.next(); }
	bool isLastAcquireFunctional()
		{ return fcAcquire.isLast(); }

	// Iterate through units to which fc data must be sent
	AocsUnitFunctional* firstSendFunctional()
		{ return fcSend.first(); }
	AocsUnitFunctional* nextSendFunctional()
		{ return fcSend.next(); }
	bool isLastSendFunctional()
		{ return fcSend.isLast(); }

	// Add/Remove items from the lists of units to be initialized/shutdown
	void addInitShutdownUnit(AocsUnitHousekeeping* initShutdownUnit)
		{ hkInit.addObject(initShutdownUnit); }
	void removeInitShutdownUnit(AocsUnitHousekeeping* initShutdownUnit)
		{ hkInit.removeObject(initShutdownUnit); }

	// Initialize and shut-down operations for the units in the trigger list
	void initialize();
	void shutdown();

	// pure virtual methods from interface Resettable
	virtual void reset();

	// pure virtual methods from interface Configurable
	virtual void resetConfiguration();
	virtual bool isConfigured();
	
	// pure virtual methods from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

};

// -------------------------------------------------------------------
#endif