// FunctionalTriggerList.h
//
// Version	1.0
// Date		08.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Encapsulation of trigger list for AOCS functional units
// -------------------------------------------------------------------

#ifndef FunctionalTriggerListH
#define FunctionalTriggerListH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../BasicObjects/AocsObject.h"
#include "../AocsUnit/TriggerList.h"
#include "../ObjectList/ObjectListTemplate.h"
#include "../AocsUnit/AocsUnitFunctional.h"
#include "../AocsUnit/AocsUnitHousekeeping.h"

// -------------------------------------------------------------------

class FunctionalTriggerList : public AocsObject, public TriggerList {

	static int tmImageLength[NTMFORMATS];

	// List of units to which functional synch messages must be sent
	ObjectListTemplate<AocsUnitFunctional> fcSynch;

	// List of units from which functional data must be acquired
	ObjectListTemplate<AocsUnitFunctional> fcAcquire;

	// List of units to which functional data must be sent
	ObjectListTemplate<AocsUnitFunctional> fcSend;

	// List of units to be initialized/shutdown
	ObjectListTemplate<AocsUnitHousekeeping> hkInit;

	// Local reset methods
	void localReset();
	void localResetConfiguration();		

public:

	FunctionalTriggerList(int fcSynchListSize,
								int fcAcquireListSize,
									int fcSendListSize,
										int hkInitSize);
	
	// Methods related to housekeeping units inherited from interface 
	// 'TriggerList' have dummy implementations
	void addHousekeepingSynchronize(AocsUnitHousekeeping* unitToBeSynchronized)
		{ return; }
	void addHousekeepingAcquire(AocsUnitHousekeeping* unitToBeTriggered)
		{ return; }
	void removeHousekeepingSynchronize(AocsUnitHousekeeping* unitToBeSynchronized)
		{ return; }
	void removeHousekeepingAcquire(AocsUnitHousekeeping* unitToBeTriggered)
		{ return; }

	AocsUnitHousekeeping* firstHousekeepingSynchronize()
		{ return (AocsUnitHousekeeping*)0; }
	AocsUnitHousekeeping* nextHousekeepingSynchronize()
		{ return (AocsUnitHousekeeping*)0; }
	bool isLastHousekeepingSynchronize()
		{ return true; }

	AocsUnitHousekeeping* firstHousekeepingAcquire()
		{ return (AocsUnitHousekeeping*)0; }
	AocsUnitHousekeeping* nextHousekeepingAcquire()
		{ return (AocsUnitHousekeeping*)0; }
	bool isLastHousekeepingAcquire()
		{ return true; }

	// Add/Remove items from the lists of functional units to be triggered
	void addFunctionalSynchronize(AocsUnitFunctional* unitToBeSynchronized)
		{ fcSynch.addObject(unitToBeSynchronized); }

	void addFunctionalAcquire(AocsUnitFunctional* unitToBeTriggered)
		{ fcAcquire.addObject(unitToBeTriggered); }

	void addFunctionalSend(AocsUnitFunctional* unitToBeTriggered)
		{ fcSend.addObject(unitToBeTriggered); }

	void removeFunctionalSynchronize(AocsUnitFunctional* unitToBeSynchronized)
		{ fcSynch.removeObject(unitToBeSynchronized); }

	void removeFunctionalAcquire(AocsUnitFunctional* unitToBeTriggered)
		{ fcAcquire.removeObject(unitToBeTriggered); }

	void removeFunctionalSend(AocsUnitFunctional* unitToBeTriggered)
		{ fcSend.removeObject(unitToBeTriggered); }

	// Iterate through units to which fc synchronization messages must be sent
	AocsUnitFunctional* firstFunctionalSynchronize()
		{ return fcSynch.first(); }
	AocsUnitFunctional* nextFunctionalSynchronize()
		{ return fcSynch.next(); }
	bool isLastFunctionalSynchronize()
		{ return fcSynch.isLast(); }

	// Iterate through units from which fc data must be acquired
	AocsUnitFunctional* firstAcquireFunctional()
		{ return fcAcquire.first(); }
	AocsUnitFunctional* nextAcquireFunctional()
		{ return fcAcquire.next(); }
	bool isLastAcquireFunctional()
		{ return fcAcquire.isLast(); }

	// Iterate through units to which fc data must be sent
	AocsUnitFunctional* firstSendFunctional()
		{ return fcSend.first(); }
	AocsUnitFunctional* nextSendFunctional()
		{ return fcSend.next(); }
	bool isLastSendFunctional()
		{ return fcSend.isLast(); }

	// Add/Remove items from the lists of units to be initialized/shutdown
	void addInitShutdownUnit(AocsUnitHousekeeping* initShutdownUnit)
		{ hkInit.addObject(initShutdownUnit); }
	void removeInitShutdownUnit(AocsUnitHousekeeping* initShutdownUnit)
		{ hkInit.removeObject(initShutdownUnit); }

	// Initialize and shut-down operations for the units in the trigger list
	void initialize();
	void shutdown();

	// pure virtual methods from interface Resettable
	virtual void reset();

	// pure virtual methods from interface Configurable
	virtual void resetConfiguration();
	virtual bool isConfigured();
	
	// pure virtual methods from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

};

// -------------------------------------------------------------------
#endif