// TorquingThruster.h
//
// Version	1.0
// Date		05.05.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Class for the AOCS prototype encapsulating a fictitious AOCS unit
// representing a set of thrusters that are directly commanded with
// the torque requests around each spacecraft axis.
//
// The unit takes three commands representing the torque requests
// around the spacecraft X, Y and Z axes. The commands are transformed
// into on-time and delay-time requests for a set of n thrusters that
// are then passed on to the physical thrusters.
// The set of physical thrusters is seen through an 'AocsFunctional' 
// interface. 
// The following lay-out is assumed for the inputs to the thrusters:
// The inputs to the SAP unit are ordered as follows:
//	- input i for i=1..n: delay-time for i-th thruster
//	- input i for i=n+1..2n: on-time for i-th thruster
//
// -------------------------------------------------------------------

#ifndef TorquingThrustersH
#define TorquingThrustersH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../BasicObjects/AocsObject.h"
#include "../AocsUnit/AocsUnitFunctional.h"
#include "../AocsData/DataItemWrite.h"
#include "../AocsUnit/AocsUnitHardware.h"
#include "../SequentialDataProcessing/AbstractControlChannel.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------

class TorquingThrusters : public AocsObject, public AocsUnitFunctional {

	static int tmImageLength[NTMFORMATS];
	static ClassId classId;

	int nOfThrusters;

	AbstractControlChannel*	torqueToThuConv;

	AocsUnitFunctional* lowLevelThuUnit;

	DataItemWrite* onTime;
	DataItemWrite* delayTime;
	DataItemWrite torque[3];

	// Local reset methods
	void localResetConfiguration();
	void localReset();

public:

	TorquingThrusters(int nOfThrusters);

	// Methods inherited from AocsUnitFunctional with dummy implementation
	virtual void acquireFunctionalData()
		{ return; }
	virtual bool isAcquireTransactionFinished()
		{ return true; }
	virtual void refreshFunctionalIn()
		{ return; }

	// Methods delegated to low level thruster unit
	virtual void sendFunctionalData()
		{ lowLevelThuUnit->sendFunctionalData(); }
	virtual bool isSendTransactionFinished()
		{ return lowLevelThuUnit->isSendTransactionFinished(); }
	virtual void synchronizeFunctional()
		{ lowLevelThuUnit->synchronizeFunctional(); }
	virtual bool isSynchronizeFunctionalFinished()
		{ return lowLevelThuUnit->isSynchronizeFunctionalFinished(); }

	// Configuration methods not applicable to this unit
	virtual void setFunctionalInConverter(AbstractControlChannel* c)
		{ reportConfigurationError(this,NO_IN_CONVERTER,this); }
	virtual void setFunctionalInLink(DataItemWrite d, int i)
		{ reportConfigurationError(this,NO_IN_CONVERTER,this); }
	virtual void setFunctionalInLink(AocsData* d)
		{ reportConfigurationError(this,NO_IN_CONVERTER,this); }
	virtual DataItemWrite getFunctionalInLink(int i);

	// Fully implemented methods inherited from AocsUnitFunctional interface
	virtual void refreshFunctionalOut();

	virtual void setFunctionalOutConverter(AbstractControlChannel* c);

	virtual void setFunctionalOutLink(DataItemWrite d, int i);

	virtual void setFunctionalOutLink(AocsData* d);
	
	virtual DataItemWrite getFunctionalOutLink(int i);

	// Setter and getter methods for the low level thruster unit
	void setLowLevelThrusterUnit(AocsUnitFunctional* u);
	AocsUnitFunctional* getLowLevelThrusterUnit()
		{ return lowLevelThuUnit; }
	
	// Methods inherited from Resettable interface
	void reset();

	// Methods inherited from Configurable interface
	void resetConfiguration();
	bool isConfigured();

	// Methods inherited from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();
};

// -------------------------------------------------------------------
#endif