// Controller.cpp
//
// Version  1.3
// Date     21.11.00
// Author   T. Brown
//
// -------------------------------------------------------------------

#include "../ControllerManagement/Controller.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"

// -------------------------------------------------------------------
// Sending data to telemetry is not implemented in f/w prototype,
// so all Telemetry Image Lengths are 0.
int Controller::tmImageLength[NTMFORMATS] = {0, 0, 0, 0};

Real Controller::zero = 0.0;

// -------------------------------------------------------------------
Controller::Controller() : superBlock(3, 3)
{
    setClassId(ID_CONTROLLER);

    defaultMeasurementInput.initialize(&zero);
    defaultReferenceInput.initialize(&zero);
    defaultFeedForwardInput.initialize(&zero);

    // The default blocks must be configured even if they are not used
    // (else the system manager will find them unconfigured and raise
    // an error)
    DataItemRead zeroDataItemRead(&zero);
    defaultMeasurementFilter.linkInput(zeroDataItemRead,0);
    defaultReferenceInputFilter.linkInput(zeroDataItemRead,0);
    defaultFeedforwardFilter.linkInput(zeroDataItemRead,0);
    defaultCompensator.linkInput(zeroDataItemRead,0);
    actuatorLimitBlock.linkInput(zeroDataItemRead,0);
    defaultActuatorLimitBlock.linkInput(zeroDataItemRead,0);

    // perform localized reset functionality to initialize state
    localReset();
    localResetConfiguration();
}

// -------------------------------------------------------------------
void Controller::localResetConfiguration()
{
    // No inputs or outputs are linked
    referenceInputLinked = false;
    measurementInputLinked = false;
    feedforwardInputLinked = false;
    actuatorCommandLinked = false;
    compensatorOutputLinked = false;
    controlErrorLinked = false;

    // No client settable transfer function blocks have been set
    measurementFilter = NULL;
    feedforwardFilter = NULL;
    referenceInputFilter = NULL;
    compensator = NULL;

    // No actuator command limits have been set
    commandLimitsSet = false;

    // Instability recovery action is not set
    instabilityRecoveryAction = NULL;

    // Normal operational mode is closed loop
    feedbackLoopClosed = true;

    // Handle internal configuration tasks
    setupInternalConfiguration();

    return;
}

// -------------------------------------------------------------------
void Controller::localReset()
{
    if (superBlock.isConfigured())
        superBlock.reset();
    return;
}

// -------------------------------------------------------------------
void Controller::reset()
{
    AocsObject::reset();
    localReset();
}

// -------------------------------------------------------------------
void Controller::resetConfiguration()
{
    AocsObject::resetConfiguration();
    localResetConfiguration();
}

// -------------------------------------------------------------------
bool Controller::isConfigured()
{
    if (!AocsObject::isConfigured()) return false;

    // measurement input is mandatory
    if (!measurementInputLinked) return false;

    // actuator command output is mandatory
    if (!actuatorCommandLinked) return false;

    // compensator transfer function is mandatory
    if (compensator == NULL) return false;

    // recovery actions must be defined
    if (instabilityRecoveryAction == NULL) return false;

    // all internal control blocks must be configured
    if (!superBlock.isConfigured()) return false;

    return true;
}

// -------------------------------------------------------------------
int Controller::getTelemetryImageLength()
{
    return (tmImageLength[getTelemetryFormat()] + AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
void Controller::writeToTelemetry(TelemetryStream* tmStream)
{
    AocsObject::writeToTelemetry(tmStream); // write super object's state

    // TBD : not implemented in framework prototype

    return;
}

// -------------------------------------------------------------------
void Controller::setCompensator(AbstractControlChannel* c)
{
    compensator = c;
    setupInternalConfiguration();
}

// -------------------------------------------------------------------
AbstractControlChannel* Controller::getCompensator()
{
    return compensator;
}

// -------------------------------------------------------------------
void Controller::setMeasurementFilter(AbstractControlChannel* c)
{
    measurementFilter = c;
    setupInternalConfiguration();
}

// -------------------------------------------------------------------
AbstractControlChannel* Controller::getMeasurementFilter()
{
    return measurementFilter;
}

// -------------------------------------------------------------------
void Controller::setFeedforwardFilter(AbstractControlChannel* c)
{
    feedforwardFilter = c;
    setupInternalConfiguration();
}

// -------------------------------------------------------------------
AbstractControlChannel* Controller::getFeedforwardFilter()
{
    return feedforwardFilter;
}

// -------------------------------------------------------------------
void Controller::setReferenceSignalFilter(AbstractControlChannel* c)
{
    referenceInputFilter = c;
    setupInternalConfiguration();
}

// -------------------------------------------------------------------
AbstractControlChannel* Controller::getReferenceSignalFilter()
{
    return referenceInputFilter;
}

// -------------------------------------------------------------------
void Controller::linkReferenceSignal(DataItemRead d)
{
    referenceInput = d;
    referenceInputLinked = true;
    setupInternalConfiguration();
}

// -------------------------------------------------------------------
void Controller::linkSensorMeasurement(DataItemRead d)
{
    measurementInput = d;
    measurementInputLinked = true;
    setupInternalConfiguration();
}

// -------------------------------------------------------------------
void Controller::linkFeedforwardSignal(DataItemRead d)
{
    feedforwardInput = d;
    feedforwardInputLinked = true;
    setupInternalConfiguration();
}

// -------------------------------------------------------------------
void Controller::linkActuatorCommand(DataItemWrite d)
{
    actuatorCommand = d;
    actuatorCommandLinked = true;
}

// -------------------------------------------------------------------
void Controller::linkCompensatorOutput(DataItemWrite d)
{
    compensatorOutput = d;
    compensatorOutputLinked = true;
}

// -------------------------------------------------------------------
void Controller::linkControllerError(DataItemWrite d)
{
    controlError = d;
    controlErrorLinked = true;
}

// -------------------------------------------------------------------
void Controller::setCommandLimits(Real high, Real low)
{
    highCommandLimit = high;
    lowCommandLimit = low;
    commandLimitsSet = true;
    setupInternalConfiguration();
}

// -------------------------------------------------------------------
void Controller::unSetCommandLimits()
{
    commandLimitsSet = false;
    setupInternalConfiguration();
}

// -------------------------------------------------------------------
bool Controller::isStable()
{
    return true;
}

// -------------------------------------------------------------------
void Controller::setInstabilityRecoveryAction(RecoveryAction* r)
{
    instabilityRecoveryAction = r;
    return;
}

// -------------------------------------------------------------------
RecoveryAction* Controller::getInstabilityRecoveryAction()
{
    return instabilityRecoveryAction;
}

// -------------------------------------------------------------------
void Controller::setOpenLoop()
{
    feedbackLoopClosed = false;
    setupInternalConfiguration();
}

// -------------------------------------------------------------------
void Controller::setClosedLoop()
{
    feedbackLoopClosed = true;
    setupInternalConfiguration();
}

// -------------------------------------------------------------------
void Controller::hold()
{
    superBlock.hold();
}

// -------------------------------------------------------------------
void Controller::resume()
{
    superBlock.release();
}

// -------------------------------------------------------------------
void Controller::determineActualControlBlocks()
{
    // determine actual measurement filter to use
    actualMeasurementFilter =
        (measurementFilter != NULL) ? measurementFilter : &defaultMeasurementFilter;

    // determine actual reference signal filter to use
    actualReferenceInputFilter =
        (referenceInputFilter != NULL) ? referenceInputFilter : &defaultReferenceInputFilter;

    // determine actual feedforward filter to use
    actualFeedforwardFilter =
        (feedforwardFilter != NULL) ? feedforwardFilter : &defaultFeedforwardFilter;

    // determine actual compensation filter to use
    actualCompensator =
        (compensator != NULL) ? compensator : &defaultCompensator;

    // determine actual limit block to use
    if (commandLimitsSet)
    {
        actuatorLimitBlock.setLimits(highCommandLimit, lowCommandLimit);
        actualActuatorLimitBlock = &actuatorLimitBlock;
    }
    else
    {
        actualActuatorLimitBlock = &defaultActuatorLimitBlock;
    }

    return;
}

// -------------------------------------------------------------------
// Warning: This function should only be called after
//          determineActualControlBlocks() is called.
void Controller::makeInternalLinks()
{
    // attach 0 output of the actual measurement filter block
    // to the 0 input of the difference block (diffBlock) which will subtract
    // the measurement (output of the measurement filter) from the
    // reference signal (output of the reference signal filter)
    diffBlock.linkInput(actualMeasurementFilter, 0, 0);

    // attach 0 output of the actual reference signal filter block
    // to the 1 input of the difference block (diffBlock) which will subtract
    // the measurement (output of the measurement filter) from the
    // reference signal (output of the reference signal filter)
    diffBlock.linkInput(actualReferenceInputFilter, 0, 1);

    // attach 0 output of the actual feedforward filter block
    // to the 1 input of the block which adds the feedforward signal
    // (output of the feedforward filter) to the compensator output
    adderBlock.linkInput(actualFeedforwardFilter, 0, 1);

    // attach 0 output of the difference block (diffBlock) to
    // the 0 input of the compensator block
    actualCompensator->linkInput(&diffBlock, 0, 0);

    // attach 0 output of the compensator block to
    // the 0 input of the adderBlock
    adderBlock.linkInput(actualCompensator, 0, 0);

    // attach 0 output of adderBlock
    // to the 0 input of the limiting block
    actualActuatorLimitBlock->linkInput(&adderBlock, 0, 0);

    return;
}

// -------------------------------------------------------------------
// Warning: This function should only be called after
//          determineActualControlBlocks() is called.
void Controller::linkBlocksIntoSuperBlock()
{
    // 0 input of the superBlock goes to 0 input of the measurement filter
    superBlock.linkInputInternal(actualMeasurementFilter, 0, 0);

    // 1 input of the superBlock goes to 0 input of the reference filter
    superBlock.linkInputInternal(actualReferenceInputFilter, 0, 1);

    // 2 input of the superBlock goes to 0 input of the feedforward filter
    superBlock.linkInputInternal(actualFeedforwardFilter, 0, 2);

    // 0 output of the limiting block goes to the 0 output of the superBlock
    superBlock.linkOutputInternal(actualActuatorLimitBlock, 0, 0);

    // 0 output of the compensator goes to the 1 output of the superBlock
    superBlock.linkOutputInternal(actualCompensator, 0, 1);

    // 0 output of the difference block goes to the 2 output of the superBlock
    superBlock.linkOutputInternal(&diffBlock, 0, 2);

    return;
}

// -------------------------------------------------------------------
// Warning: This function should only be called after
//          determineActualControlBlocks() is called.
void Controller::embedBlocksIntoSuperBlock()
{
    actualMeasurementFilter->setEnclosingControlChannel(&superBlock);
    actualReferenceInputFilter->setEnclosingControlChannel(&superBlock);
    actualFeedforwardFilter->setEnclosingControlChannel(&superBlock);
    diffBlock.setEnclosingControlChannel(&superBlock);
    actualCompensator->setEnclosingControlChannel(&superBlock);
    adderBlock.setEnclosingControlChannel(&superBlock);
    actualActuatorLimitBlock->setEnclosingControlChannel(&superBlock);

    return;
}

// -------------------------------------------------------------------
void Controller::linkExternalInputsToSuperBlock()
{
    // Link measurement signal input
    if (measurementInputLinked && feedbackLoopClosed)
    {
        superBlock.linkInput(measurementInput, 0);
    }
    else
    {
        superBlock.linkInput(defaultMeasurementInput, 0);
    }

    // Link reference signal input
    if (referenceInputLinked)
    {
        superBlock.linkInput(referenceInput, 1);
    }
    else
    {
        superBlock.linkInput(defaultReferenceInput, 1);
    }

    // Link feedforward signal input
    if (feedforwardInputLinked)
    {
        superBlock.linkInput(feedforwardInput, 2);
    }
    else
    {
        superBlock.linkInput(defaultFeedForwardInput, 2);
    }

    return;
}

// -------------------------------------------------------------------
void Controller::setupInternalConfiguration()
{
    // determine actual control blocks to use
    determineActualControlBlocks();

    // make appropriate internal links
    makeInternalLinks();

    // link the blocks into the superblock
    linkBlocksIntoSuperBlock();

    // embed the blocks into the superblock
    embedBlocksIntoSuperBlock();

    // link external inputs of the superblock
    linkExternalInputsToSuperBlock();

    return;
}

// -------------------------------------------------------------------
void Controller::doControl()
{
    // propagate data through the super block
    superBlock.propagate(getTime());

    // get outputs from the superblock and write them
    // to the appropriate places

    // get acuator command output
    if (actuatorCommandLinked)
    {
        actuatorCommand.set(superBlock.getOutput(0).get());
    }

    // get Compensator output
    if (compensatorOutputLinked)
    {
        compensatorOutput.set(superBlock.getOutput(1).get());
    }

    // get control error output
    if (controlErrorLinked)
    {
        controlError.set(superBlock.getOutput(2).get());
    }

    return;
}
