// TestCaseController_2.cpp
//
// Version	1.0
// Date		06.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseController_2.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsEvent/AocsEvent.h"
#include "../../SequentialDataProcessing/P_Block.h"
#include "../../SequentialDataProcessing/AbstractControlChannel.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../ControllerManagement/Controller.h"

// -------------------------------------------------------------------
TestCaseController_2::TestCaseController_2():
	TestCaseGenericSetUp(ID_CONTROLLERMANAGER*10+2,
									"TestCaseController_2") {
		return;
}

// -------------------------------------------------------------------

void TestCaseController_2::runTestCase() {

    // Create a Controller
    Controller myController;

    // Create and load dummy instability recovery action
    NullRecoveryAction nra;
    myController.setInstabilityRecoveryAction(&nra);

    // Link measurement input
    Real sensorInputData;
    sensorInputData = 17.0;
    DataItemRead sensorInput(&sensorInputData);
    myController.linkSensorMeasurement(sensorInput);

    // Link the Reference signal input
    Real referenceSignalData;
    referenceSignalData = 20.0;
    DataItemRead referenceSignal(&referenceSignalData);
    myController.linkReferenceSignal(referenceSignal);

    // Link the Feedforward signal input
    Real feedforwardSignalData;
    feedforwardSignalData = 16.0;
    DataItemRead feedforwardSignal(&feedforwardSignalData);
    myController.linkFeedforwardSignal(feedforwardSignal);

    // Link actuator command output
    Real actuatorCommandOutputData;
    AocsTime actuatorOutputTime;
    DataItemWrite actuatorCommandOutput(&actuatorOutputTime, &actuatorCommandOutputData);
    myController.linkActuatorCommand(actuatorCommandOutput);

    // Link the compensator output
    Real compensatorOutputData;
    AocsTime compensatorOutputTime;
    DataItemWrite compensatorOutput(&compensatorOutputTime, &compensatorOutputData);
    myController.linkCompensatorOutput(compensatorOutput);

    // Link the control error output
    Real controlErrorOutputData;
    AocsTime controlErrorOutputTime;
    DataItemWrite controlErrorOutput(&controlErrorOutputTime, &controlErrorOutputData);
    myController.linkControllerError(controlErrorOutput);


    // Set Measurement Filter
    P_Block measurementFilter;
    measurementFilter.setGain(0.75);
    myController.setMeasurementFilter(&measurementFilter);

    // Set Reference Signal Filter
    P_Block referenceFilter;
    referenceFilter.setGain(0.85);
    myController.setReferenceSignalFilter(&referenceFilter);

    // Set Feedforward Filter
    P_Block feedforwardFilter;
    feedforwardFilter.setGain(0.1);
    myController.setFeedforwardFilter(&feedforwardFilter);

    // Set the compensator transfer function
    P_Block myProportionalTransferBlock;
    myProportionalTransferBlock.setGain(0.5);
    myController.setCompensator(&myProportionalTransferBlock);

    // Ask the controller to do its job
    aocsClock.run(0);
    myController.doControl();

    // check actuator command output
    if (actuatorCommandOutputData != 3.725)
	{	setTestResult(TEST_FAILURE, "Controller test 2 failed");
		return;
	}

    // check compensator output
    if (compensatorOutputData != 2.125)
	{	setTestResult(TEST_FAILURE, "Controller test 2 failed");
		return;
	}

    // check control error output
    if (controlErrorOutputData != 4.25)
	{	setTestResult(TEST_FAILURE, "Controller test 2 failed");
		return;
	}

    // Change input data slightly
    sensorInputData = 20.0;
    referenceSignalData = 20.0;
    feedforwardSignalData = 16.0;

    // Ask the controller to do its job
    aocsClock.run(0);
    myController.doControl();

    // check actuator command output
    if (actuatorCommandOutputData != 2.6)
	{	setTestResult(TEST_FAILURE, "Controller test 2 failed");
		return;
	}

    // check compensator output
    if (compensatorOutputData != 1.0)
	{	setTestResult(TEST_FAILURE, "Controller test 2 failed");
		return;
	}

    // check control error output
    if (controlErrorOutputData != 2.0)
	{	setTestResult(TEST_FAILURE, "Controller test 2 failed");
		return;
	}

    // Change input again data slightly
    sensorInputData = 20.0;
    referenceSignalData = 3000.0;
    feedforwardSignalData = 16.0;

    // Ask the controller to do its job
    aocsClock.run(0);
    myController.doControl();

    // check actuator command output
    if (actuatorCommandOutputData != 1269.1)
	{	setTestResult(TEST_FAILURE, "Controller test 2 failed");
		return;
	}

    // check compensator output
    if (compensatorOutputData != 1267.5)
	{	setTestResult(TEST_FAILURE, "Controller test 2 failed");
		return;
	}

    // check control error output
    if (controlErrorOutputData != 2535.0)
	{	setTestResult(TEST_FAILURE, "Controller test 2 failed");
		return;
	}

    // Add some limits
    myController.setCommandLimits(1001.0, 0.0);

    // Ask the controller to do its job
    aocsClock.run(0);
    myController.doControl();

    // check actuator command output
    if (actuatorCommandOutputData != 1001.0)
	{	setTestResult(TEST_FAILURE, "Controller test 2 failed");
		return;
	}

    // check compensator output
    if (compensatorOutputData != 1267.5)
	{	setTestResult(TEST_FAILURE, "Controller test 2 failed");
		return;
	}

    // check control error output
    if (controlErrorOutputData != 2535.0)
	{	setTestResult(TEST_FAILURE, "Controller test 2 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}


