// TestCaseController_5.cpp
//
// Version	1.0
// Date		06.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseController_5.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/DataItemRead.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../SequentialDataProcessing/PassThruBlock.h"
#include "../../SequentialDataProcessing/D_Block.h"
#include "../../SequentialDataProcessing/I_Block.h"
#include "../../SequentialDataProcessing/AbstractControlChannel.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../ControllerManagement/Controller.h"
#include "../../ControllerManagement/ControllerManager.h"
#include "../../ControllerManagement/FollowerControllerModeManager.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"
#include "../../ManoeuvreManagement/ManoeuvreManager.h"
#include "../../ManoeuvreManagement/AttitudeSlew.h"

// -------------------------------------------------------------------
TestCaseController_5::TestCaseController_5():
	TestCaseGenericSetUp(ID_CONTROLLERMANAGER*10+5,
									"TestCaseController_5") {
		return;
}

// -------------------------------------------------------------------

void TestCaseController_5::runTestCase() {

    // Create and configure a ManoeuvreManager
    ManoeuvreManager mnManager;

    // Initialize the ManoeuvreManager's manoeuvre list
    mnManager.initializeManoeuvreList(20);

    // Get access to the current clock
    DummyAocsClock* myClock = (DummyAocsClock*) AocsObject::getAocsClock();
    AocsTime currentTime = myClock->getTime();
    int clockPeriod = myClock->getClockPeriod();


    // create an AttitudeSlew manoeuvre which will start soon
    AocsTime slewStartTime = currentTime + clockPeriod;
    AocsTime slewEndTime = slewStartTime + 30 * clockPeriod;
    AocsTime slewOutputTime;
    Real slewValue = 100.0;
    DataItemWrite slewDataItem(&slewOutputTime, &slewValue);

    AttitudeSlew slew(slewStartTime, slewEndTime, 200.0, slewDataItem, 100.0);
    slew.setMonitorListSize(2);
    mnManager.addManoeuvre(&slew);


    // Create a simple controller which uses the value that
    // is being manipulated by the AttitudeSlew as reference input
    Controller controller;

    // create and set compensator (pure passthru)
    PassThruBlock compensator;
    controller.setCompensator(&compensator);

    // establish null recovery action
    NullRecoveryAction nra;
    controller.setInstabilityRecoveryAction(&nra);

    // make input reference signal come from the value that the
    // AttitudeSlew is setting
    DataItemRead referenceSignal(&slewValue);
    controller.linkReferenceSignal(referenceSignal);

    // create place for controller to send its actuator commands
    Real actuatorOutputData;
    AocsTime actuatorOutputTime;
    DataItemWrite actuatorOutput(&actuatorOutputTime, &actuatorOutputData);
    controller.linkActuatorCommand(actuatorOutput);

    // create a controller list
    ObjectListTemplate<Controller> cList;
    cList.allocateMemory(10);
    cList.addObject(&controller);

    // create and configure an AOCS mission mode manager
    NullModeChangeAction nmca;
    AocsMissionModeManager aocsMissionModeManager(1, 0, 5);
    aocsMissionModeManager.setModeChangeAction(&nmca);

    // create and configure a controller mode manager
    FollowerControllerModeManager fcmm(1, // int numberOfModes
                                       5, // int maxNumberOfMonitors
                                       &aocsMissionModeManager);  // ModeManager* masterModeManager
    fcmm.loadControllerList(0, &cList);
    fcmm.setModeChangeAction(&nmca);

    // create and configure a ControllerManager
    ControllerManager cManager;
    cManager.setModeManager(&fcmm);

    // move the clock forward and run the manoeuvre manager and
    // controller manager a "few" times

    for (int runCount=0; runCount<300; runCount++)
    {
        // alter the actuatorOutputData to some value that the
        // controller should never be setting it to
        actuatorOutputData = -999.9;

        // run the manoeuvre manager
        mnManager.run(0);

        // run the controller manager
        cManager.run(0);

        // as the AttitudeSlew changes the slewValue, the controller
        // should be using the slewValue as its reference value and
        // passing that value on to the actuatorOutputData,
        // so the actuatorOutputData should be equal to
        // the slewValue
        if (actuatorOutputData != slewValue) 
		{	setTestResult(TEST_FAILURE, "Controller test 5 failed");
			return;
		}

        // advance the time and keep on going
        myClock->run(0);
    }


    // verify that the slew has changed the slewValue to
    // its end value (200.0)
    if (slewValue != 200.0) 
	{	setTestResult(TEST_FAILURE, "Controller test 5 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}

