// TestCaseFailureDetection_1.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseFailureDetection_1.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../BasicObjects/AocsObject.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../FailureDetection/FailureDetectionManager.h"
#include "../../FailureDetection/FollowerFailureDetectionModeManager.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"

// -------------------------------------------------------------------
TestCaseFailureDetection_1::TestCaseFailureDetection_1():
	TestCaseGenericSetUp(ID_FAILUREDETECTIONMANAGER*10+1,
									"TestCaseFailureDetection_1") {
		return;
}

// -------------------------------------------------------------------

void TestCaseFailureDetection_1::runTestCase() {

	// Create a Scalar and a twoEulerAngles object to be used as 
	// sample consistencyCheckable objects
	TwoEulerAngles euler;
	Scalar scalar;

	// Create and allocate memory for the list of consistency 
	// checkable objects
	ObjectListTemplate<ConsistencyCheckable> ccList;
	ccList.allocateMemory(10);

	// Load the consistency checkable list as follows:
	// item 1 : TwoEulerAngles object
	// item 2 : mode event repository
	// item 3 : Scalar object
	ccList.addObject(&euler);
	ccList.addObject(ModeManager::getModeEventRepository());
	ccList.addObject(&scalar);

	// Create and allocate memory for the list of monitoring 
	// check objects (kept empty for this test)
	ObjectListTemplate<MonitoringCheck> mcList;
	mcList.allocateMemory(10);

	// Create and configure the AOCS mission mode manager
	AocsMissionModeManager aocsMissionModeManager(2,0,5);
	NullModeChangeAction mca;
	aocsMissionModeManager.setModeChangeAction(&mca);

	// Create the failure detection mode manager
	FollowerFailureDetectionModeManager fdmm(2,5,&aocsMissionModeManager);

	// Load the consistency checkable and monitoring check lists
	// in the failure detection mode manager
	fdmm.loadConsistencyCheckableList(0,&ccList);
	fdmm.loadConsistencyCheckableList(1,&ccList);
	fdmm.loadMonitoringCheckList(0,&mcList);
	fdmm.loadMonitoringCheckList(1,&mcList);

	// Load the mode change actions in the FD mode manager
	fdmm.setModeChangeAction(&mca);

	// Create the failure detection manager
	FailureDetectionManager fdm;

	// Plug the mode manager in the failure detection manager
	fdm.setFailureDetectionModeManager(&fdmm);
	fdm.setChangeEventRepository(ModeManager::getChangeEventRepository());

	// Retrieve the number of failure events
	failureEvtN = AocsObject::getFailureEventRepository()->getCounter();

	// Run the failure detection manager
	fdm.run(0);

	// Check that no failure events were generated (all objects 
	// loaded in the consistency checkable list were consistent)
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN)
	{	setTestResult(TEST_FAILURE, "Failure event was generated");
		return;
	}

	// Make the mode event repository inconsistent by forcing
	// the creation of a number of events greater than the
	// repository size.
	aocsMissionModeManager.setMode(1);
	aocsMissionModeManager.setMode(0);
	aocsMissionModeManager.setMode(1);
	aocsMissionModeManager.setMode(0);

	// Run the failure detection manager
	fdm.run(0);

	// Check that one failure event was generated (the mode event
	// repository should have been found to be inconsistent)
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN+1)
	{	setTestResult(TEST_FAILURE, "Wrong number of failure events");
		return;
	}

	// Check that the correct failure event was created
	if (AocsObject::getFailureEventRepository()->latestFailureEvent()->getEventType()!=
			CONSISTENCY_CHECK_FAILURE)
	{	setTestResult(TEST_FAILURE, "Wrong failure event");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
