// FailureDetectionManager.cpp
//
// Version	1.0
// Date		07.04.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../FailureDetection/FailureDetectionManager.h"
#include "../FailureDetection/FailureDetectionModeManager.h"
#include "../FailureDetection/ConsistencyCheckable.h"
#include "../FailureDetection/MonitoringCheck.h"
#include "../ModeManagement/ModeManager.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../AocsEvent/EventTypeId.h"
#include "../AocsEvent/ChangeEventRepository.h"
#include "../ObjectList/ObjectListTemplate.h"
#include "../ObjectMonitoring/ChangeObject.h"

// -------------------------------------------------------------------
// Define static variables
int FailureDetectionManager::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

FailureDetectionManager::FailureDetectionManager()
{
	setClassId(ID_FAILUREDETECTIONMANAGER);		

	tmImageLength[shortTm]  = 0;			// not implemented in f/w prototype
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	localResetConfiguration();
}

// -------------------------------------------------------------------

void FailureDetectionManager::localResetConfiguration()
{
	modeManager = NULL;
	changeEventRepository = NULL;
}

// -------------------------------------------------------------------

void FailureDetectionManager::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool FailureDetectionManager::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (modeManager==NULL)
		return false;

	if (changeEventRepository==NULL)
		return false;
	
	return true;
}

// -------------------------------------------------------------------

void FailureDetectionManager::run(AocsTime deadline)
{
	// Load the list of objects on which to perform a consistency check
	consistencyCheckableList = modeManager->getConsistencyCheckableList();

	// Load the list of monitoring check objects 
	monitoringCheckList = modeManager->getMonitoringCheckList();

	// Perform the consistency checks 
	ConsistencyCheckable* t;
	for ( t=consistencyCheckableList->first(); 
			 !consistencyCheckableList->isLast(); 
				t=consistencyCheckableList->next() )

		if ( !t->doConsistencyCheck() )
			reportFailure(this,CONSISTENCY_CHECK_FAILURE,(AocsObject*)t,t->getRecoveryAction());

	// Perform the monitoring checks 
	MonitoringCheck* m;
	for ( m=monitoringCheckList->first(); 
			 !monitoringCheckList->isLast(); 
				m=monitoringCheckList->next() )
	
		if ( m->getChangeObject()->checkValue(m->getPropertyValue()) )
		{	
			// record the change in the monitored property
			changeEventRepository->create(this,
										  MONITORED_PROPERTY_CHANGE,
										  m->getProperty().getPropertyId(),
										  m->getChangeObject(),
										  m->getPropertyValue());

			// record the detection of the failure 
			reportFailure(this,MONITORING_CHECK_FAILURE,m,m->getRecoveryAction());
		}

}

// -------------------------------------------------------------------
// From Telemeterable interface

int FailureDetectionManager::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void FailureDetectionManager::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// TBD : not implemented in framework prototype
}
