// TestCaseFailureRecovery_3.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseFailureRecovery_3.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/RecoveryEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../BasicObjects/AocsObject.h"
#include "../../ObjectList/ObjectListTemplate.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"
#include "../../FailureRecovery/ObjectReset.h"
#include "../../FailureRecovery/ModeChange.h"
#include "../../FailureRecovery/SystemReset.h"
#include "../../FailureRecovery/SystemResetOnTooManyFailures.h"
#include "../../FailureRecovery/LocalRecoveryActions.h"
#include "../../FailureRecovery/FollowerFailureRecoveryModeManager.h"
#include "../../FailureRecovery/FailureRecoveryManager.h"

// -------------------------------------------------------------------
TestCaseFailureRecovery_3::TestCaseFailureRecovery_3():
	TestCaseGenericSetUp(ID_FAILURERECOVERYMANAGER*10+3,
									"TestCaseFailureRecovery_3") {
		return;
}

// -------------------------------------------------------------------
// Check the failure recovery manager (FRM).
// A FRM with two modes is considered. The recovery strategy associated
// the first mode consists of the following two linked recovery strategies:
//	- srRecStr	(system reset on too many failures)
//	- lraRecStr	(local recovery actions)
// The recovery strategy associated to the second mode consists only of
// srRecStr.

void TestCaseFailureRecovery_3::runTestCase() {

	// Create a scalar to serve as dummy AocsObject and as dummy object 
	// to be reset (the scalar is made its own data owner)
	Scalar scalar;
	scalar.getDataItemWrite(0).set(2);

	// Create and configure a SystemResetOnTooManyFailures recovery strategy
	SystemResetOnTooManyFailures srRecStr;
	srRecStr.setFailureEventThreshold(3);

	// Create a LocalRecoveryAction recovery strategy
	LocalRecoveryActions lraRecStr;

	// Link the two recovery strategies
	lraRecStr.setNextRecoveryStrategy(&srRecStr);

	// create and configure and AocsMissionManager 
	AocsMissionModeManager ammm(2,0,5);
	NullModeChangeAction mca;
	ammm.setModeChangeAction(&mca);

	// Create a follower failure recovery mode manager (the AOCS mission
	// mode manager acts as master)
	FollowerFailureRecoveryModeManager ffrmm(2,4,&ammm);

	// Configure the mode manager
	ffrmm.loadRecoveryStrategy(0,&lraRecStr);
	ffrmm.loadRecoveryStrategy(1,&srRecStr);

	// Load the mode change actions in the FD mode manager
	ffrmm.setModeChangeAction(&mca);

	// Create and configure the failure recovery manager
	FailureRecoveryManager frm;
	frm.setFailureRecoveryModeManager(&ffrmm);

	// Create a null recovery action
	NullRecoveryAction nRecAct;

	// Create a recovery action to reset the scalar
	ObjectReset orRecAct(&scalar);

	// Recover the number of recovery events
	int recEvtN=RecoveryStrategy::getRecoveryEventRepository()->getCounter();

	// Run the failure recovery manager
	frm.run(0);

	// Check that no recovery events were created 
	if (RecoveryStrategy::getRecoveryEventRepository()->getCounter()!=recEvtN)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 3 failed");
		return;
	}

	// Create two failure events of which one is associated to a recovery
	// action to reset the scalar
	FailureEventRepository* fer=AocsObject::getFailureEventRepository();
	fer->create(&scalar,DUMMY_EVENT,&scalar,&orRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);

	// Run the failure recovery manager
	frm.run(0);

	// Check that only the lraRecStr was executed
	if (RecoveryStrategy::getRecoveryEventRepository()->getCounter()!=recEvtN+2)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 3 failed");
		return;
	}
	if (RecoveryStrategy::getRecoveryEventRepository()->latestRecoveryEvent()->
			getEventType()!=LOCAL_RECOVERY_ACTION)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 3 failed");
		return;
	}

	// Disable the lraRecStr and create four failure events
	lraRecStr.disable();
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);

	// Run the failure recovery manager	and check that only the srRecStr was executed
	frm.run(0);
	if (RecoveryStrategy::getRecoveryEventRepository()->getCounter()!=recEvtN+3)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 3 failed");
		return;
	}
	if (RecoveryStrategy::getRecoveryEventRepository()->latestRecoveryEvent()->
			getEventType()!=SYSTEM_RESET_RECOVERY)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 3 failed");
		return;
	}

	// Enable the lraRecStr
	lraRecStr.enable();

	// Switch operational mode
	ammm.setMode(1);

	// Create four failure events and run the failure recovery managers
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	frm.run(0);

	// Check that no recovery events were created 
	if (RecoveryStrategy::getRecoveryEventRepository()->getCounter()!=recEvtN+3)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 3 failed");
		return;
	}

	// Create four failure events and run the failure recovery managers
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);
	fer->create(&scalar,DUMMY_EVENT,&scalar,&nRecAct);

	// Run the failure recovery manager	and check that only the srRecStr was executed
	frm.run(0);
	if (RecoveryStrategy::getRecoveryEventRepository()->getCounter()!=recEvtN+4)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 3 failed");
		return;
	}
	if (RecoveryStrategy::getRecoveryEventRepository()->latestRecoveryEvent()->
			getEventType()!=SYSTEM_RESET_RECOVERY)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 3 failed");
		return;
	}
	
	// Unload the system manager from AocsObject
	AocsObject::setSystemManager(NULL);

	// Check that no hidden failure events were generated
	if (AocsObject::getFailureEventRepository()->getCounter() != failureEvtN+14)
	{	setTestResult(TEST_FAILURE, "Failure recovery test 3 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
