// FlushTelemetryBuffers.cpp 
//
// Version	1.0
// Date		31.07.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Sends the content of the TM buffers to the output device.
// The TM stream is a sequence of images from Telemeterable objects. 
// The class identifier is used to identify the type of the 
// telemeterable objects and to select the appropriate flush routine
// to process the image.
// -------------------------------------------------------------------

#include "../GeneralUtilities/FlushTelemetryBuffers.h"
#include "../GeneralUtilities/TestUtilities.h"
#include "../GeneralInclude/ClassId.h"

// -------------------------------------------------------------------
// This version only works for the case of TM images made up of bytes.

void flushTelemetryBuffers(FILE* out, AocsTestHarness* aocsTestHarness)
{
	printString(out,"AttitudeDataPool Layout: \n");
	printString(out,"  FssX FssY Gyr Sas1x Sas1y Sas2x Sas2y Sas3x Sas3y Rw1 Rw2 Rw3 Rw4\n");
	printString(out,"  TorX TorY TorZ RwAngMomX RwAngMomY RwAngMomZ RwTor1 RwTor2 \n");
	printString(out,"  RwTor3 RwTor4 ThuOn1 ... ThuOn6 ThuDel1 .. ThuDel6 AttEst1 AttEst2 AttEst3 \n");
	printString(out,"  SetPt1 SetPt2 SetPt3 AttErr1 AttErr2 AttErr3 \n\n");
	 
	DmaTelemetryStreamOnGround dmaStreamReader;

	char* tmBuffer = aocsTestHarness->getTmBuffer();

	// Iterate on the TM buffers
	for (int i=0; i<aocsTestHarness->getNumberOfTmBuffers(); i++)
	{	
		// Recover the TM frame number (written as an "unsigned short" in 
		// the first two bytes of the TM buffer
		unsigned short* pTmFrameNumber;
		pTmFrameNumber=(unsigned short*)&tmBuffer[i*(aocsTestHarness->getTmBufferSize())];
		unsigned short tmFrameNumber=(*pTmFrameNumber);

		// Write the TM frame number
		printString(out,"Telemetry Frame: ");
		printInt(out,(int)tmFrameNumber);
		printString(out,"\n");

		// Recover the number of bytes in the TM buffer (they are written
		// as an "unsigned short" in the second and third bytes of the buffer
		unsigned short* pTmBufferLength;
		pTmBufferLength=(unsigned short*)&tmBuffer[i*(aocsTestHarness->getTmBufferSize())+2];
		unsigned short tmBufferLength=(*pTmBufferLength);

		// Recover the start address of the TM buffer
		char* tmBufferStart=&tmBuffer[i*(aocsTestHarness->getTmBufferSize())];

		// Reset the DMA buffer reader
		dmaStreamReader.reset(tmBufferStart, tmBufferLength);

		// Define buffer variables
		TelemetryFormat tmFormat;
		int nOfEvents;

		// Process a single TM buffer
		while (!dmaStreamReader.isFinished(tmBufferLength))
		{	
			int classId = dmaStreamReader.readUnsignedChar();
			
			switch (classId) {

			case ID_FAILUREEVENTREPOSITORY:
				printString(out,"FailureEvtRep (eventId) ");
				flushFailureEventRepository(out,&dmaStreamReader,&tmFormat,&nOfEvents);
				printString(out,"\n");
				break;
			
			case ID_CONFIGURATIONEVENTREPOSITORY:
				printString(out,"ConfigurationEvtRep (eventId) ");
				flushConfigurationEventRepository(out,&dmaStreamReader,&tmFormat,&nOfEvents);
				printString(out,"\n");
				break;

			case ID_MODEEVENTREPOSITORY:
				printString(out,"ModeEvtRep (eventId, oldMode, newMode, modeMngId) ");
				flushModeEventRepository(out,&dmaStreamReader,&tmFormat,&nOfEvents);
				printString(out,"\n");
				break;

			case ID_SYSTEMEVENTREPOSITORY:
				printString(out,"SystemEvtRep (eventId) ");
				flushSystemEventRepository(out,&dmaStreamReader,&tmFormat,&nOfEvents);
				printString(out,"\n");
				break;

			case ID_RECONFIGURATIONEVENTREPOSITORY:
				printString(out,"ReconfigurationEvtRep (eventId, oldCnf newCnf rcnManId) ");
				flushReconfigurationEventRepository(out,&dmaStreamReader,&tmFormat,&nOfEvents);
				printString(out,"\n");
				break;

			case ID_TELECOMMANDEVENTREPOSITORY:
				printString(out,"TelecommandEvtRep (eventId, TcId) ");
				flushTelecommandEventRepository(out,&dmaStreamReader,&tmFormat,&nOfEvents);
				printString(out,"\n");
				break;

			case ID_RECOVERYEVENTREPOSITORY:
				printString(out,"RecoveryEvtRep (eventId) ");
				flushRecoveryEventRepository(out,&dmaStreamReader,&tmFormat,&nOfEvents);
				printString(out,"\n");
				break;

			case ID_MANOEUVREEVENTREPOSITORY:
				printString(out,"ManoeuvreEvtRep (eventId) ");
				flushManoeuvreEventRepository(out,&dmaStreamReader,&tmFormat,&nOfEvents);
				printString(out,"\n");
				break;

			case ID_ATTITUDEDATAPOOL:
				printString(out,"AttitudeDataPool ");
				flushAttitudeDataPool(out,&dmaStreamReader,&tmFormat);
				printString(out,"\n");
				break;

			case ID_AOCSDATA:
				printString(out,"AocsData ");
				flushAocsData(out,&dmaStreamReader,&tmFormat);
				printString(out,"\n");
				break;

			default:
				printMessage(out,"ERROR - Unexpected Class Identifier ");
			}
		}				

		printMessage(out,"____________________________________________________________________");
	}
}

// -------------------------------------------------------------------

void flushAocsObject(FILE* out, 
						DmaTelemetryStreamOnGround* stream, 
							TelemetryFormat* tmFormat)
{
	unsigned short instanceIdFormat= stream->readUnsignedShort();
	int t = instanceIdFormat & (unsigned short)0x0003;
	switch(t) {
		case 0:
			*tmFormat = shortTm;
			break;
		case 1:
			*tmFormat = normalTm;
			break;
		case 2:
			*tmFormat = longTm;
			break;
		case 3:
			*tmFormat = debugTm;
			break;
	}

	int instanceId = instanceIdFormat>>2;

	printInt(out,instanceId);
	printInt(out,(int)(*tmFormat));
}

// -------------------------------------------------------------------

void flushAocsEvent(FILE* out, 
						DmaTelemetryStreamOnGround* stream, 
							TelemetryFormat* tmFormat)
{
	if ((*tmFormat) == longTm)
	{
		int timeStamp = stream->readUnsignedInt();
		int creatorId = stream->readUnsignedShort();
	
		printInt(out,timeStamp);
		printInt(out,creatorId);
	}
	
	int eventId= stream->readUnsignedChar();

	printInt(out,eventId);
}

// -------------------------------------------------------------------

void flushEventRepository(FILE* out, 
							DmaTelemetryStreamOnGround* stream, 
								TelemetryFormat* tmFormat,
									int* numberOfEvents)
{
	flushAocsObject(out, stream, tmFormat); 
	
	if ((*tmFormat) == longTm)
	{	int recoveryActionInstanceId = stream->readUnsignedShort();
		printInt(out,recoveryActionInstanceId);
	}

	int counter = stream->readUnsignedShort();

	printInt(out,counter);

	int n = stream->readUnsignedChar();	// repository size

	// The number of events in the repository image is either the 
	// event counter (if the repository is not yet full) or the 
	// repository size (if the repository is not yet full)
	if (counter<n)
		*numberOfEvents = counter;
	else
		*numberOfEvents = n;
}

// -------------------------------------------------------------------

void flushFailureEvent(FILE* out, 
							DmaTelemetryStreamOnGround* stream, 
								TelemetryFormat* tmFormat)
{
	flushAocsEvent(out,stream,tmFormat);
	
	if ((*tmFormat) == longTm)
	{
		int recoveryActionId = stream->readUnsignedShort();
		int locationId = stream->readUnsignedShort();
	
		printInt(out,recoveryActionId);
		printInt(out,locationId);
	}

}

// -------------------------------------------------------------------

void flushFailureEventRepository(FILE* out, 
									DmaTelemetryStreamOnGround* stream, 
										TelemetryFormat* tmFormat,
											int* numberOfEvents)
{
	flushEventRepository(out, stream, tmFormat, numberOfEvents); 
	
	for (int i=0; i<(*numberOfEvents); i++)
		flushFailureEvent(out,stream,tmFormat);
}

// -------------------------------------------------------------------

void flushConfigurationEvent(FILE* out, 
								DmaTelemetryStreamOnGround* stream, 
									TelemetryFormat* tmFormat)
{
	flushAocsEvent(out,stream,tmFormat);
	
	if ((*tmFormat) == longTm)
	{
		int locationId = stream->readUnsignedShort();
		printInt(out,locationId);
	}

}

// -------------------------------------------------------------------

void flushConfigurationEventRepository(FILE* out, 
										DmaTelemetryStreamOnGround* stream, 
											TelemetryFormat* tmFormat,
												int* numberOfEvents)
{
	flushEventRepository(out, stream, tmFormat, numberOfEvents); 
	
	for (int i=0; i<(*numberOfEvents); i++)
		flushConfigurationEvent(out,stream,tmFormat);
}

// -------------------------------------------------------------------

void flushModeEvent(FILE* out, 
						DmaTelemetryStreamOnGround* stream, 
							TelemetryFormat* tmFormat)
{
	flushAocsEvent(out,stream,tmFormat);

	unsigned char mode = stream->readUnsignedChar();

	int newMode = mode & 0x0F;
	int oldMode = (mode & 0xF0) >> 4;

	printInt(out,oldMode);
	printInt(out,newMode);

	int modeManagerId = stream->readUnsignedShort();
	printInt(out,modeManagerId);
}

// -------------------------------------------------------------------

void flushModeEventRepository(FILE* out, 
										DmaTelemetryStreamOnGround* stream, 
											TelemetryFormat* tmFormat,
												int* numberOfEvents)
{
	flushEventRepository(out, stream, tmFormat, numberOfEvents); 
	
	for (int i=0; i<(*numberOfEvents); i++)
		flushModeEvent(out,stream,tmFormat);
}

// -------------------------------------------------------------------

void flushReconfigurationEvent(FILE* out, 
								DmaTelemetryStreamOnGround* stream, 
								 TelemetryFormat* tmFormat)
{
	flushAocsEvent(out,stream,tmFormat);

	unsigned char config = stream->readUnsignedChar();

	int newConfig = config & 0x0F;
	int oldConfig = (config & 0xF0) >> 4;

	printInt(out,oldConfig);
	printInt(out,newConfig);

	int reconfigManagerId = stream->readUnsignedShort();
	printInt(out,reconfigManagerId);
}

// -------------------------------------------------------------------

void flushReconfigurationEventRepository(FILE* out, 
											DmaTelemetryStreamOnGround* stream, 
												TelemetryFormat* tmFormat,
													int* numberOfEvents)
{
	flushEventRepository(out, stream, tmFormat, numberOfEvents); 
	
	for (int i=0; i<(*numberOfEvents); i++)
		flushReconfigurationEvent(out,stream,tmFormat);
}

// -------------------------------------------------------------------

void flushTelecommandEvent(FILE* out, 
								DmaTelemetryStreamOnGround* stream, 
								 TelemetryFormat* tmFormat)
{
	flushAocsEvent(out,stream,tmFormat);

	int telecommandId = stream->readUnsignedShort();
	printInt(out,telecommandId);
}

// -------------------------------------------------------------------

void flushTelecommandEventRepository(FILE* out, 
											DmaTelemetryStreamOnGround* stream, 
												TelemetryFormat* tmFormat,
													int* numberOfEvents)
{
	flushEventRepository(out, stream, tmFormat, numberOfEvents); 
	
	for (int i=0; i<(*numberOfEvents); i++)
		flushTelecommandEvent(out,stream,tmFormat);
}

// -------------------------------------------------------------------

void flushRecoveryEvent(FILE* out, 
								DmaTelemetryStreamOnGround* stream, 
								 TelemetryFormat* tmFormat)
{
	flushAocsEvent(out,stream,tmFormat);

	if (*tmFormat == longTm)
	{	int recoveryActionId = stream->readUnsignedShort();
		printInt(out,recoveryActionId);

		int recoveryStrategyId = stream->readUnsignedShort();
		printInt(out,recoveryStrategyId);
	}
}

// -------------------------------------------------------------------

void flushRecoveryEventRepository(FILE* out, 
											DmaTelemetryStreamOnGround* stream, 
												TelemetryFormat* tmFormat,
													int* numberOfEvents)
{
	flushEventRepository(out, stream, tmFormat, numberOfEvents); 
	
	for (int i=0; i<(*numberOfEvents); i++)
		flushRecoveryEvent(out,stream,tmFormat);
}

// -------------------------------------------------------------------

void flushManoeuvreEvent(FILE* out, 
								DmaTelemetryStreamOnGround* stream, 
								 TelemetryFormat* tmFormat)
{
	flushAocsEvent(out,stream,tmFormat);

	if (*tmFormat == longTm)
	{	int manoeuvreId = stream->readUnsignedShort();
		printInt(out,manoeuvreId);
	}
}

// -------------------------------------------------------------------

void flushManoeuvreEventRepository(FILE* out, 
											DmaTelemetryStreamOnGround* stream, 
												TelemetryFormat* tmFormat,
													int* numberOfEvents)
{
	flushEventRepository(out, stream, tmFormat, numberOfEvents); 
	
	for (int i=0; i<(*numberOfEvents); i++)
		flushManoeuvreEvent(out,stream,tmFormat);
}

// -------------------------------------------------------------------

void flushSystemEvent(FILE* out, 
						DmaTelemetryStreamOnGround* stream, 
							TelemetryFormat* tmFormat)
{
	flushAocsEvent(out,stream,tmFormat);
}

// -------------------------------------------------------------------

void flushSystemEventRepository(FILE* out, 
										DmaTelemetryStreamOnGround* stream, 
											TelemetryFormat* tmFormat,
												int* numberOfEvents)
{
	flushEventRepository(out, stream, tmFormat, numberOfEvents); 
	
	for (int i=0; i<(*numberOfEvents); i++)
		flushSystemEvent(out,stream,tmFormat);
}

// -------------------------------------------------------------------

void flushAttitudeDataPool(FILE* out, 
							DmaTelemetryStreamOnGround* stream, 
								TelemetryFormat* tmFormat)
{
	flushDataPool(out, stream, tmFormat); 
}

// -------------------------------------------------------------------

void flushDataPool(FILE* out, 
					DmaTelemetryStreamOnGround* stream, 
		 			 TelemetryFormat* tmFormat)
{
	flushAocsObject(out, stream, tmFormat);
	
	// Read number of data items/AocsData in the image
	int n = stream->readUnsignedChar();

	if ((*tmFormat) == longTm)
		for (int i=0; i<n; i++)
			flushAocsData(out, stream, tmFormat);
	
	if ((*tmFormat) == normalTm)
		for (int i=0; i<n; i++)
		{	float f = stream->readFloat();
			printDouble(out,(double)f);
		}
}

// -------------------------------------------------------------------

void flushAocsData(FILE* out, 
					DmaTelemetryStreamOnGround* stream, 
		 			 TelemetryFormat* tmFormat)
{
	flushAocsObject(out, stream, tmFormat);
	
	// Read number of data items in the image
	int n = stream->readUnsignedChar();

	if ((*tmFormat) == longTm)
		for (int i=0; i<n; i++)
		{	unsigned int timeTag = stream->readUnsignedInt();
			float f = stream->readFloat();

			printInt(out, timeTag);
			printDouble(out,(double)f);
		}
}

