// AttitudeSlew.h
//
// Version  1.0
// Date     29.11.2000
// Author   T. Brown
//
// -------------------------------------------------------------------

#ifndef AttitudeSlewH
#define AttitudeSlewH

// -------------------------------------------------------------------

#include "../ManoeuvreManagement/Manoeuvre.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../AocsData/DataItemWrite.h"

// -------------------------------------------------------------------

class AttitudeSlew : public Manoeuvre {

private :

    // array for storage of telemetry image lengths
    static int tmImageLength[NTMFORMATS];

    // Attitude at the time when the manoeuvre starts
    Real att_start;

    // target attitude for the manoeuvre
    Real att_end;

    // The time when the manoeuvre actually starts
    AocsTime t0;

    // The end time for the manoeuvre (tf -> time final)
    AocsTime tf;

    // The DataItem which is to be set with calculated values
    // to cause the manoeuvre to occur.
    DataItemWrite itemToSet;

    // Slope of linear slew that will take the value of
    // the itemToSet from att_start to att_end in the
    // period of time from t0 to tf.
    Real slope;

    // whenever the difference between the current
    // attitude value and the value that this manoeuvre
    // is about to set for the attitude is greater than
    // this allowed difference, the manoeuvre is cannot continue.
    Real allowedDifference;

    // calculate next value to assign to the itemToSet
    Real calculateNextValue();

    // determine whether or not the specified value is
    // okay as a new value to set for the itemToSet
    // @param   newValue    value to check
    bool isOkayAsNewValue(Real newValue);

    // reset local configuration
    void localResetConfiguration();

	// Default target location for the attitude slew (overriden when the
	// attitude slew is configured) 
	static Real dummyTarget;
	static AocsTime dummyTargetTimeTag;

public :

    // Default constructor
    AttitudeSlew();

    // create an AttitudeSlew
    // @param   startTime       when the attitude slew manoeuvre should
    //                          begin execution
    // @param   endTime         when the attitude slew manoeuvre should
    //                          finish execution
    // @param   goalValue       the desired ending value of the attitude
    // @param   attitude        the attitude to set over time in order
    //                          to ultimately end up at the goalValue
    // @param   errorThreshold  whenever the difference between the current
    //                          attitude value and the value that this manoeuvre
    //                          is about to set for the attitude is greater than
    //                          this threshold, the manoeuvre is cannot continue.
    AttitudeSlew(AocsTime startTime, AocsTime endTime,
                 Real goalValue, DataItemWrite attitude,
                 Real errorThreshold);

    // Configure the AttitudeSlew
    // @param   startTime       when the attitude slew manoeuvre should
    //                          begin execution
    // @param   endTime         when the attitude slew manoeuvre should
    //                          finish execution
    // @param   goalValue       the desired ending value of the attitude
    // @param   attitude        the attitude value to set over time in
    //                          order to ultimately end up at the goalValue
    // @param   errorThreshold  whenever the difference between the current
    //                          attitude value and the value that this manoeuvre
    //                          is about to set for the attitude is greater than
    //                          this threshold, the manoeuvre is cannot continue.
    virtual void configure(AocsTime startTime, AocsTime endTime,
                           Real goalValue, DataItemWrite attitude,
                           Real errorThreshold);

    // @return  an indication of whether or not the manoeuvre is configured
    //          so that it can be run
    virtual bool isConfigured();

    // reset the configuration of the manoeuvre
    virtual void resetConfiguration();

    // @return  an indication of whether or not the current AOCS context
    //          makes it safe for execution of the Manoeuvre to start.
    virtual bool canStart();


    // perform initialization action for this AttitudeSlew Manoeuvre
    virtual void initialize();

    // @return  an indication of whether or not conditions for safe
    //          continuation of this Manoeuvre hold
    virtual bool canContinue();

    // perform any actions that are due and appropriate
    // for the current AocsTime
    virtual void doContinue();

    // from the Telemeterable interface
    virtual int getTelemetryImageLength();
    virtual void writeToTelemetry(TelemetryStream* tmStream);


};

// -------------------------------------------------------------------
#endif // AttitudeSlewH
