// Manoeuvre.cpp
//
// Version	1.0
// Date		17.07.00
// Author	T. Brown
//
// -------------------------------------------------------------------

#include "../ManoeuvreManagement/Manoeuvre.h"
#include "../AocsEvent/ManoeuvreEventRepository.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../AocsEvent/EventTypeId.h"
#include "../AocsEvent/ChangeEventRepository.h"
#include "../ObjectMonitoring/ChangeObject.h"
#include "../ObjectMonitoring/PropertyMonitor.h"

// -------------------------------------------------------------------
// Define static variables

int Manoeuvre::tmImageLength[NTMFORMATS];
ManoeuvreEventRepository*	Manoeuvre::manoeuvreEventRepository;
ChangeEventRepository*		Manoeuvre::changeEventRepository;

// -------------------------------------------------------------------
Manoeuvre::Manoeuvre() 
{
	setClassId(ID_MANOEUVRE);	

	tmImageLength[shortTm]  = 0;	// not implemented in f/w prototype
	tmImageLength[normalTm] = 0;
	tmImageLength[longTm]   = 0;
	tmImageLength[debugTm]  = 0;

	inUse = false;
	listSizeSet = false;

	localReset();
	localResetConfiguration();
}

// -------------------------------------------------------------------
void Manoeuvre::setMonitorListSize(int n)
{
	// This method should only be called once.
	if (listSizeSet) 
	{
		reportConfigurationError(this, MEMORY_ALREADY_ALLOCATED, this);
		return;
	}

	monitorList.allocateMemory(n);
	changeObjectList.allocateMemory(n);
	listSizeSet = true;
}

// -------------------------------------------------------------------
void Manoeuvre::notifyMonitors(int newManoeuvreState)
{
	// Go throught the list of registered monitors and their associated
	// change objects

	PropertyMonitor* monitor=monitorList.first(); 
	ChangeObject* changeObject=changeObjectList.first();
	while (!monitorList.isLast())
	{	if (changeObject->checkValue(currentState))
		{	changeEventRepository->create(this,
											MONITORED_PROPERTY_CHANGE,
											  getStatePropertyId(),
											    changeObject,
												  getStateProperty().get());
			monitor->propertyChange(this,getStateProperty(),changeObject);
		}
		monitor=monitorList.next();
		changeObject=changeObjectList.next();
	}
}
	
	
// -------------------------------------------------------------------
inline bool Manoeuvre::canStart() {
	return (currentState==ready) || (currentState==inProcess) || (currentState==onHold);
}

// -------------------------------------------------------------------
void Manoeuvre::initialize() 
{
	if (currentState == ready) 
	{
		currentState = inProcess;
		notifyMonitors(currentState);
	}

	manoeuvreEventRepository->create(this, MANOEUVRE_STARTED, this);
}

// -------------------------------------------------------------------
inline bool Manoeuvre::canContinue()
{
    return (currentState == inProcess);
}

// -------------------------------------------------------------------
inline void Manoeuvre::abort()
{
	if (currentState != finished) 
	{
		currentState = aborted;
		notifyMonitors(currentState);
	}

	manoeuvreEventRepository->create(this, MANOEUVRE_TERMINATED, this);
}

// -------------------------------------------------------------------
inline bool Manoeuvre::wasAborted() 
{
	return (currentState == aborted);
}

// -------------------------------------------------------------------
inline void Manoeuvre::terminate() 
{
	manoeuvreEventRepository->create(this, MANOEUVRE_TERMINATED, this);
}

// -------------------------------------------------------------------
inline bool Manoeuvre::isFinished() 
{
	return (currentState == finished);
}

// -------------------------------------------------------------------
void Manoeuvre::hold() 
{
	if ((currentState == ready) || (currentState == inProcess))
	{
		statePriorToHold = currentState;
		currentState = onHold;
		notifyMonitors(currentState);
	}

	manoeuvreEventRepository->create(this, MANOEUVRE_HELD, this);
}

// -------------------------------------------------------------------
void Manoeuvre::resume()
{
	if (currentState == onHold) 
	{
		currentState = statePriorToHold;
		notifyMonitors(currentState);
	}	

	manoeuvreEventRepository->create(this, MANOEUVRE_RESUMED, this);
}

// -------------------------------------------------------------------
void Manoeuvre::addMonitor(PropertyMonitor* m, ChangeObject* c)
{
	monitorList.addObject(m);
	changeObjectList.addObject(c);
}

// -------------------------------------------------------------------
void Manoeuvre::removeMonitor(PropertyMonitor* m)
{
	int position=monitorList.addObject(m);
	if (position>0)
		changeObjectList.removeObjectByPosition(position);
}

// -------------------------------------------------------------------
int Manoeuvre::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
void Manoeuvre::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// TBD : not implemented in framework prototype
}

// -------------------------------------------------------------------
void Manoeuvre::localReset()
{
	if (inUse)
	{	if ((currentState != aborted) && (currentState != finished))
		{
			currentState = beenReset;
			notifyMonitors(currentState);
		}
	}
	else
	{	
		execTime = 0;
		currentState = ready;
		notifyMonitors(currentState);
	}

	monitorList.reset();
}

// -------------------------------------------------------------------

void Manoeuvre::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------

void Manoeuvre::localResetConfiguration()
{
	monitorList.resetConfiguration();
}

// -------------------------------------------------------------------

void Manoeuvre::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

void Manoeuvre::resetStaticConfiguration()
{
	manoeuvreEventRepository = NULL;
	changeEventRepository = NULL;
}

// -------------------------------------------------------------------

bool Manoeuvre::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (!monitorList.isConfigured())
		return false;

	if (!changeObjectList.isConfigured())
		return false;

	if (manoeuvreEventRepository == NULL)
		return false;

	if (changeEventRepository == NULL)
		return false;

	return true;
}