// Manoeuvre.h
//
// Version	1.0
// Date		16.07.00
// Author	T. Brown/A. Pasetti
//
// -------------------------------------------------------------------
// Manoeuvre is an abstract class providing default, and mostly trivial,
// implementations of its methods.  Concrete manoeuvres are instances
// of subclasses of Manoeuvre.
// -------------------------------------------------------------------

#ifndef ManoeuvreH
#define ManoeuvreH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../BasicObjects/AocsObject.h"
#include "../ObjectList/ObjectListTemplate.h"
#include "../ObjectMonitoring/Property.h"

// -------------------------------------------------------------------

class Manoeuvre : public AocsObject {

	static int tmImageLength[NTMFORMATS];

	// List of manoeuvre monitors and associated change objects
	ObjectListTemplate<PropertyMonitor> monitorList;
	ObjectListTemplate<ChangeObject> changeObjectList;

	// Indication of whether or not the monitor list's
	// size has been established
	bool listSizeSet;

	// reference to event repositories
	static ManoeuvreEventRepository* manoeuvreEventRepository;	
	static ChangeEventRepository* changeEventRepository;

	// reset local state and configuration
	void localReset();
	void localResetConfiguration();

protected :
	
	// the time after which this
	// Manoeuvre should be considered for execution
	AocsTime execTime;

	// Flag is true if manoeuvre is currently loaded in manoeuvre manager
	bool inUse;

	// indication of the current state of this Manoeuvre
	ManoeuvreState currentState;

	// indication of state the Manoeuvre was in prior
	// to being put on hold
	ManoeuvreState statePriorToHold;

	// Call this method when the manoeuvre state changes. The method goes 
	// through the list of registered monitors and notifies them of the change
	// in the manoeuvre state
	void notifyMonitors(int newManoeuvreState);

public :
	
	// create a Manoeuvre
	// The created Manoeuvre is in the ready state
	Manoeuvre();

	// @return  an indication of whether or not the current AOCS context
	//          makes it safe for execution of the Manoeuvre to start
	// The default implementation returns true if the Manoeuvre is
	// in the ready state, the inProcess state, or the onHold state.
	// Typically, this method should only be called when the Manoeuvre
	// is in the ready state.
	virtual bool canStart();

	// allow the Manoeuvre to perform any special initialization 
	// action for this Manoeuvre
	// The default implementation transitions the Manoeuvre 
	// from the ready state to the inProcess state.  If the
	// Manoeuvre is not in the ready state, no transition occurs.
	virtual void initialize();

	// @return  an indication of whether or not conditions for safe
	//          continuation of this Manoeuvre hold
	// The default implementation returns true if the Manoeuvre
	// is in the inProcess state.
	virtual bool canContinue();

	// advance the execution of the Manoeuvre
	// (i.e. perform any actions that are due and appropriate
	// for the current AocsTime)
	// Notes:
	//   1. There is no default implementation.  Subclass 
	//      implementors must override this method
	//      to provide the appropriate behavior for the 
	//      particular type of Manoeuvre being implemented.
	//   2. This is a replacement for continue() in early
	//      versions of the documentation.  Since continue 
	//      is a keyword in the C++ language, it cannot be 
	//      used as a method name.
	//   3. When the Manoeuvre detects, during a call to this
	//      method, that it has completed its called for work,
	//      it should transition into the finished state.
	virtual void doContinue()=0;

	// allow the Manoeuvre to perform any clean-up actions that 
	// are necessary before the Manoeuvre is descheduled
	// The default implementation transitions the Manoeuvre into
	// the aborted state from any state but finished.  The 
	// finished state cannot be left.
	virtual void abort();

	// @return  an indication of whether or not the Manoeuvre
	//          was aborted
	// The default implementation returns true if the 
	// Manoeuvre's state is aborted.
	virtual bool wasAborted();

	// allow the Manoeuvre to perform any final close-down
	// actions prior to the Manoeuvre being descheduled.
	// The default implementation performs no final close-down
	// actions.
	// The default implementation does not transition the Manoeuvre
	// into the finished state.  The doContinue() method is
	// where this transition should occur.
	// Note: This method and the abort method are intended to be 
	//       mutually exclusive in nature.  If the ManoeuvreManager 
	//       detects that the Manoeuvre has terminated normally (see 
	//       isFinished()), then this method is called to allow normal cleanup.
	//       If the Manoeuvre is aborted, either because the ManoeuvreManager
	//       decides to abort the Manoeuvre or because the Manoeuvre itself
	//       indicates that it cannot continue, then the abort() method
	//       is used instead of this method.
	// 
    virtual void terminate();

	// @return  an indication of whether or not the Manoeuvre
	//          has terminated (completed normally)
	// The default implemenation returns true if the Manoeuvre
	// is in the finished state.
	virtual bool isFinished();

	// @return	an indication of whether or not the Manoeuvre
	//			can be put on hold.
	// The default implementation returns true if the Manoeuvre
	// is ready or inProcess.
	virtual bool canHold() 
	{	return ((currentState==ready) || (currentState==inProcess));}

	// allow the Manoeuvre to take actions associated with 
	// temporary suspension of the Manoeuvre.
	// The default implementation transitions the Manoeuvre
	// into the onHold state from the ready or inProcess states.
	// The default implementation takes no other special actions.
	virtual void hold();	

	// @return	an indication of whether or not the Manoeuvre
	//			is currently on hold (suspended)
	// The default implementation returns true if the Manoeuvre
	// is in the onHold state.
	virtual bool isHolding() 
	{	return (currentState==onHold);}

	// @return	an indication of whether or not the Manoeuvre
	//			can resume
	// The default implementation returns true if the Manoeuvre
	// is holding.
	virtual bool canResume() 
	{	return (currentState==onHold);}

	// allow the Manoeuvre to take actions associated with
	// resuming the Manoeuvre after a temporary suspension.
	// If the Manoeuvre is currently holding, then the default 
	// implementation transitions the Manoeuvre back to the 
	// state it was in when it was put on hold
	virtual void resume();

	// from the Telemeterable interface
	virtual int getTelemetryImageLength();
	virtual void writeToTelemetry(TelemetryStream* tmStream);

	// reset the Manoeuvre
	// The default implementation will place transition
	// the Manoeuvre into the reset state from any
	// state but aborted or finished.  Note that in the 
	// typical situation, once a Manoeuvre has been reset, 
	// it's canContinue() method will return false and 
	// therefore it will soon be aborted.
	virtual void reset();
 
	// Reset the monitor list
	virtual void resetConfiguration();

	// Unload the manoeuvre event repository
	static void resetStaticConfiguration();

	// Returns true if the monitor list has been configured and if
	// the manoeuvre event repository has been loaded
	virtual bool isConfigured();

	// Management of inUse flag (set to true by the manoeuvre monitor
	// when it loads the manoeuvre, reset to false when the manoeuvre
	// is unloaded
	void putInUse()	
		{ inUse = true; }

	void unUse()	
		{ inUse = false; }

	bool isInUse()
		{ return inUse; }

	// Management of manoeuvre monitors 
	void setMonitorListSize(int n);

	void addMonitor(PropertyMonitor* m, ChangeObject* c);

	void removeMonitor(PropertyMonitor* m);

	// Straightforwawd getter and setter methods
	static void setManoeuvreEventRepository(ManoeuvreEventRepository* rep)
		{ manoeuvreEventRepository = rep; }
	static ManoeuvreEventRepository* getManoeuvreEventRepository()
		{ return manoeuvreEventRepository; }

	static void setChangeEventRepository(ChangeEventRepository* rep)
		{ changeEventRepository = rep; }
	static ChangeEventRepository* getChangeEventRepository()
		{ return changeEventRepository; }

	AocsTime getTimeTag()
		{ return execTime; }
	void setTimeTag(AocsTime t)
		{ execTime = t; }

	virtual ManoeuvreState getState() 
		{ return currentState; }

	virtual PropertyId getStatePropertyId()
		{ return (PropertyId)getInstanceId();}

	virtual Property getStateProperty()
		{ return Property((int*)&currentState,getStatePropertyId());}

	virtual ManoeuvreType getType()		// manoeuvre type = class identifier
		{ return getClassId(); }

};

// -------------------------------------------------------------------
#endif // ManoeuvreH
