// ManoeuvreManagementTestSuite.cpp
//
// Version  1.0
// Date     17 Jul 2000
// Author   T. Brown/A. Pasetti
//
// Change Record:
//  AP, 15.08.00: commented all fprintf statements. They can be
//      automatically reinstated by replacing string "// fprintf"
//      with string "fprint"
//
// -------------------------------------------------------------------
// Test suite for manoeuvre management framelet
// -------------------------------------------------------------------

// Include system header files
#include <stdio.h>
#include <math.h>

#include "ManoeuvreManagementTestSuite.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../OperatingSystemObjects/DummyAocsClock.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/ManoeuvreEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../ObjectMonitoring/TestPropertyMonitor.h"
#include "../../ObjectMonitoring/SimpleChange.h"
#include "../../ManoeuvreManagement/ManoeuvreManager.h"
#include "../../ManoeuvreManagement/DummyManoeuvre.h"

// -------------------------------------------------------------------
char* describeTMMode(TelemetryFormat fmode)
{
    switch (fmode)
    {
    case shortTm:
        return "shortTm";
        // statement not reached break;
    case normalTm:
        return "normalTm";
        // statement not reached break;
    case longTm:
        return "longTm";
        // statement not reached break;
    case debugTm:
        return "debugTm";
        // statement not reached break;
    default:
        return "UNKNOWN TM!!!!";
        // statement not reached break;
    }
}

// -------------------------------------------------------------------
bool testManoeuvreTMLength(  TelemetryFormat fmode, AocsObject* ao, Manoeuvre* m)
{
    // Currently writing to telemetry for manoeuvres is not implemented.
    // So the TM length for a Manoeuvre should be equal to the
    // TM length for an AocsObject

    m->setTelemetryFormat(fmode);
    ao->setTelemetryFormat(fmode);

    int expectedImageLength = ao->getTelemetryImageLength();
    int testImageLength = m->getTelemetryImageLength();

    if (testImageLength!=expectedImageLength)
    {
         return false;
    }

    return true;
}

// -------------------------------------------------------------------
bool testManoeuvreEventTMLength(  TelemetryFormat fmode,
                                AocsEvent* aoe, ManoeuvreEvent* me)
{
    me->setTelemetryFormat(fmode);
    aoe->setTelemetryFormat(fmode);

    int expectedImageLength;
    switch (fmode)
    {
    case shortTm:
    case normalTm:
        // Currently, writing to telemetry for manoeuvre events is not implemented
        // for shortTm or normalTm mode.  So, in those two cases, the TM length for
        // a ManoeuvreEvent should be equal to the TM length for an AocsEvent
        expectedImageLength = aoe->getTelemetryImageLength();
        break;

    case longTm:
    case debugTm:
        // Writing to telemetry for manoeuvre events adds an InstanceId object
        // to the telemetry stream for longTm and debugTm modes.  So the expected
        // length is the length of an AocsEvent's telemetry image plus the length
        // of something of type InstanceId in bits
        expectedImageLength = aoe->getTelemetryImageLength() + 8*sizeof(InstanceId);
        break;

    default:
        expectedImageLength = -1;
        break;
    }

    int testImageLength = me->getTelemetryImageLength();

    if (testImageLength!=expectedImageLength)
    {
        return false;
    }

    return true;
}

// -------------------------------------------------------------------
bool verifyLatestConfigurationEvent( 
                                    ConfigurationEvent* ce,
                                    AocsTime expectedTime,
                                    int expectedType,
                                    void* expectedCreator,
                                    void* expectedLocation)
{
    // If there is no such event, we've failed our test.
    if (ce == NULL)
    {
        return false;
    }

    // Make sure event has the expected timestamp.
    AocsTime timeStamp = ce->getTimeStamp();

    if (timeStamp != expectedTime)
    {
        return false;
    }

    // Make sure event has expected type
    int evtType = ce->getEventType();

    if (evtType != expectedType)
    {
        // // fprintf( "\tevent type is not as expected\n");
        return false;
    }

    // Make sure event was created by the expected object

    if (ce->getCreator() != expectedCreator)
    {
        return false;
    }

    // Make sure event's location is as expected

    if (ce->getLocation() != expectedLocation)
    {
        // fprintf( "\tevent location is not as expected\n");
        return false;
    }

    return true;
}

// -------------------------------------------------------------------
bool checkDummyManoeuvreState(  Manoeuvre* m, AocsTime curTime,
                              AocsTime startTime, int stepCount, int clockPeriod)
{
    if (curTime < startTime)
    {   // manoeuvre should be ready
        if (m->getState() != ready)
        {
            // fprintf( "\tmanoeuvre (%p) is not ready as expected\n", m);
            return false;
        }
    }

    else if ((curTime >= startTime) &&
             (curTime < startTime + (stepCount * clockPeriod)))
    {   // manoeuvre should be inProcess
        if (m->getState() != inProcess)
        {
            return false;
        }
    }

    else
    {
        if (m->getState() != finished)
        {
            return false;
        }
    }

    return true;
}

// -------------------------------------------------------------------
// ...
bool checkSlew(AocsTime curTime, AocsTime actualStartTime, AocsTime endTime,
               Real curValue, Real initialValue, Real endValue,
               Real changePerCycle, int cyclePeriod)
{
    if (curTime < actualStartTime)
    {
        // manoeuvre should not have started yet
        if (!AreValuesClose(curValue, initialValue, 0.001))
            return false;
    }
    else if (curTime == actualStartTime)
    {
        // manoeuvre should have been initialized
        if (!AreValuesClose(curValue, initialValue, 0.001))
            return false;
    }
    else if (curTime >= endTime)
    {
        // manoeuvre should have set the value at the ending point
        if (!AreValuesClose(curValue, endValue, 0.001))
            return false;
    }
    else // (curTime > actualStartTime) && (curTime < endTime)
    {
        // manoeuvre should be changing the value

        // how long has it been since the manoeuvre actually started
        AocsTime timeSinceStart = curTime - actualStartTime;

        // how many cycles have we gone passed the start time
        long cycleCount = timeSinceStart / cyclePeriod;

        // how much should the value have changed
        Real changeAmount = cycleCount * changePerCycle;

        // what should the current value be
        Real curValueShouldBe = initialValue + changeAmount;

        // check
        if (!AreValuesClose(curValueShouldBe, curValue, 0.001))
            return false;
    }

    return true;
}

// -------------------------------------------------------------------
// ...
bool AreValuesClose(Real val1, Real val2, Real threshold)
{
    Real diff = fabs(val1 - val2);
    if (diff > threshold) return false;
    return true;
}

