// ManoeuvreManager.cpp
//
// Version  1.0
// Date     26.Jul.2000
// Author   T. Brown
//
// -------------------------------------------------------------------

#include "../ManoeuvreManagement/ManoeuvreManager.h"
#include "../ManoeuvreManagement/Manoeuvre.h"
#include "../AocsEvent/ManoeuvreEventRepository.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------
// Define static variables
ManoeuvreEventRepository*		ManoeuvreManager::manoeuvreEventRepository;
int ManoeuvreManager::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------
ManoeuvreManager::ManoeuvreManager() 
{
	setClassId(ID_MANOEUVREMANAGER);     // assign class identifier

	tmImageLength[shortTm] = 0;          // not implemented in f/w prototype 
	tmImageLength[shortTm] = 0;
	tmImageLength[shortTm] = 0;
	tmImageLength[shortTm] = 0;

	localResetConfiguration();
}

// -------------------------------------------------------------------
void ManoeuvreManager::localResetConfiguration() 
{
	// Cycle through manoeuvre list aborting them 
	Manoeuvre* mm;

	for (mm=manoeuvreList.first(); !manoeuvreList.isLast(); mm=manoeuvreList.next())
	{
		mm->abort();
	}

	manoeuvreList.resetConfiguration();

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::resetConfiguration() 
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------
bool ManoeuvreManager::isConfigured() 
{
	if (!AocsObject::isConfigured()) 
		return false;	

	if (manoeuvreList.getListSize() == 0) 
		return false;

	if (!manoeuvreList.isConfigured()) 
		return false;

	if (manoeuvreList.getListSize()==0)
		return false;

	if (manoeuvreEventRepository == NULL)
		return false;

	return true;
}

// -------------------------------------------------------------------
void ManoeuvreManager::resetStaticConfiguration()
{
	manoeuvreEventRepository = NULL;
}

// -------------------------------------------------------------------
void ManoeuvreManager::setManoeuvreEventRepository(ManoeuvreEventRepository* m)
{ 
	manoeuvreEventRepository = m; 
}

// -------------------------------------------------------------------
ManoeuvreEventRepository* ManoeuvreManager::getManoeuvreEventRepository()
{ 
	return manoeuvreEventRepository;
}

// -------------------------------------------------------------------
void ManoeuvreManager::writeToTelemetry(TelemetryStream* stream)
{
	AocsObject::writeToTelemetry(stream);	// write super object's state

	// TBD : not implemented in framework prototype
}

// -------------------------------------------------------------------
int ManoeuvreManager::getTelemetryImageLength()
{
	return AocsObject::getTelemetryImageLength() +
		tmImageLength[getTelemetryFormat()];
}

// -------------------------------------------------------------------
// This method should only be called once to set the list size to a 
// value greater than zero. Note that the manoeuvreList is initialized
// with a list size of zero.
void ManoeuvreManager::initializeManoeuvreList(int listSize) 
{
	if (manoeuvreList.getListSize()!=0)	
	{
		reportConfigurationError(this, MEMORY_ALREADY_ALLOCATED, this);
		return;
	}

	manoeuvreList.allocateMemory(listSize);
}

// -------------------------------------------------------------------
void ManoeuvreManager::addManoeuvre(Manoeuvre* m)
{
	// Check to see if the manoeuvre list is full
	if (manoeuvreList.isListFull()) 
	{
		reportConfigurationError(this, TOO_MANY_MANOEUVRES, this);
		return;
	}
	else
	{	manoeuvreList.addObject(m);
		m->putInUse();
	}

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::removeManoeuvre(Manoeuvre* m)
{
	// Check whether the manoeuvre was actually loaded
	if (!manoeuvreList.isObjectInList(m))
	{
		reportConfigurationError(this, MANOEUVRE_NOT_IN_LIST, this);
		return;		
	}

	// remove the specified manoeuvre if it is 
	// finished or aborted.  Otherwise, it cannot be 
	// removed
	if (m->isFinished() || m->wasAborted())
	{
		manoeuvreList.removeObject(m);
		m->unUse();
		return;		
	}

	// if we arrive here it means we've attempted to remove a manoeuvre
	// that was not finished or aborted.
	reportConfigurationError(this, INVALID_MANOEUVRE_REMOVAL, this);
	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::holdManoeuvre(Manoeuvre* m) 
{
	// Check whether the manoeuvre was actually loaded
	if (!manoeuvreList.isObjectInList(m))
	{
		reportConfigurationError(this, MANOEUVRE_NOT_IN_LIST, this);
		return;		
	}
	
	// determine if the specified manoeuvre is in a state 
	// in which it can be put on hold
	if (m->canHold())
	{
		// put the specified manoeuvre on hold
		m->hold();
	}
	else
	{
		reportConfigurationError(this, INVALID_MANOEUVRE_HOLD, this);
	}

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::holdManoeuvre(ManoeuvreType t) 
{
	// Cycle through manoeuvre list looking for manoeuvres
	// of the specified type
	Manoeuvre* mm;

	for (mm=manoeuvreList.first(); !manoeuvreList.isLast(); mm=manoeuvreList.next())
	{
		// If the manoeuvre is of the specified type
		// AND is in a state in which it can be put on hold,
		// hold it.
		if ((mm->getType()==t) && mm->canHold())
		{
			mm->hold();
		}
	}

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::resumeManoeuvre(Manoeuvre* m)
{
	// Check whether the manoeuvre was actually loaded
	if (!manoeuvreList.isObjectInList(m))
	{
		reportConfigurationError(this, MANOEUVRE_NOT_IN_LIST, this);
		return;		
	}
	
	// verify that the specified manoeuvre can resume
	if (m->canResume())
	{
		reportConfigurationError(this, INVALID_MANOEUVRE_RESUME, this);
		return;
	}

	// resume the specified manoeuvre
	m->resume();
	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::resumeManoeuvre(ManoeuvreType t)
{
	// Cycle through manoeuvre list looking for manoeuvres
	// of the specified type
	Manoeuvre* mm;

	for (mm=manoeuvreList.first(); !manoeuvreList.isLast(); mm=manoeuvreList.next())
	{
		// If the manoeuvre is of the specified type
		// AND is in a state in which it can be resumed,
		// resume it.
		if ((mm->getType()==t) && mm->canResume())
		{
			mm->resume();
		}
	}

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::startManoeuvre(Manoeuvre* m) 
{
	// Check whether the manoeuvre was actually loaded
	if (!manoeuvreList.isObjectInList(m))
	{
		reportConfigurationError(this, MANOEUVRE_NOT_IN_LIST, this);
		return;		
	}

	// initialize the manoeuvre
	m->initialize();

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::startManoeuvre(ManoeuvreType t) 
{
	// Cycle through manoeuvre list looking for manoeuvres
	// of the specified type
	Manoeuvre* mm;

	for (mm=manoeuvreList.first(); !manoeuvreList.isLast(); mm=manoeuvreList.next())
	{
		// If the manoeuvre is of the specified type
		// initialize it.
		if (mm->getType()==t)
		{
			mm->initialize();
		}
	}

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::abortManoeuvre(Manoeuvre* m) 
{
	// Check whether the manoeuvre was actually loaded
	if (!manoeuvreList.isObjectInList(m))
	{
		reportConfigurationError(this, MANOEUVRE_NOT_IN_LIST, this);
		return;		
	}

	// abort the manoeuvre
	m->abort();

	// remove the manoeuvre from our list
	manoeuvreList.removeObject(m);
	
	// Mark the manoeuvre as "not loaded"
	m->unUse();

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::abortManoeuvre(ManoeuvreType t) 
{
	// Cycle through manoeuvre list looking for manoeuvres
	// of the specified type
	Manoeuvre* mm;

	for (mm=manoeuvreList.first(); !manoeuvreList.isLast(); mm=manoeuvreList.next())
	{
		// If the manoeuvre is of the specified type,
		// abort it and remove it from our manoeuvre list
		if (mm->getType()==t)
		{
			mm->abort();
			manoeuvreList.removeObject(mm);
			mm->unUse();
		}
	}

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::addManoeuvreMonitor(ManoeuvreType t, ManoeuvreMonitor* o)
{
	// To be implemented

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::removeManoeuvreMonitor(ManoeuvreType t, ManoeuvreMonitor* o)
{
	// to be implemented

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::runManoeuvre(Manoeuvre* mm)
{
	// Find out what state this manoeuvre is in and dispatch accordingly
	ManoeuvreState state = mm->getState();

	switch (state) 
	{
		case inProcess:
			runInProcessManoeuvre(mm);
			break;

		case ready:
			runReadyManoeuvre(mm);
			break;

		case finished:
			runFinishedManoeuvre(mm);
			break;

		case beenReset:
			runResetManoeuvre(mm);
			break;

		case aborted:
			// In this implementation, there should not be any 
			// previously aborted manoeuvres in the list.  However,
			// subclasses might change this behavior, so we provide 
			// a hook for handling such manoeuvres.
			runAbortedManoeuvre(mm);
			break;

		case onHold:
			// In this implementation, no action is required on
			// manoeuvres that are on hold.  However, subclasses 
			// might change this behavior (maybe removing any 
			// manoeuvre that has been on hold for "too long"), so
			// we provide a hook for handling such manoeuvres.
			runHeldManoeuvre(mm);
			break;

		default:
			reportConfigurationError(this, UNKNOWN_MANOEUVRE_STATE, this);
			break;

	} // switch
}

// -------------------------------------------------------------------
void ManoeuvreManager::runInProcessManoeuvre(Manoeuvre* mm)
{
	// Verify that the manoeuvre can continue.  
	// If it can continue, then do so.  
	// If not, then abort and remove it from our list.
	if (mm->canContinue()) 
	{
		mm->doContinue();
	}
	else 
	{
		mm->abort();
		manoeuvreList.removeObject(mm);
		mm->unUse();
	}

	return;
}
		
// -------------------------------------------------------------------
void ManoeuvreManager::runReadyManoeuvre(Manoeuvre* mm)
{
	// Check to see if is the appropriate time to start the manoeuvre.	
	if (getTime() >= mm->getTimeTag()) 
	{
		// It is time to start the manoeuvre, so 
		// ask the manoeuvre if conditions are right.  
		// If they are, start the manoeuvre.  If not, don't.
		if (mm->canStart()) 
		{
			mm->initialize();
		}
	}

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::runFinishedManoeuvre(Manoeuvre* mm)
{
	// Notify the manoeuvre that it can perform its final
	// close down and remove it from our list.
	mm->terminate();
	manoeuvreList.removeObject(mm);
	mm->unUse();

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::runResetManoeuvre(Manoeuvre* mm)
{
	// Notify the manoeuvre that it is being aborted and remove it 
	// from our list.
	mm->abort();
	manoeuvreList.removeObject(mm);
	mm->unUse();

	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::runAbortedManoeuvre(Manoeuvre* mm)
{
	manoeuvreList.removeObject(mm);
	mm->unUse();
	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::runHeldManoeuvre(Manoeuvre* mm)
{
	// Manoeuvres that are onHold require no action
	return;
}

// -------------------------------------------------------------------
void ManoeuvreManager::run(AocsTime Deadline)
{
	// Cycle through the manoeuvres in our list
	Manoeuvre* mm;
	for (mm=manoeuvreList.first(); !manoeuvreList.isLast(); mm=manoeuvreList.next())
	{
		runManoeuvre(mm);
	}
}

// -------------------------------------------------------------------