// ManoeuvreManager.h
//
// Version	1.0
// Date		26.Jul.00
// Author	T. Brown/A. Pasetti
//
// -------------------------------------------------------------------
// ManoeuvreManager is an active component that is responsible for 
// controlling execution of manoeuvres.
// -------------------------------------------------------------------

#ifndef ManoeuvreManagerH
#define ManoeuvreManagerH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../BasicObjects/AocsObject.h"
#include "../OperatingSystemObjects/Runnable.h"
#include "../ObjectList/ObjectListTemplate.h"

// -------------------------------------------------------------------

class ManoeuvreManager : public AocsObject, public Runnable {

	static int tmImageLength[NTMFORMATS];

	// reference to manoeuvre event repository
	static ManoeuvreEventRepository* manoeuvreEventRepository;

	// List of manoeuvres to be executed
	ObjectListTemplate<Manoeuvre> manoeuvreList;

	// Local reset methods
	void localResetConfiguration();

	// handle one run cycle for the specified manoeuvre 
	//
	// @param	mm	monitored manoeuvre to run
	//
	// This method determines the current state of the manoeuvre
	// and dispatches it to other methods accordingly.
	// Subclasses can override this method to change the way
	// manoeuvres are dispatched without needing to manipulate
	// the manoeuvre list
	virtual void runManoeuvre(Manoeuvre* mm);

	// handle one run cycle for the specified manoeuvre which
	// is currently in the inProcess state
	//
	// @param	mm	manoeuvre to run
	//
	// This implementation verifies that the manoeuvre can continue.  
	// If it can continue, then it is continued.  Otherwise,
	// it is aborted and removed from the manoeuvre list.
	//
	// Subclasses can override this method to change just the way
	// an inProcess manoeuvre is handled.
	virtual void runInProcessManoeuvre(Manoeuvre* mm);

	// handle one run cycle for the specified manoeuvre which
	// is currently in the ready state
	//
	// @param	mm	manoeuvre to run
	//
	// This implementation checks to see if is the appropriate time 
	// to start the manoeuvre.  If it is and conditions are right
	// (as determined by a call to canStart()), then the manoeuvre 
	// is initialized.
	//
	// Subclasses can override this method to change just the way
	// a ready manoeuvre is handled.
	virtual void runReadyManoeuvre(Manoeuvre* mm);

 	// handle one run cycle for the specified manoeuvre which
	// is currently in the finished state
	//
	// @param	mm	manoeuvre to run
	//
	// This implementation notifies the manoeuvre that it can perform 
	// its final close down and removes it from the manoeuvre list.
	//
	// Subclasses can override this method to change just the way
	// a finished manoeuvre is handled.
	virtual void runFinishedManoeuvre(Manoeuvre* mm);

 	// handle one run cycle for the specified manoeuvre which
	// is currently in the beenReset state
	//
	// @param	mm	manoeuvre to run
	//
	// This implementation notifies the manoeuvre that it is 
	// being aborted and removes it from the manoeuvre list. 
	//
	// Subclasses can override this method to change just the way
	// a reset manoeuvre is handled.
	virtual void runResetManoeuvre(Manoeuvre* mm);

	// handle one run cycle for the specified manoeuvre which
	// is currently in the aborted state
	//
	// @param	mm	manoeuvre to run
	//
	// This implementation removes the manoeuvre from the manoeuvre list. 
	//
	// Subclasses can override this method to change just the way
	// an aborted manoeuvre is handled.	
	virtual void runAbortedManoeuvre(Manoeuvre* mm);

	// handle one run cycle for the specified manoeuvre which
	// is currently in the onHold state
	//
	// @param	mm	monitored manoeuvre to run
	//
	// This implementation does nothing.
	//
	// Subclasses can override this method to change just the way
	// an onHold manoeuvre is handled.
	virtual void runHeldManoeuvre(Manoeuvre* mm);

public :
	// create a ManoeuvreManager
	ManoeuvreManager();

	// methods from Configurable interface
	virtual void resetConfiguration();
	virtual bool isConfigured();

	// methods from the Telemeterable interface
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

	// methods to reset and check the static part of 
	// the ManoeuvreManager configuration
	static void resetStaticConfiguration();

	// Straightforward getter and setter methods
	static void setManoeuvreEventRepository(ManoeuvreEventRepository* m);
	static ManoeuvreEventRepository* getManoeuvreEventRepository();
	
	// initialize the ManoeuvreManager
	// @param	listSize	the number of manoeuvres supported by this
	//						ManoeuvreManager
	// ManoeuvreManager class must be statically configured prior to 
	// initializing an individual ManoeuvreManager
	virtual void initializeManoeuvreList(int listSize);

	// load a manoeuvre
	// @param	m	manoeuvre to load
	virtual void addManoeuvre(Manoeuvre* m);

	// unload a manoeuvre
	// @param	m	manoeuvre to unload
	virtual void removeManoeuvre(Manoeuvre* m);

	// hold a manoeuvre
	// @param	m	manoeuvre to hold
	virtual void holdManoeuvre(Manoeuvre* m);

	// hold all manoeuvres of the specified type
	// @param	t	type of manoeuvres to hold
	virtual void holdManoeuvre(ManoeuvreType t);

	// resume a held manoeuvre
	// @param	m	manoeuvre to resume
	virtual void resumeManoeuvre(Manoeuvre* m);

	// resume all manoeuvres of the specified type
	// @param	t	type of manoeuvres to resume
	virtual void resumeManoeuvre(ManoeuvreType t);

	// start a manoeuvre
	// @param	m	manoeuvre to start
	//
	// This method forces the start of a manoeuvre.  Thereby
	// ignoring the manoeuvres specified start time _and_
	// ignoring its specified starting conditions as given
	// by it's canStart() method.
	virtual void startManoeuvre(Manoeuvre* m);

	// start all manoeuvres of the specified type
	// @param	t	type of manoeuvres to start
	//
	// This method forces the starting of all manoeuvres of 
	// the specified type.  Just as the startManoeuvre(Manoeuvre* m)
	// method does, this method ignores both the specified 
	// start time for its manoeuvres and their starting conditions.
	virtual void startManoeuvre(ManoeuvreType t);

	// abort a manoeuvre
	// @param	m	manoeuvre to abort
	virtual void abortManoeuvre(Manoeuvre* m);

	// abort all manoeuvres of the specified type
	// @param	t	type of manoeuvres to abort
	virtual void abortManoeuvre(ManoeuvreType t);

	// add a monitor for all manoeuvres of a specific type
	// @param	t	type of manoeuvres to which to add a monitor
	// @param	o	monitor to be notified of significant events
	//				related to any manoeuvres of the specified type
	virtual void addManoeuvreMonitor(ManoeuvreType t, ManoeuvreMonitor* o);

	// remove a monitor for all manoeuvres of a specific type
	// @param	t	type of manoeuvres from which to remove a monitor
	// @param	o	monitor to no longer be notified of significant events
	//				related to any manoeuvres of the specified type	
	virtual void removeManoeuvreMonitor(ManoeuvreType t, ManoeuvreMonitor* o);

	// from Runnable interface
	// Scheduler entry point
	virtual void run(AocsTime Deadline);
};

// -------------------------------------------------------------------
#endif // ManoeuvreManagerH
