// ModeManager.cpp
//
// Version	1.0
// Date		01.04.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../ModeManagement/ModeManager.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/ClassId.h"
#include "../BasicObjects/AocsObject.h"
#include "../ObjectMonitoring/Property.h"
#include "../ObjectMonitoring/PropertyMonitor.h"
#include "../ObjectMonitoring/ChangeObject.h"
#include "../ModeManagement/ModeChangeAction.h"
#include "../AocsEvent/EventTypeId.h"
#include "../AocsEvent/ModeEventRepository.h"
#include "../AocsEvent/ChangeEventRepository.h"

// -------------------------------------------------------------------
// Define static variables
ChangeEventRepository*	ModeManager::changeEventRepository=NULL;
ModeEventRepository*	ModeManager::modeEventRepository=NULL;
int ModeManager::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

ModeManager::ModeManager(int m, int s, int d, int n)
{
	setClassId(ID_MODEMANAGER);				

	tmImageLength[shortTm]  = 0;	
	tmImageLength[normalTm] = 8*sizeof(int);	// current mode
	tmImageLength[longTm]   = tmImageLength[normalTm];
	tmImageLength[debugTm]  = tmImageLength[longTm]+8*2*sizeof(int) ;
	
	nStrategies = s;
	nModes = m;
	defaultMode = d;
	currentMode = d;

	notYetUsed = true;

	// Allocate memory
	implementerArray = new RootObject**[m];
	for (int i=0; i<m; i++)
		implementerArray[i] = new RootObject*[s];

	inhibitModeArray = new bool[m];

	monitorList.allocateMemory(n);
	changeObjectList.allocateMemory(n);

	localReset();
	localResetConfiguration();
}

// -------------------------------------------------------------------

void ModeManager::localReset()
{
	for (int i=0; i<nModes; i++)
		inhibitModeArray[i] = false;

	forcedMode = false;

	changeMode(defaultMode);
}

// -------------------------------------------------------------------

void ModeManager::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------

void ModeManager::localResetConfiguration()
{
	modeChangeAction = NULL;

	for (int i=0; i<nModes; i++)
		for (int j=0; j<nStrategies; j++)
			loadImplementer(i,j,(RootObject*)NULL);

	monitorList.resetConfiguration();
	changeObjectList.resetConfiguration();
}

// -------------------------------------------------------------------

void ModeManager::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

void ModeManager::resetStaticConfiguration()
{
	modeEventRepository = NULL;
	changeEventRepository = NULL;
}

// -------------------------------------------------------------------

bool ModeManager::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (modeEventRepository==NULL)
		return false;

	if (changeEventRepository==NULL)
		return false;

	if (modeChangeAction==NULL)
		return false;

	if (!monitorList.isConfigured())
		return false;

	if (!changeObjectList.isConfigured())
		return false;
	
	for (int i=0; i<nModes; i++)
		for (int j=0; j<nStrategies; j++)
			if (implementerArray[i][j] == NULL)
				return false;

	return true;
}

// -------------------------------------------------------------------
// Configuration method to load the implementations
void ModeManager::loadImplementer(int mode, int strategy, RootObject* implementer)
{
	if ( (isModeLegal(mode)) && 
		((strategy>=0) && (strategy<nStrategies)) )
		implementerArray[mode][strategy] = implementer; 
	else
		reportConfigurationError(this,ILLEGAL_STRATEGY_OR_MODE,this);
}

// -------------------------------------------------------------------
// Only modes in the [0..nModes-1] range are legal

bool ModeManager::isModeLegal(int mode)
{
	return ( (mode>=0) && (mode<nModes) );
}

// -------------------------------------------------------------------
// Return the identifier of the mode property

PropertyId ModeManager::getModePropertyId()
{
	return (PropertyId)getInstanceId();
}

// -------------------------------------------------------------------
// Return the current mode as a property object

Property ModeManager::getModeProperty()
{
	return Property(&currentMode, getModePropertyId());
}

// -------------------------------------------------------------------
// Return the strategy corresponding to the current mode.
// Clear the notYetUsed flag to signify that the mode manager has been
// used.
// If this was the first time the mode manager was used, perform the
// mode entry action.

RootObject* ModeManager::getImplementer(int strategy)
{
	if (notYetUsed)
	{	modeChangeAction->doModeEntryAction(currentMode);
		notYetUsed = false;
	}

	if ( (strategy>=0) && (strategy<nStrategies) )
		return implementerArray[currentMode][strategy];
	else
	{	reportConfigurationError(this,ILLEGAL_STRATEGY,this);
		return NULL;
	}
}

// -------------------------------------------------------------------

void ModeManager::setMode(int newMode)
{ 
	if (isModeLegal(newMode))
		changeMode(newMode);
	else
		reportConfigurationError(this,ILLEGAL_MODE,this);
}

// -------------------------------------------------------------------

void ModeManager::forceMode(int newMode)
{ 
	if (isModeLegal(newMode))
	{	changeMode(newMode);
		forcedMode = true;
	}
	else
		reportConfigurationError(this,ILLEGAL_MODE,this);
}

// -------------------------------------------------------------------

void ModeManager::releaseMode()
{ 
	forcedMode = false;
}

// -------------------------------------------------------------------

void ModeManager::inhibitMode(int mode)
{ 
	if (isModeLegal(mode))
		inhibitModeArray[mode] = true;
	else
		reportConfigurationError(this,ILLEGAL_MODE,this);
}

// -------------------------------------------------------------------

void ModeManager::enableMode(int mode)
{ 
	inhibitModeArray[mode] = false; 
}

// -------------------------------------------------------------------

void ModeManager::addMonitor(PropertyMonitor* m, ChangeObject* c)
{
	monitorList.addObject(m);
	changeObjectList.addObject(c);
}

// -------------------------------------------------------------------

void ModeManager::removeMonitor(PropertyMonitor* m)
{
	int position=monitorList.removeObject(m);
	if (position>=0)
		changeObjectList.removeObjectByPosition(position);
}

// -------------------------------------------------------------------

void ModeManager::changeMode(int targetMode)
{
	// Check if mode transition is allowed
	if (forcedMode)
		return;
	if (inhibitModeArray[targetMode])
		return;

	// Do mode change only if the target mode is different from the current mode
	if (targetMode == currentMode)
		return;

	// Create event describing mode change
	modeEventRepository->create(this,OPERATIONAL_MODE_CHANGE,currentMode,targetMode,this);

	// Before exiting the mode, perform mode exit actions
	modeChangeAction->doModeExitAction(currentMode);

	// Change the mode 
	currentMode = targetMode;

	// Upon entering the new mode, perform mode entry actions
	modeChangeAction->doModeEntryAction(currentMode);

	// Notify all monitors
	PropertyMonitor* monitor=monitorList.first(); 
	ChangeObject* changeObject=changeObjectList.first();
	while (!monitorList.isLast())
	{	if (changeObject->checkValue(targetMode))
		{	changeEventRepository->create(this,MONITORED_PROPERTY_CHANGE,getModePropertyId(),
										changeObject,getModeProperty().get());
			monitor->propertyChange(this,getModeProperty(),changeObject);
		}
		monitor=monitorList.next();
		changeObject=changeObjectList.next();
	}
}

// -------------------------------------------------------------------
// From Telemeterable interface

int ModeManager::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------
// From Telemeterable interface

void ModeManager::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// TBD : not implemented in framework prototype
}
