// ModeManager.h
//
// Version	1.0
// Date		01.04.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Abstract base class for the core mode manager
//
// -------------------------------------------------------------------

#ifndef ModeManagerH
#define ModeManagerH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/Constants.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../BasicObjects/AocsObject.h"
#include "../ObjectList/ObjectListTemplate.h"
#include "../ObjectMonitoring/Property.h"
#include "../ObjectMonitoring/PropertyMonitor.h"

// -------------------------------------------------------------------

class ModeManager : public AocsObject {

	static int tmImageLength[NTMFORMATS];

	// Reference to mode and change event repositories
	static ModeEventRepository* modeEventRepository;
	static ChangeEventRepository* changeEventRepository;

	// Array holding implementers (implementerArray[i][j] holds the j-th
	// strategy for the i-th mode)
	RootObject*** implementerArray;

	// List of registered monitors (to be notified in case of mode changes)
	// and of their associated change objects
	ObjectListTemplate<PropertyMonitor> monitorList;
	ObjectListTemplate<ChangeObject> changeObjectList;

	// If inhibitMode[i] is true, then entry into the i-th mode is not possible
	bool* inhibitModeArray;

	// If forcedMode is true, then no mode changes are possible
	bool forcedMode;

	// Default mode indicator
	int defaultMode;

	// True until the mode manager is used for the first time by its
	// mode-dependent component (ie. until its getImplementer method
	// is called for the first time). This flag is used to ensure that
	// the entry mode change action is executed after the mode manager
	// has been created and before it is used.
	bool notYetUsed;

	// Mode change action
	ModeChangeAction* modeChangeAction;

	// Local reset methods
	void localResetConfiguration();		
	void localReset();

	// Getter method for the mode property identifier
	PropertyId getModePropertyId();

protected :

	// Constructor
	ModeManager(int numberOfModes, 
					int numberOfStrategies, 
						int defaultMode,
							int maxNumberOfMonitors);

	// Change to a new mode (notifying registered monitors of the change)
	void changeMode(int targetMode);

	// Update the current mode
	virtual void updateMode()
		{ return; }

	// Check whether the argument mode is legal
	virtual bool isModeLegal(int mode);

	// Mode variables
	int nStrategies;			// number of strategies
	int nModes;					// number of modes
	int currentMode;			// current mode indicator

public :

	// Getter method for the mode property 
	Property getModeProperty();

	// Configuration method to load the implementers
	void loadImplementer(int mode, int strategy, RootObject* implementer);

	// Mode control methods
	void setMode(int newMode);
	void forceMode(int newMode);
	void releaseMode();
	void inhibitMode(int mode);
	void enableMode(int mode);

	// Strategy getter method
	virtual RootObject* getImplementer(int strategy);

	// Add and remove a mode monitor
	void addMonitor(PropertyMonitor* monitor, ChangeObject* changeObject);
	void removeMonitor(PropertyMonitor* monitor);

	// Methods inherited from interface Configurable
	virtual void resetConfiguration();		
	virtual bool isConfigured();		

	// Methods inherited from interface Resettable
	virtual void reset();					

	// Methods inherited from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

	// Reset the static part of the ModeManager configuration
	static void resetStaticConfiguration();

	// Straightforward getter and setter methods
	int getMode()
		{ return currentMode; }

	static void setChangeEventRepository(ChangeEventRepository* c)
		{ changeEventRepository = c; }
	static ChangeEventRepository* getChangeEventRepository()
		{ return changeEventRepository; }

	static void setModeEventRepository(ModeEventRepository* m)
		{ modeEventRepository = m; }
	static ModeEventRepository* getModeEventRepository()
		{ return modeEventRepository; }

	void setModeChangeAction(ModeChangeAction* m)
		{ modeChangeAction = m; }
	ModeChangeAction* getModeChangeAction()
		{ return modeChangeAction; }
};

// -------------------------------------------------------------------
#endif
