// BasicUnitReconfigurer.h
//
// Version	1.0
// Date		26.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Manages a group of n identical redundant units. The units are 
// exposed as instances of class AocsFunctional but are treated as
// instances of class AocsUnit.
// A reconfigurer helper object is used to manage the reconfiguration
// status.
//
// -------------------------------------------------------------------

#ifndef BasicUnitReconfigurerH
#define BasicUnitReconfigurerH

// -------------------------------------------------------------------

#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../BasicObjects/AocsObject.h"
#include "../AocsUnit/AocsUnit.h"
#include "../ReconfigurationManagement/Reconfigurable.h"
#include "../ReconfigurationManagement/ReconfigurerHelper.h"

// -------------------------------------------------------------------

class BasicUnitReconfigurer : public AocsObject, 
								public Reconfigurable,
								  public AocsUnitFunctional {

	static int tmImageLength[NTMFORMATS];

	ReconfigurerHelper* helper;

	// Array of units to  be reconfigured
	AocsUnit** unit;

	// Local reset methods
	void localReset();
	void localResetConfiguration();

public :

	BasicUnitReconfigurer(int numberOfUnit, 
							int numberOfMonitors,
							  ConfigurationId defaultConfiguration);

	// Configuration property management
	virtual PropertyId getConfigurationPropertyId()
		{ return helper->getConfigurationPropertyId(); }
	virtual Property getConfigurationProperty()
		{ return helper->getConfigurationProperty(); }

	// Return current configuration
	virtual ConfigurationId getConfiguration()
		{ return helper->getConfiguration(); }

	// Force the current configuration to n
	virtual void setConfiguration(ConfigurationId n);

	// Mark configuration n healthy (true) or unhealthy (false)
	virtual void markConfiguration(ConfigurationId n, bool mark)
		{ helper->markConfiguration(n,mark); }

	// Set the rank of configuration n to r
	virtual void rankConfiguration(ConfigurationId n, ConfigurationRank r)
		{ return; }

	// Enable and disable a particular configuration
	virtual void enableConfiguration(ConfigurationId n)
		{ helper->enableConfiguration(n); }
	virtual void disableConfiguration(ConfigurationId n)
		{ helper->disableConfiguration(n); }

	// Enable and disable all reconfigurations
	virtual void enableReconfiguration()
		{ helper->enableReconfiguration(); }
	virtual void disableReconfiguration()
		{ helper->disableReconfiguration(); }

	// Command a reconfiguration
	virtual void reconfigure();

	// Return true if a reconfiguration is possible
	virtual bool canReconfigure()
		{ return helper->canReconfigure(); }

	// Return true if configuration n is healthy
	virtual bool isConfigurationHealthy(ConfigurationId n)
		{ return helper->isConfigurationHealthy(n); }

	// Return the reconfiguration order
	virtual int getReconfigurationOrder()
		{ return helper->getReconfigurationOrder(); }

	// Return the rank of configuration n
	virtual ConfigurationRank getConfigurationRank(ConfigurationId n)
		{ return helper->getConfigurationRank(n); }

	// Setter and getter methods for the reconfiguration state
	virtual void setConfigurationState(ConfigurationState s)
		{ helper->setConfigurationState(s); }
	virtual ConfigurationState getConfigurationState()
		{ return helper->getConfigurationState(); }

	// Add and remove a configuration monitor
	void addMonitor(PropertyMonitor* monitor, ChangeObject* changeObject)
		{ helper->addMonitor(monitor,changeObject); }
	void removeMonitor(PropertyMonitor* monitor)
		{ helper->removeMonitor(monitor); }

	// Setter and getter method for recovery action associated to 
	// failed reconfiguration request
	void setReconfigurationErrorRecoveryAction(RecoveryAction* r)
		{ helper->setReconfigurationErrorRecoveryAction(r);}
	RecoveryAction* getReconfigurationErrorRecoveryAction()
		{ return helper->getReconfigurationErrorRecoveryAction(); }

	// Methods inherited from interface Configurable
	virtual void resetConfiguration();
	virtual bool isConfigured();

	// Methods inherited from interface Resettable
	virtual void reset();
		
	// Methods inherited from interface Telemeterable
	virtual void writeToTelemetry(TelemetryStream* stream);
	virtual int getTelemetryImageLength();

	// Methods inherited from AocsUnitFunctional interface 
	// implemented by delegation to the currently active unit
	virtual void acquireFunctionalData()
		{ unit[getConfiguration()]->acquireFunctionalData(); }
	virtual void sendFunctionalData()
		{ unit[getConfiguration()]->sendFunctionalData(); }
	virtual bool isAcquireTransactionFinished()
		{ return unit[getConfiguration()]->isAcquireTransactionFinished(); }
	virtual bool isSendTransactionFinished()
		{ return unit[getConfiguration()]->isSendTransactionFinished(); }
	virtual void refreshFunctionalIn()
		{ unit[getConfiguration()]->refreshFunctionalIn(); }
	virtual void refreshFunctionalOut()
		{ unit[getConfiguration()]->refreshFunctionalOut(); }
	virtual void synchronizeFunctional()
		{ unit[getConfiguration()]->synchronizeFunctional(); }
	virtual bool isSynchronizeFunctionalFinished()
		{ return unit[getConfiguration()]->isSynchronizeFunctionalFinished(); }
	virtual DataItemWrite getFunctionalInLink(int i)
		{ return unit[getConfiguration()]->getFunctionalInLink(i); }
	virtual DataItemWrite getFunctionalOutLink(int i)
		{ return unit[getConfiguration()]->getFunctionalOutLink(i); }

	// Methods inherited from AocsUnitFunctional interface 
	// with dummy implementation
	virtual void setFunctionalInConverter(AbstractControlChannel* c)
		{ return; }
	virtual void setFunctionalOutConverter(AbstractControlChannel* c)
		{ return; }
	virtual void setFunctionalInLink(DataItemWrite d, int i)
		{ return; }
	virtual void setFunctionalOutLink(DataItemWrite d, int i)
		{ return; }
	virtual void setFunctionalInLink(AocsData* d)
		{ return; }
	virtual void setFunctionalOutLink(AocsData* d)
		{ return; }

	// Setter and getter methods for reconfigurable units
	void setUnit(AocsUnit* u, int n);
	AocsUnit* getUnit(int n);

};

// -------------------------------------------------------------------
#endif