// TestCaseReconfiguration_3.cpp
//
// Version	1.0
// Date		03.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseReconfiguration_3.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../AocsData/DataItemWrite.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ConfigurationEvent.h"
#include "../../AocsEvent/ReconfigurationEvent.h"
#include "../../AocsEvent/SystemEvent.h"
#include "../../AocsEvent/EventTypeId.h"
#include "../../AocsEvent/FailureEventRepository.h"
#include "../../AocsEvent/ConfigurationEventRepository.h"
#include "../../AocsEvent/ReconfigurationEventRepository.h"
#include "../../SequentialDataProcessing/RawBusDataConverterIn.h"
#include "../../AocsUnit/MacsTcController.h"
#include "../../AocsUnit/FssPrototype.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../ReconfigurationManagement/BasicUnitReconfigurer.h"

// -------------------------------------------------------------------
TestCaseReconfiguration_3::TestCaseReconfiguration_3():
	TestCaseGenericSetUp(ID_BASICUNITRECONFIGURER*10+3,
									"TestCaseReconfiguration_3") {
		return;
}

// -------------------------------------------------------------------
// Check the configuration state registration mechanism.
// Two basic unit reconfigurer are set up. Both manage the same set of
// three fine sun sensors.
// The configuration of the basic reconfigurers is changed and a system
// reset is commanded. After the reset, the configuration of the basic
// reconfigurer should have been brought back to where it was just before
// the system reset.

void TestCaseReconfiguration_3::runTestCase() {

	// Retrieve the number of various types of relevant events
	int fen = AocsObject::getFailureEventRepository()->getCounter();
	int cen = AocsObject::getConfigurationEventRepository()->getCounter();
	int ren = ReconfigurerHelper::getReconfigurationEventRepository()->getCounter();
	int hen = ReconfigurerHelper::getChangeEventRepository()->getCounter();
	
	// Create and configure a MACS TC object
	short macsCs=0;
	short macsInstr=0;
	short macsData=0;
	MacsTcController macsTcCont(&macsCs, &macsInstr, &macsData);
	macsTcCont.setMaxPolling(2);

	// Default recovery action for various objects
	NullRecoveryAction nra;

	// Create and configure three fine sun sensors
	FssPrototype fss1(1,11,12);
	FssPrototype fss2(2,11,12);
	FssPrototype fss3(3,11,12);

	Real fss_x, fss_y;
	AocsTime fss_x_tag, fss_y_tag;
	DataItemWrite fss_x_diw(&fss_x_tag, &fss_x);
	DataItemWrite fss_y_diw(&fss_y_tag, &fss_y);

	RawBusDataConverterIn convIn(2,0.1);

	fss1.setAocsUnitHardware(&macsTcCont);
	fss1.setFunctionalInConverter(&convIn);
	fss1.setFunctionalInLink(fss_x_diw,0);
	fss1.setFunctionalInLink(fss_y_diw,1);
	fss1.setHwFailureRecoveryAction(&nra);

	fss2.setAocsUnitHardware(&macsTcCont);
	fss2.setFunctionalInConverter(&convIn);
	fss2.setFunctionalInLink(fss_x_diw,0);
	fss2.setFunctionalInLink(fss_y_diw,1);
	fss2.setHwFailureRecoveryAction(&nra);

	fss3.setAocsUnitHardware(&macsTcCont);
	fss3.setFunctionalInConverter(&convIn);
	fss3.setFunctionalInLink(fss_x_diw,0);
	fss3.setFunctionalInLink(fss_y_diw,1);
	fss3.setHwFailureRecoveryAction(&nra);

	// Create and configure two basic unit reconfigurers
	BasicUnitReconfigurer bur1(3,1,0);
	BasicUnitReconfigurer bur2(3,1,1);

	bur1.setReconfigurationErrorRecoveryAction(&nra);
	bur1.setUnit(&fss1,0);
	bur1.setUnit(&fss2,1);
	bur1.setUnit(&fss3,2);

	bur2.setReconfigurationErrorRecoveryAction(&nra);
	bur2.setUnit(&fss1,0);
	bur2.setUnit(&fss2,1);
	bur2.setUnit(&fss3,2);

	// Force a new configurations in basic reconfigurer 1
	bur1.setConfiguration(1);
	if (bur1.getConfigurationState().getMemoryPad() != 0x00060001)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 3 failed");
		return;
	}

	// Resetting the basic reconfigurer should result in default
	// conditions being reestablished
	bur1.reset();
	if (bur1.getConfigurationState().getMemoryPad() != 0x00070000)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 3 failed");
		return;
	}

	// Force new configurations in the two basic reconfigurer
	bur1.setConfiguration(1);
	if (bur1.getConfigurationState().getMemoryPad() != 0x00060001)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 3 failed");
		return;
	}
	bur2.setConfiguration(2);
	if (bur2.getConfigurationState().getMemoryPad() != 0x00050002)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 3 failed");
		return;
	}

	// Force a system reset and check that the configurations of the
	// basic reconfigurers have not been affected
	AocsObject::getSystemManager()->systemReset();
	if (bur1.getConfigurationState().getMemoryPad() != 0x00060001)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 3 failed");
		return;
	}
	if (bur2.getConfigurationState().getMemoryPad() != 0x00050002)
	{	setTestResult(TEST_FAILURE, "Reconfiguration test 3 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}



