// RwSet.cpp
//
// Version	1.0
// Date		06.07.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include framework header files
#include "../ReconfigurationManagement/RwSet.h"
#include "../ReconfigurationManagement/ReconfigurerHelper.h"
#include "../AocsUnit/AocsUnit.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../GeneralInclude/BasicTypes.h"
#include "../GeneralInclude/Constants.h"
#include "../AocsEvent/EventTypeId.h"	
#include "../GeneralUtilities/MathUtilities.h"
#include "../AocsData/AocsData.h"

// -------------------------------------------------------------------
// Define static variables
int RwSet::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------

RwSet::RwSet(int nOfMon, Real rwMat[3][4], Real ri)
{
	setClassId(ID_RWSET);				

	tmImageLength[shortTm]  = 0;		// Not implemented in fw prototype
	tmImageLength[normalTm] = 0;	
	tmImageLength[longTm]   = tmImageLength[normalTm];
	tmImageLength[debugTm]  = tmImageLength[longTm];

	helper = new ReconfigurerHelper(4,nOfMon,3,this);

	rwInertia = ri;

	for (int i=0; i<3; i++)
		for (int j=0; j<4; j++)
			rwMatrix[i][j] = rwMat[i][j];

	// Compute matrices mapping sc torques to rw torques
	Real temp[3][3];
	for (i=0; i<3; i++)
	{	temp[i][0]=rwMatrix[i][1];
		temp[i][1]=rwMatrix[i][2];
		temp[i][2]=rwMatrix[i][3];
	}
	invMat(temp,rwMatrixInv[0],3);

	for (i=0; i<3; i++)
	{	temp[i][0]=rwMatrix[i][0];
		temp[i][1]=rwMatrix[i][2];
		temp[i][2]=rwMatrix[i][3];
	}
	invMat(temp,rwMatrixInv[1],3);

	for (i=0; i<3; i++)
	{	temp[i][0]=rwMatrix[i][0];
		temp[i][1]=rwMatrix[i][1];
		temp[i][2]=rwMatrix[i][3];
	}
	invMat(temp,rwMatrixInv[2],3);
	
	for (i=0; i<3; i++)
	{	temp[i][0]=rwMatrix[i][0];
		temp[i][1]=rwMatrix[i][1];
		temp[i][2]=rwMatrix[i][2];
	}
	invMat(temp,rwMatrixInv[3],3);

	// All configurations have the same rank
	for (i=0; i<4; i++)
		helper->rankConfiguration(i,1);
	
}

// -------------------------------------------------------------------

void RwSet::localResetConfiguration()
{
	helper->resetConfiguration();
	for (int i=0; i<getReconfigurationOrder(); i++)
		if (rw[i]!=NULL)
			rw[i]->resetConfiguration();

	DataItemRead dr((Real*)NULL);
	DataItemWrite dw((AocsTime*)NULL,(Real*)NULL);

	// Remove the links to all external buffers,
	for (i=0; i<3; i++)
		scAngMomenta[i]=dw;

	for (i=0; i<3; i++)
		scTorque[i]=dw;

	for (i=0; i<4; i++)
		rwTorque[i]=dw;

	for (i=0; i<4; i++)
		rwSpeed[i]=dr;
}

// -------------------------------------------------------------------

void RwSet::resetConfiguration()
{
	AocsObject::resetConfiguration();
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool RwSet::isConfigured()
{
	if (!AocsObject::isConfigured())
		return false;

	if (!helper->isConfigured())
		return false;

	for (int i=0; i<getReconfigurationOrder(); i++)
		if (rw[i]==NULL)
			return false;

	for (i=0; i<getReconfigurationOrder(); i++)
		if (!rw[i]->isConfigured())
			return false;

	// Check that all external buffers are configured
	for (i=0; i<3; i++)
		if (!scAngMomenta[i].isDataItemWriteConfigured())
			return false;

	for (i=0; i<3; i++)
		if (!scTorque[i].isDataItemReadConfigured())
			return false;

	for (i=0; i<4; i++)
		if (!rwTorque[i].isDataItemWriteConfigured())
			return false;

	for (i=0; i<4; i++)
		if (!rwSpeed[i].isDataItemReadConfigured())
			return false;

	return true;
}

// -------------------------------------------------------------------

void RwSet::localReset()
{
	helper->reset();
	for (int i=0; i<getReconfigurationOrder(); i++)
		if (rw[i]!=NULL)
			rw[i]->reset();
}

// -------------------------------------------------------------------

void RwSet::reset()
{
	AocsObject::reset();
	localReset();
}

// -------------------------------------------------------------------

void RwSet::setConfiguration(ConfigurationId n)
{
	if (n != getConfiguration())
	{	rw[getConfiguration()]->switchOn();
		rw[getConfiguration()]->initialize();
		helper->setConfiguration(n);
		rw[getConfiguration()]->switchOff();
	}
}

// -------------------------------------------------------------------
// This implementation generates a failure event if the reconfiguration
// is not possible because there are no more free configurations 
// This implementation assumes that units that are configured in have 
// to be switched on and initialized whereas units that are configured
// out are switched off

void RwSet::reconfigure()
{
	if (canReconfigure())
	{	rw[getConfiguration()]->switchOn();
		rw[getConfiguration()]->initialize();
		helper->reconfigure();
		rw[getConfiguration()]->switchOff();
	}
	else
		reportFailure(this,
						NO_MORE_CONFIGURATION,
						  this,
						    helper->getReconfigurationErrorRecoveryAction());
}

// -------------------------------------------------------------------

void RwSet::setRw(AocsUnit* u, int n)
{
	if ( (n<0) || (n>=4) )
		reportConfigurationError(this,ILLEGAL_CONFIGURATION,this);

	rw[n] = u;
	rwSpeed[n] = u->getFunctionalInLink(0);
	rwTorque[n] = u->getFunctionalOutLink(0);
}

// -------------------------------------------------------------------

void RwSet::acquireFunctionalData()
{
	for (int i=0; i<4; i++)
		if (i!=getConfiguration())
			rw[i]->acquireFunctionalData();
}

// -------------------------------------------------------------------

void RwSet::sendFunctionalData()
{
	for (int i=0; i<4; i++)
		if (i!=getConfiguration())
			rw[i]->sendFunctionalData();
}

// -------------------------------------------------------------------

bool RwSet::isAcquireTransactionFinished()
{
	for (int i=0; i<4; i++)
		if (i!=getConfiguration())
			if (!rw[i]->isAcquireTransactionFinished())
				return false;

	return true;
}

// -------------------------------------------------------------------

bool RwSet::isSendTransactionFinished()
{
	for (int i=0; i<4; i++)
		if (i!=getConfiguration())
			if (!rw[i]->isSendTransactionFinished())
				return false;
	return true;
}

// -------------------------------------------------------------------

void RwSet::refreshFunctionalIn()
{
	for (int i=0; i<4; i++)
		if (i!=getConfiguration())
			rw[i]->refreshFunctionalIn();

	// Compute the wheel momenta in sc reference frame 
	Real rwSpeedTemp[4];

	for (i=0; i<4; i++)
		if (i!=getConfiguration())
			rwSpeedTemp[i] = rwSpeed[i].get();
		else
			rwSpeedTemp[i] = 0;

	for (i=0; i<3; i++)
	{	Real accumulator = 0;
		for (int j=0; j<4; j++)
		{	accumulator += rwMatrix[i][j]*rwSpeedTemp[j];
			scAngMomenta[i].set(accumulator*rwInertia);
		}
	}
}

// -------------------------------------------------------------------

void RwSet::refreshFunctionalOut()
{
	// Compute the torques on the reaction wheel axes
	Real rwTorqueTemp[3];
	for (int i=0; i<3; i++)
	{	rwTorqueTemp[i] = 0;
		for (int j=0; j<3; j++)
			rwTorqueTemp[i] += rwMatrixInv[getConfiguration()][i][j]*
								scTorque[j].get();
	}

	int j = 0;
	for (i=0; i<4; i++)
		if (i!=getConfiguration())
		{	rwTorque[i].set(rwTorqueTemp[j]);
			j++;
		}

	for (i=0; i<4; i++)
		if (i!=getConfiguration())
			rw[i]->refreshFunctionalOut();
}

// -------------------------------------------------------------------

void RwSet::synchronizeFunctional()
{
	for (int i=0; i<4; i++)
		if (i!=getConfiguration())
			rw[i]->synchronizeFunctional();
}

// -------------------------------------------------------------------

bool RwSet::isSynchronizeFunctionalFinished()
{
	for (int i=0; i<4; i++)
		if (i!=getConfiguration())
			if (!rw[i]->isSynchronizeFunctionalFinished())
				return false;

	return true;
}

// -------------------------------------------------------------------

void RwSet::setFunctionalInLink(DataItemWrite d, int i)
{
	if ( (i<0) || (i>2) )
	{	reportConfigurationError(this,ILLEGAL_FC_DEST_BUF,this);
		return;
	}

	scAngMomenta[i] = d;
}

// -------------------------------------------------------------------

void RwSet::setFunctionalInLink(AocsData* d)
{
	if ( d->getNumberOfItems()!=3 )
	{	reportConfigurationError(this,ILLEGAL_FC_DEST_BUF,this);
		return;
	}

	for (int i=0; i<3; i++)
		setFunctionalInLink(d->getDataItemWrite(i),i);
}

// -------------------------------------------------------------------

DataItemWrite RwSet::getFunctionalInLink(int i)
{
	if ( (i<0) || (i>3) )
	{	reportConfigurationError(this,ILLEGAL_FC_DEST_BUF,this);
		return DataItemWrite((AocsTime*)NULL,(Real*)NULL);
	}

	return scAngMomenta[i];
}

// -------------------------------------------------------------------

void RwSet::setFunctionalOutLink(DataItemWrite d, int i)
{
	if ( (i<0) || (i>2) )
	{	reportConfigurationError(this,ILLEGAL_FC_SOURCE_BUF,this);
		return;
	}

	scTorque[i] = d;
}

// -------------------------------------------------------------------

void RwSet::setFunctionalOutLink(AocsData* d)
{
	if ( d->getNumberOfItems()!=3 )
	{	reportConfigurationError(this,ILLEGAL_FC_SOURCE_BUF,this);
		return;
	}

	for (int i=0; i<3; i++)
		setFunctionalOutLink(d->getDataItemWrite(i),i);
}

// -------------------------------------------------------------------

DataItemWrite RwSet::getFunctionalOutLink(int i)
{
	if ( (i<0) || (i>2) )
	{	reportConfigurationError(this,ILLEGAL_FC_SOURCE_BUF,this);
		return DataItemWrite((AocsTime*)NULL,(Real*)NULL);
	}

	return scTorque[i];
}

// -------------------------------------------------------------------

int RwSet::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]
								+AocsObject::getTelemetryImageLength());
}

// -------------------------------------------------------------------

void RwSet::writeToTelemetry(TelemetryStream* tmStream)
{
	AocsObject::writeToTelemetry(tmStream);		// write super object's state

	// TBD : not implemented in framework prototype
}
