// ControlChannelSuperBlock.cpp
//
// Version	1.0
// Date		20.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include required for use of sqrt function
#include <stdio.h>

// Include framework header files
#include "../SequentialDataProcessing/ControlChannelSuperBlock.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------
// Default constructor.

ControlChannelSuperBlock::ControlChannelSuperBlock(int nInputs, int nOutputs)				
{
	setClassId(ID_CONTROLCHANNELSUPERBLOCK);	// assign class identifier

	setInputOrder(nInputs);
	inputLink = new IoLink[getInputOrder()];
	ccLink = new AbstractControlChannel*[getInputOrder()];
	
	setOutputOrder(nOutputs);
	outputLink = new IoLink[getOutputOrder()];

	localResetConfiguration();
}

// -------------------------------------------------------------------

void ControlChannelSuperBlock::localResetConfiguration()
{
	for (int i=0; i<getInputOrder(); i++)
	{	ccLink[i]=NULL;
		inputLink[i].cc=NULL;
		inputLink[i].n=0;
	}

	for (i=0; i<getOutputOrder(); i++)
	{	outputLink[i].cc=NULL;
		outputLink[i].n=0;
	}
}

// -------------------------------------------------------------------

void ControlChannelSuperBlock::resetConfiguration()
{
	AbstractControlChannel::resetConfiguration();		// reset super object's configuration
	localResetConfiguration();
}

// -------------------------------------------------------------------

bool ControlChannelSuperBlock::isConfigured()
{
	// Check that the super object is configured
	if (!AocsObject::isConfigured())
		return false;

	// Configured super blocks must have more than 0 outputs
	if (getOutputOrder()==0)		// NB: Blocks can have zero inputs!
		return false;

	// Check that the inputs are connected
	for (int i=0; i<getInputOrder(); i++)
	{	if ( inputLink[i].cc == NULL )
			return false;
	}

	// Check that the outputs are connected
	for (i=0; i<getOutputOrder(); i++)
	{	if ( outputLink[i].cc == NULL )
			return false;
	}

	// Check that all the control channels embedded in the super block are
	// configured. Use the current time as a token for the propagation request.
	int token=(int)getTime();
	for (i=0; i<getOutputOrder(); i++)
	{	if (outputLink[i].cc->getPropagationToken() != token)
		{	if ( !outputLink[i].cc->isConfigured(channelPropagation,token) )
				return false;	
			outputLink[i].cc->setPropagationToken(token);
		}
	}

	return true;
}

// -------------------------------------------------------------------

void ControlChannelSuperBlock::reset()
{
	AbstractControlChannel::reset();		// reset super object

	// Reset all the control channels embedded in the super block.
	// Use the current time as a token for the propagation request.
	int token=(int)getTime();
	for (int i=0; i<getOutputOrder(); i++)
	{	if (outputLink[i].cc->getPropagationToken() != token)
		{	outputLink[i].cc->reset(channelPropagation,token);
			outputLink[i].cc->setPropagationToken(token);
		}
	}
}

// -------------------------------------------------------------------

void ControlChannelSuperBlock::propagate(AocsTime t)
{
	if (t<lastPropagatedTime)
	{	reportFailure(this, ILLEGAL_PROPAGATION_TIME, this, getIllegalTimeRecoveryAction());
		return;
	}

	for (int i=0; i<getOutputOrder(); i++)
		outputLink[i].cc->propagate(t);
	
	lastPropagatedTime = t;
}

// -------------------------------------------------------------------

void ControlChannelSuperBlock::linkInput(AbstractControlChannel* c, int i, int j)
{
	if ( j >= getInputOrder() )
	{	reportConfigurationError(this, NON_EXISTENT_INPUT, this);
		return;
	}

	if ( inputLink[j].cc == NULL )
	{	reportConfigurationError(this, NON_CONFIGURED_INPUT, this);
		return;
	}

	inputLink[j].cc->linkInput(c,i,inputLink[j].n);
	ccLink[j]=c;
}

// -------------------------------------------------------------------

void ControlChannelSuperBlock::linkInput(DataItemRead d, int j)
{
	if ( j >= getInputOrder() )
	{	reportConfigurationError(this, NON_EXISTENT_INPUT, this);
		return;
	}

	if ( inputLink[j].cc == NULL )
	{	reportConfigurationError(this, NON_CONFIGURED_INPUT, this);
		return;
	}

	inputLink[j].cc->linkInput(d, inputLink[j].n);
}

// -------------------------------------------------------------------

DataItemRead ControlChannelSuperBlock::getOutput(int j)
{ 
	return outputLink[j].cc->getOutput(outputLink[j].n); 
}

// -------------------------------------------------------------------

void ControlChannelSuperBlock::linkInputInternal(AbstractControlChannel* c, int i, int j)
{
	inputLink[j].cc = c;
	inputLink[j].n = i;
}

// -------------------------------------------------------------------

void ControlChannelSuperBlock::linkOutputInternal(AbstractControlChannel* c, int i, int j)
{
	outputLink[j].cc = c;
	outputLink[j].n = i;
}

// -------------------------------------------------------------------

void ControlChannelSuperBlock::release()
{
	// not implemented in framework prototype
}

// -------------------------------------------------------------------

void ControlChannelSuperBlock::hold()
{
	// not implemented in framework prototype
}
