// TestCaseControlChannel_7.cpp
//
// Version	1.0
// Date		27.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseControlChannel_7.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../BasicObjects/AocsObject.h"
#include "../../AocsData/Scalar.h"
#include "../../AocsData/TwoEulerAngles.h"
#include "../../SequentialDataProcessing/I_Block.h"
#include "../../SequentialDataProcessing/P_Block.h"
#include "../../SequentialDataProcessing/TwoByTwoMatrixBlock.h"
#include "../../SequentialDataProcessing/ControlChannelSuperBlock.h"

// -------------------------------------------------------------------
TestCaseControlChannel_7::TestCaseControlChannel_7():
	TestCaseGenericSetUp(ID_ABSTRACTCONTROLCHANNELBLOCK*10+7,
									"TestControlChannel_7") {
		return;
}

// -------------------------------------------------------------------

void TestCaseControlChannel_7::runTestCase() {

	// Advance the clock (necessary because the time is used as propagation token)
	aocsClock.run(0);

	// Create a proportional block, an integral block and one 2x2 matrix block
	P_Block p1;
	I_Block i1;
	TwoByTwoMatrixBlock matrixBlock;

	// Set the gains of the P and I blocks
	Real p1Gain=2.0;
	Real i1Gain=4.0;
	p1.setGain(p1Gain);
	i1.setGain(i1Gain);

	// Set the elements of the 2x2 matrix 
	matrixBlock.setMatrixElement(0,0,1);
	matrixBlock.setMatrixElement(0,1,0);
	matrixBlock.setMatrixElement(1,0,0);
	matrixBlock.setMatrixElement(1,1,2);

	// Create the two super blocks
	ControlChannelSuperBlock superBlock1(1,1);
	ControlChannelSuperBlock superBlock2(1,2);

	// Embed the I block in the first super block
	superBlock1.linkInputInternal(&i1,0,0);
	superBlock1.linkOutputInternal(&i1,0,0);
	i1.setEnclosingControlChannel(&superBlock1);

	// Connect the first super block to the inputs of the 2x2 matrix block
	matrixBlock.linkInput(&superBlock1,0,0);
	matrixBlock.linkInput(&superBlock1,0,1);

	// Link the input of the first super block to the output of P block
	superBlock1.linkInput(&p1,0,0);

	// Embed and link the blocks in the second superblock
	superBlock2.linkInputInternal(&p1,0,0);
	superBlock2.linkOutputInternal(&matrixBlock,0,0);
	superBlock2.linkOutputInternal(&matrixBlock,1,1);
	superBlock1.setEnclosingControlChannel(&superBlock2);
	p1.setEnclosingControlChannel(&superBlock2);
	matrixBlock.setEnclosingControlChannel(&superBlock2);

	// Create a dummy data to serve as input for the super block
	Scalar inputSource;

	// Create a dummy object to act as owner of the data items in the input source
	AocsObject dataOwner;

	// Set a value for the input source
	Real input = 5;
	inputSource.getDataItemWrite(0).set(input);

	// Link the external input of the second super block
	superBlock2.linkInput(inputSource.getDataItemRead(0),0);

	// Check that the outer superblock is correctly configured
	if ( !superBlock2.isConfigured() )
	{	setTestResult(TEST_FAILURE, "Superblock is not correctly configured");
		return;
	}

	// Check that the super block outputs are again zero
	if (superBlock2.getOutput(0).get()!=0)
	{	setTestResult(TEST_FAILURE, "Superblock output is wrong");
		return;
	}
	if (superBlock2.getOutput(1).get()!=0)
	{	setTestResult(TEST_FAILURE, "Superblock output is wrong");
		return;
	}

	// Advance the clock and propagate the superblock to the new time
	AocsTime t0=aocsClock.getTime();
	aocsClock.run(0);
	AocsTime t1=aocsClock.getTime();
	superBlock2.propagate( aocsClock.getTime() );

	// Check that the super block outputs have the correct values
	if ( superBlock2.getOutput(0).get() != (input*(t1-t0)*0.5*p1Gain*i1Gain) )
	{	setTestResult(TEST_FAILURE, "Superblock output is wrong");
		return;
	}
	if ( superBlock2.getOutput(1).get() != (2*input*(t1-t0)*0.5*p1Gain*i1Gain) )
	{	setTestResult(TEST_FAILURE, "Superblock output is wrong");
		return;
	}

	// Reset the superblock
	superBlock2.reset();

	// Check that the super block outputs are again zero
	if (superBlock2.getOutput(0).get()!=0)
	{	setTestResult(TEST_FAILURE, "Superblock output is wrong");
		return;
	}
	if (superBlock2.getOutput(1).get()!=0)
	{	setTestResult(TEST_FAILURE, "Superblock output is wrong");
		return;
	}

	// Check that no failure or configuration events have been created
	if ( failureEvtRep.getCounter() != failureEvtN )
	{	setTestResult(TEST_FAILURE, "Failure event was generated");
		return;
	}
	if ( configEvtRep.getCounter() != configEvtN )
	{	setTestResult(TEST_FAILURE, "Configuration event was generated");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
