// TorqueToThrusterPrototype.h
//
// Version	1.0
// Date		06.06.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Control block to perform the conversion from torque requests around
// the spacecraft axes to the on- and delay-times for the set of six
// thrusters of the AOCS prototype.
//
// Outputs 1 to 6 represent the delay-times. Outputs 7 to 12 represent
// the on-times.

// The nominal torque level on each thruster is 'thuTorque'. The 
// nominal torques in spacecraft axes generated by the firing of each
// thruster are as shown in the table below:
//
//				X Axis		Y Axis		Z Axis
//		THU_1	+thuTorque		0			0
//		THU_2	-thuTorque		0			0
//		THU_3		0		+thuTorque		0
//		THU_4		0		-thuTorque		0
//		THU_5		0			0		+thuTorque
//		THU_6		0			0		-thuTorque
//
// The thrusters are controlled in on-modulation.
//
// The thruster delay-times are set to zero.
//
// The on-time are computed as follows:
//
//		onTime = torqueRequest*period/thuTorque
//
// where 'torqueRequest' is the torque request, 'period' is the on-
// modulation period and 'thuTorque' is the nominal torque delivered
// by the thrusters. If the torque request is larger than the thruster
// torque, then the on-time is saturated and set equal to the 
// on-modulation period.
// 
// -------------------------------------------------------------------

#ifndef TorqueToThrusterPrototypeH
#define TorqueToThrusterPrototypeH

// -------------------------------------------------------------------

#include "../SequentialDataProcessing/ControlChannelBlock.h"
#include "../GeneralInclude/BasicTypes.h"

// -------------------------------------------------------------------

class TorqueToThrusterPrototype : public ControlChannelBlock {

	Real thuTorque;		// nominal thruster torque level
	Real period;		// on-modulation period

	virtual void propagationAlgorithm(AocsTime t);
	
public :

	TorqueToThrusterPrototype(Real thuTorque, Real period);
};

#endif
