// XmathUcbBlock.cpp
//
// Version	1.0
// Date		29.02.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

// Include framework header files
#include "../SequentialDataProcessing/XmathUcbBlock.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/Constants.h"
#include "../AocsEvent/EventTypeId.h"

// -------------------------------------------------------------------
// Define static variables
int XmathUcbBlock::tmImageLength[NTMFORMATS];

// -------------------------------------------------------------------
// Default constructor.

XmathUcbBlock::XmathUcbBlock(int i, int j, int k, int nR, int nI, 
							 AocsTime sT, UcbHookFunction uH)				
{
	setClassId(ID_XMATHUCBBLOCK);			

	allocateMemory(i,j,k);				// i inputs, j outputs, k states
	stateDer = new Real[k];				// where is this initialized?
	R_P = new Real[nR];
	I_P = new int[nI];

	n = i,
	m = j;
	p = k;
	
	ucbHook = uH;
	samplingTime = sT;

	iinfo[0] = 0;			// error flag is reset
	iinfo[1] = 1;			// init mode flag is set
	iinfo[2] = 0;			// state mode flag is reset 
	iinfo[3] = 0;			// output mode flag is reset

	rinfo[0] = getLastPropagatedTime();		// current time 
	rinfo[1] = (Real)sT;	// sampling time
	rinfo[2] = 0;			// skew time
	rinfo[3] = 0;			// always zero 

	tmImageLength[shortTm]  = 0;	
	tmImageLength[normalTm] = 0;	
	tmImageLength[longTm]   = getStateOrder()*sizeof(Real)+sizeof(AocsTime);
	tmImageLength[debugTm]  = tmImageLength[longTm]+getInputOrder()*sizeof(Real)+
									getOutputOrder()*sizeof(Real);

	localReset();					
	localResetConfiguration();					
}

// -------------------------------------------------------------------

void XmathUcbBlock::propagationAlgorithm(AocsTime t)
{
	// The propagationAlgorithm routine should update the output to 
	// time t. The UCB routine updates its output to time rinfo[0].
	rinfo[0] = t;

	// Call the UCB subroutine
	ucbHook(iinfo,rinfo,input,&n,state,stateDer,&p,output,&m,R_P,I_P);

	// The UCB routine writes the new state in array stateDer. In order
	// to be ready for the next activation, this state has to be copied
	// into array state 
	for (int i=0; i<getStateOrder(); i++)
		state[i]=stateDer[i];

	// Reset the init flag (this ensures that the UCB routine is called
	// in initialization mode only once after a reset operation) and set
	// the output update flag (for reasons that are unclear, it seems 
	// necessary to clear this flag when the UCB routine is called the 
	// first time)
	iinfo[1] = 0;
	iinfo[3] = 1;

	// Check if the UCB routine reported an error
	if (iinfo[0])
		reportFailure(this,XMATH_UCB_ERROR,this,getUcbErrorRecoveryAction());
}
	
// -------------------------------------------------------------------

void XmathUcbBlock::localReset()
{
	// UCB mode is: no ERROR, INIT, OUTPUT update, STATES update
	iinfo[0] = 0;			// error flag is reset
	iinfo[1] = 1;			// init mode flag is set
	iinfo[2] = 1;			// state mode flag is set 
	iinfo[3] = 1;			// output mode flag is set

	// This call of the UCB routine should have no effect other than
	// initializing the %variables. 
	ucbHook(iinfo,rinfo,input,&n,state,stateDer,&p,output,&m,R_P,I_P);

	// When the UCB routine is called the first time by the propagation
	// method, the following should hold: iinfo[1]=1 and iinfo[3]=0. 
	// These settings should be reversed after the first call.
	iinfo[3] = 0;

	// Check if the UCB routine reported an error
	if (iinfo[0])
		reportFailure(this,XMATH_UCB_ERROR,this,getUcbErrorRecoveryAction());
}

// -------------------------------------------------------------------

void XmathUcbBlock::localResetConfiguration()
{
	recoveryAction = NULL;
	rinfo[1]=samplingTime; 

	// UCB mode is: no ERROR, INIT, OUTPUT update, STATES update
	iinfo[0] = 0;			// error flag is reset
	iinfo[1] = 1;			// init mode flag is set
	iinfo[2] = 1;			// state mode flag is set 
	iinfo[3] = 1;			// output mode flag is set

	// This call of the UCB routine should have no effect other than
	// initializing the %variables. 
	ucbHook(iinfo,rinfo,input,&n,state,stateDer,&p,output,&m,R_P,I_P);

	// When the UCB routine is called the first time by the propagation
	// method, the following should hold: iinfo[1]=1 and iinfo[3]=0. 
	// These settings should be reversed after the first call.
	iinfo[3] = 0;

	// Check if the UCB routine reported an error
	if (iinfo[0])
		reportFailure(this,XMATH_UCB_ERROR,this,getUcbErrorRecoveryAction());
}

// -------------------------------------------------------------------

void XmathUcbBlock::reset()
{
	ControlChannelBlock::reset();	// reset super object
	localReset();
}

// -------------------------------------------------------------------

void XmathUcbBlock::resetConfiguration()
{
	ControlChannelBlock::resetConfiguration();	// reset super object
	localResetConfiguration();
}

// -------------------------------------------------------------------

void XmathUcbBlock::writeToTelemetry(TelemetryStream* stream)
{
	ControlChannelBlock::writeToTelemetry(stream);		// write super object's state

	switch(getTelemetryFormat()) 
	{
		case shortTm:
			break;
		case normalTm:
			// TBW
			break;
		case longTm:
			// TBW
			break;
		case debugTm:
			// TBW
			break;
	}
}

// -------------------------------------------------------------------

int XmathUcbBlock::getTelemetryImageLength()
{
	return (tmImageLength[getTelemetryFormat()]+
				ControlChannelBlock::getTelemetryImageLength());
}
