// SystemManager.cpp
//
// Version	1.1
// Date		13.01.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../SystemManagement/SystemManager.h"
#include "../GeneralInclude/ClassId.h"
#include "../GeneralInclude/ForwardDeclarations.h"
#include "../ReconfigurationManagement/ConfigurationState.h"
#include "../ReconfigurationManagement/Reconfigurable.h"
#include "../AocsEvent/EventTypeId.h"
#include "../AocsEvent/SystemEventRepository.h"

// -------------------------------------------------------------------

SystemManager::SystemManager()
{
	setClassId(ID_SYSTEMMANAGER);			

	counter = 0;
	counterReconfigurable = 0;
	maxNumberOfObjects = 0;
	maxNumberOfReconfigurable = 0;
	systemEventRepository = NULL;

	enableConfigStateUpdate = true;
}

// -------------------------------------------------------------------

SystemManager::SystemManager(int n, int m, SystemEventRepository* systemEvtRep)
{
	setClassId(ID_SYSTEMMANAGER);			

	counter = 0;
	counterReconfigurable = 0;
	maxNumberOfObjects = n;
	maxNumberOfReconfigurable = m;
	systemEventRepository = systemEvtRep;

	enableConfigStateUpdate = true;

	list.allocateMemory(maxNumberOfObjects);
	listReconfigurable.allocateMemory(maxNumberOfObjects);
	configurationState = new ConfigurationState[maxNumberOfReconfigurable];
}

// -------------------------------------------------------------------

SystemManager::initialize(int n, int m, SystemEventRepository* systemEvtRep)
{
	counter = 0;
	counterReconfigurable = 0;
	maxNumberOfObjects = n;
	maxNumberOfReconfigurable = m;
	systemEventRepository = systemEvtRep;

	enableConfigStateUpdate = true;

	list.allocateMemory(maxNumberOfObjects);
	listReconfigurable.allocateMemory(maxNumberOfObjects);
	configurationState = new ConfigurationState[maxNumberOfReconfigurable];
}

// -------------------------------------------------------------------

void SystemManager::add(AocsObject* object)
{
	counter++;
	if (counter>maxNumberOfObjects)
	{	counter--;
		systemEventRepository->create(TOO_MANY_SYSTEM_OBJECTS);
	}
	else
		list.addObject(object);
}

// -------------------------------------------------------------------

void SystemManager::remove(AocsObject* object)
{
	counter--;
	list.removeObject(object);
}

// -------------------------------------------------------------------

void SystemManager::addReconfigurable(Reconfigurable* r)
{
	counterReconfigurable++;
	if (counterReconfigurable>maxNumberOfReconfigurable)
	{	counter--;
		systemEventRepository->create(TOO_MANY_RECONFIGURABLE);
	}
	else
	{	int i=listReconfigurable.addObject(r);
		configurationState[i] = r->getConfigurationState();
	}
}

// -------------------------------------------------------------------

void SystemManager::removeReconfigurable(Reconfigurable* r)
{
	counterReconfigurable--;
	int i = listReconfigurable.removeObject(r);
}

// -------------------------------------------------------------------

void SystemManager::systemReset()
{
	// Disable updates of configuration state changes in 
	// reconfigurable objects (NB the act of resetting a reconfigurable
	// object may cause state changes that should NOT be recorded by the
	// system manager)
	enableConfigStateUpdate = false;

	for (AocsObject* obj=list.first(); !list.isLast(); obj=list.next())
		obj->reset();

	// Reenable updates of configuration state changes in 
	// reconfigurable objects
	enableConfigStateUpdate = true;
	
	loadConfigurations();

	systemEventRepository->create(SYSTEM_RESET);
}

// -------------------------------------------------------------------

void SystemManager::loadConfigurations()
{
	for (Reconfigurable* r=listReconfigurable.first(); 
							!listReconfigurable.isLast(); 
								r=listReconfigurable.next())
		r->setConfigurationState(configurationState[listReconfigurable.getPosition()]);
}	
	
// -------------------------------------------------------------------

bool SystemManager::isSystemConfigured()
{
	for (AocsObject* obj=list.first(); !list.isLast(); obj=list.next())
		if (!obj->isConfigured())
			return false;

	return true;
}

// -------------------------------------------------------------------

void SystemManager::configurationStateChange(Reconfigurable* caller)
{
	if (!enableConfigStateUpdate)
		return;

	for (Reconfigurable* r=listReconfigurable.first(); 
							!listReconfigurable.isLast(); 
								r=listReconfigurable.next())
		if (r == caller)
		{	configurationState[listReconfigurable.getPosition()] = 
											r->getConfigurationState();
			return;
		}
}


	