// Erc32DmaTelecommandLoader.cpp
//
// Version	1.0
// Date		04.09.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "../GeneralInclude/Include.h"
#include "../TelemetryManagement/Telemeterable.h"
#include "../BasicObjects/Include.h"
#include "../OperatingSystemObjects/Runnable.h"
#include "../FailureDetection/ConsistencyCheckable.h"
#include "../ObjectList/Include.h"
#include "../AocsData/DataItemRead.h"
#include "../ObjectMonitoring/Include.h"
#include "../AocsEvent/Include.h"
#include "../ModeManagement/Include.h"
#include "../TelecommandManagement/Include.h"

// -------------------------------------------------------------------

Erc32DmaTelecommandLoader::Erc32DmaTelecommandLoader(int* tcBufferStart):
								DmaTelecommandLoader(tcBufferStart)
{
	setClassId(ID_ERC32DMATELECOMMANDLOADER);
	
	instanceIdCounter = TC_INSTANCE_ID_BASE;
}

// -------------------------------------------------------------------
// When this method is called, ptr should be pointing to the first 
// byte in a TC block.

Telecommand* Erc32DmaTelecommandLoader::makeSimpleTelecommand()
{
	int lengthDataSection = tcBuffer[ptr];
	int lengthVTableSection = tcBuffer[ptr+1];
	ptr +=2;

	if (lengthVTableSection==0)
		return makeDataTelecommand(lengthDataSection);
	else
		return makeCodeTelecommand(lengthDataSection, lengthVTableSection);
}

// -------------------------------------------------------------------
// Construct a simple TC whose code is already present in memory.
// When this method is called, ptr should be pointing to the first 
// byte in the data section of a TC block.

Telecommand* Erc32DmaTelecommandLoader::makeDataTelecommand(int lengthDataSection)
{
	// Create the instance ID for the new telecommand
	InstanceId instanceId = instanceIdCounter++;			

	char* tcStart = (char*)newData(instanceId);

	tcStart[12] = tcBuffer[ptr];		// copy TC VTable pointer
	tcStart[13] = tcBuffer[ptr+1];		
	tcStart[14] = tcBuffer[ptr+2];		
	tcStart[15] = tcBuffer[ptr+3];		

	char* instanceIdPtr = (char*)&instanceId;
	tcStart[0] = instanceIdPtr[0];	// copy instance ID
	tcStart[1] = instanceIdPtr[1];

	tcStart[2] = tcBuffer[ptr+8];	// copy class ID		
	tcStart[3] = 0;					// unused byte		

	tcStart[4] = 0;					// unused byte	
	tcStart[5] = 0;					// unused byte
	tcStart[6] = 0;					// unused byte
	tcStart[7] = 1;					// initialize 'enabled' to 'true'

	tcStart[8] = tcBuffer[ptr+4];	// copy time tag
	tcStart[9] = tcBuffer[ptr+5];		
	tcStart[10] = tcBuffer[ptr+6];		
	tcStart[11] = tcBuffer[ptr+7];		

	// Copy remaining data
	for (int i=0; i<(lengthDataSection-9); i++)
		tcStart[16+i] = tcBuffer[ptr+9+i];

	// Update the tcBuffer pointer
	ptr += lengthDataSection;

	return (Telecommand*)tcStart;
}

// -------------------------------------------------------------------
// When this method is called, ptr should be pointing to the first 
// byte in a TC block.
// If 'isFirst' is 'true', then this TC block contains the first TC in 
// a TC transaction and the method returns the time tag of the TC 
// transaction in 'timeTag'.
// If 'isFirst' is 'false', then 'timeTag' is an input parameter and
// it contains the time tag to be assigned to the TC.

TransactionTelecommand* Erc32DmaTelecommandLoader::makeTransactionTelecommand(
							bool isFirst, TelecommandId tcId, AocsTime* timeTag)
{
	int lengthDataSection = tcBuffer[ptr];
	int lengthVTableSection = tcBuffer[ptr+1];
	ptr +=2;

	if (lengthVTableSection==0)
		return makeDataTransactionTelecommand(
					lengthDataSection, isFirst, tcId, timeTag);
	else
		return makeCodeTransactionTelecommand(
			lengthDataSection, lengthVTableSection, isFirst, tcId, timeTag);
}

// -------------------------------------------------------------------
// Construct a transaction TC whose code is already present in memory.
// When this method is called, ptr should be pointing to the first 
// byte in the data section of a TC block.

TransactionTelecommand* Erc32DmaTelecommandLoader::makeDataTransactionTelecommand(
			int lengthDataSection, bool isFirst, TelecommandId tcId, AocsTime* timeTag)
{
	// Create the instance ID for the new telecommand
	InstanceId instanceId = instanceIdCounter++;			

	// Allocate memory for the new TC
	char* tcStart;
	if (isFirst)
		tcStart = (char*)newData((TelecommandId)instanceId);
	else
		tcStart = (char*)newData(tcId);

	tcStart[12] = tcBuffer[ptr];			// copy TC VTable pointer
	tcStart[13] = tcBuffer[ptr+1];		
	tcStart[14] = tcBuffer[ptr+2];		
	tcStart[15] = tcBuffer[ptr+3];		

	char* instanceIdPtr = (char*)&instanceId;
	tcStart[0] = instanceIdPtr[0];		// copy instance ID
	tcStart[1] = instanceIdPtr[1];

	tcStart[4] = 0;					// unused byte	
	tcStart[5] = 0;					// unused byte
	tcStart[6] = 0;					// unused byte
	tcStart[7] = 1;					// initialize 'enabled' to 'true'

	if (isFirst)
	{	
		tcStart[8] = tcBuffer[ptr+4];	// copy time tag
		tcStart[9] = tcBuffer[ptr+5];		
		tcStart[10] = tcBuffer[ptr+6];		
		tcStart[11] = tcBuffer[ptr+7];		
		AocsTime* timeTagDummy = (AocsTime*)&tcStart[8];
		*timeTag = *timeTagDummy;

		tcStart[2] = tcBuffer[ptr+8];	// copy class ID
		tcStart[3] = 0;					// unused byte		

		// Initialize 'next' and 'previous' fields to NULL
		tcStart[16] = 0;
		tcStart[17] = 0;
		tcStart[18] = 0;
		tcStart[19] = 0;
		tcStart[20] = 0;
		tcStart[21] = 0;
		tcStart[22] = 0;
		tcStart[23] = 0;

		// Copy remaining data 
		for (int i=0; i<(lengthDataSection-9); i++)
			tcStart[24+i] = tcBuffer[ptr+9+i];
	}
	else
	{
		char* timeTagChar = (char*)timeTag;
		tcStart[8] = timeTagChar[0];	// copy time tag
		tcStart[9] = timeTagChar[1];		
		tcStart[10] = timeTagChar[2];		
		tcStart[11] = timeTagChar[3];		

		tcStart[2] = tcBuffer[ptr+4];	// copy class ID		
		tcStart[3] = 0;					// unused byte		

		// Initialize 'next' and 'previous' fields to NULL
		tcStart[16] = 0;
		tcStart[17] = 0;
		tcStart[18] = 0;
		tcStart[19] = 0;
		tcStart[20] = 0;
		tcStart[21] = 0;
		tcStart[22] = 0;
		tcStart[23] = 0;

		// Copy remaining data
		for (int i=0; i<(lengthDataSection-5); i++)
			tcStart[24+i] = tcBuffer[ptr+5+i];
	}

	// Update the tcBuffer pointer
	ptr += lengthDataSection;

	return (TransactionTelecommand*)tcStart;
}

