// TelecommandManagementTestSuite.h 
//
// Version	1.0
// Date		05.08.00
// Author	A. Pasetti
//
// -------------------------------------------------------------------
// Test suite for telecommand management framelet
// -------------------------------------------------------------------

// Include system header files
#include <stdio.h>  

// Include framework header files
#include "../../GeneralInclude/Include.h"
#include "../../TelemetryManagement/Telemeterable.h"
#include "../../BasicObjects/Include.h"
#include "../../OperatingSystemObjects/Runnable.h"
#include "../../OperatingSystemObjects/AocsClock.h"
#include "../../TelemetryManagement/TelemetryStream.h"
#include "../../FailureDetection/ConsistencyCheckable.h"
#include "../../AocsEvent/Include.h"
#include "../../ObjectList/Include.h"
#include "../../AocsData/DataItemRead.h"		
#include "../../ObjectMonitoring/Include.h"
#include "../../AocsData/Include.h"
#include "../../AocsEvent/Include.h"
#include "../../ModeManagement/Include.h"
#include "../../FailureRecovery/Include.h"
#include "../../Xmath/Include.h"
#include "../../SequentialDataProcessing/Include.h"
#include "../../AocsUnit/Include.h"			
#include "../../ReconfigurationManagement/Include.h"
#include "../../SystemManagement/Include.h"
#include "../../TelecommandManagement/Include.h"
#include "../../ManoeuvreManagement/Include.h"

// Include test header files
#include "../../BasicObjects/DummyAocsObject.h"
#include "../../OperatingSystemObjects/DummyAocsClock.h"
#include "../../GeneralUtilities/TestUtilities.h"
#include "../TestTelecommandLoader.h"
#include "TelecommandManagementTestSuite.h"

// -------------------------------------------------------------------

void telecommandManagementTestMaster(FILE* out)
{
	// Create and configure the system manager and the system event repository
	SystemEventRepository systemEvtRep;
	systemEvtRep.setRepositorySize(4);	
	SystemManager systemManager(100,10,&systemEvtRep);

	// Create the event repositories 
	FailureEventRepository failureEvtRep;
	ConfigurationEventRepository configEvtRep;
	ReconfigurationEventRepository reconfigEvtRep;
	ModeEventRepository modeEvtRep;
	ChangeEventRepository changeEvtRep;
	TelecommandEventRepository tcEvtRep;
	ManoeuvreEventRepository manoeuvreEvtRep;

	// Allocate memory to event repositories
	failureEvtRep.setRepositorySize(4);	
	configEvtRep.setRepositorySize(4);	
	reconfigEvtRep.setRepositorySize(4);	
	tcEvtRep.setRepositorySize(4);	
	modeEvtRep.setRepositorySize(4);	
	changeEvtRep.setRepositorySize(4);	
	manoeuvreEvtRep.setRepositorySize(4);

	// Create dummy AOCS clock and advance it by two cycles
	DummyAocsClock aocsClock;
	aocsClock.run(0);
	aocsClock.run(0);

	// Configure the static part of the ModeManager class
	ModeManager::setChangeEventRepository(&changeEvtRep);
	ModeManager::setModeEventRepository(&modeEvtRep);

	// Configure the static part of the ReconfigurerHelper class
	ReconfigurerHelper::setChangeEventRepository(&changeEvtRep);
	ReconfigurerHelper::setReconfigurationEventRepository(&reconfigEvtRep);

	// Configure the static part of AocsObject 
	AocsObject::setAocsClock(&aocsClock);
	AocsObject::setFailureEventRepository(&failureEvtRep);		
	AocsObject::setConfigurationEventRepository(&configEvtRep);
	AocsObject::setSystemManager(&systemManager);
	NullRecoveryAction nullRecAct;
	AocsObject::setDestructionRecoveryAction(&nullRecAct);

	// Configure the static part of the TelecommandManager
	TelecommandManager::setTelecommandEventRepository(&tcEvtRep);

	// Configure the static part of the Manoeuvre
	Manoeuvre::setChangeEventRepository(&changeEvtRep);
	Manoeuvre::setManoeuvreEventRepository(&manoeuvreEvtRep);

	// Configure the static part of the ManoeuvreManager
	ManoeuvreManager::setManoeuvreEventRepository(&manoeuvreEvtRep);

	// Configure the static part of the DataItemWrite
	DataItemWrite::setAocsClock(&aocsClock);

	// Create auxiliary variables
	bool outcome;

	// Check the telecommand manager
	outcome=telecommandManagementTest_1(&aocsClock);
	printTestOutcome(out,"telecommandManagementTest_1",outcome);
	
	outcome=telecommandManagementTest_2();
	printTestOutcome(out,"telecommandManagementTest_2",outcome);

	outcome=telecommandManagementTest_3();
	printTestOutcome(out,"telecommandManagementTest_3",outcome);

	outcome=telecommandManagementTest_4(&aocsClock);
	printTestOutcome(out,"telecommandManagementTest_4",outcome);	

	outcome=telecommandManagementTest_5(&aocsClock);
	printTestOutcome(out,"telecommandManagementTest_5",outcome);	

	outcome=telecommandManagementTest_6(&aocsClock);
	printTestOutcome(out,"telecommandManagementTest_6",outcome);	

	outcome=telecommandManagementTest_7(&aocsClock);
	printTestOutcome(out,"telecommandManagementTest_7",outcome);	
}

// -------------------------------------------------------------------

bool telecommandManagementTest_1(DummyAocsClock* aocsClock)
{
	// Retrieve the number of failure, configuration and TC events
	int failureEvtN = AocsObject::getFailureEventRepository()->getCounter();
	int configEvtN = AocsObject::getConfigurationEventRepository()->getCounter();
	int tcEvtN = TelecommandManager::getTelecommandEventRepository()->getCounter();

	// Create the TC manager and check that it is not configured
	int ntc=3;

	TelecommandManager tcm(ntc);

	if (tcm.isConfigured())
		return false;

	// Configure the TC manager
	NullRecoveryAction nra;
	tcm.setTelecommandListFullRecoveryAction(&nra);
	tcm.setTelecommandStackErrorRecoveryAction(&nra);

	TestTelecommandLoader ttl;
	tcm.setTelecommandLoader(&ttl);

	if (!tcm.isConfigured())
		return false;

	// Create four dummy telecommands with different time tags
	Telecommand tc1(aocsClock->getTime());
	Telecommand tc2(aocsClock->getTime()+aocsClock->getClockPeriod());
	Telecommand tc3(aocsClock->getTime()+aocsClock->getClockPeriod());
	Telecommand tc4(aocsClock->getTime()+2*aocsClock->getClockPeriod());

	// Disable the second telecommand
	tc2.disable();

	// Code to explore layout of objects in memory
	char* memchar;
	int* memint;
	memchar = (char*)&tc2;
	memint = (int*)&tc2;

	// Load the first three telecommands
	tcm.loadTelecommand(&tc3);
	tcm.loadTelecommand(&tc2);
	tcm.loadTelecommand(&tc1);

	// Check that the telecommand count is correct
	if (tcm.getNumberOfTelecommands()!=3)
		return false;

	// Check that three tc events were created
	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+3)
		return false;
	if (tcm.getTelecommandEventRepository()->latestTelecommandEvent()->
			getEventType()!=TC_LOADED)
		return false;

	// Attempt to load the fourth telecommand. This should produce a 
	// failure event in the TC manager due to the TC list being 
	// already full
	tcm.loadTelecommand(&tc4);

	if (tcm.getNumberOfTelecommands()!=3)
		return false;

	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN+1)
		return false;
	if (AocsObject::getFailureEventRepository()->latestFailureEvent()->
			getEventType()!=TC_LIST_FULL)
		return false;

	// Run the TC manager. Only TC1 should be executed (it is due for
	// execution and it is enabled)
	tcm.run(0);

	if (tcm.getNumberOfTelecommands()!=2)
		return false;

	if (ttl.getReleaseCounter()!=1)
		return false;

	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+4)
		return false;
	if (tcm.getTelecommandEventRepository()->latestTelecommandEvent()->
			getEventType()!=TC_SUCCESS)
		return false;

	// Advance time by two cycles
	aocsClock->run(0);
	aocsClock->run(0);

	// Run the TC manager. Both TC2 and TC3 are due for execution but only
	// TC3 should be executed as TC2 is disabled
	tcm.run(0);

	if (tcm.getNumberOfTelecommands()!=0)
		return false;

	if (ttl.getReleaseCounter()!=3)
		return false;

	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+6)
		return false;

	// Before exiting, check failure and configuration error event numbers
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN+1)
		return false;
	if (AocsObject::getConfigurationEventRepository()->getCounter()!=configEvtN)
		return false;

	// If this point is reached, test was successful
	return true;
}

// -------------------------------------------------------------------
// Test telecommands to: change mode, perform reconfiguration, change
// telemetry format

bool telecommandManagementTest_2()
{
	// Retrieve the number of failure, configuration and TC events
	int failureEvtN = AocsObject::getFailureEventRepository()->getCounter();
	int configEvtN = AocsObject::getConfigurationEventRepository()->getCounter();
	int tcEvtN = TelecommandManager::getTelecommandEventRepository()->getCounter();

	// Create and configure the TC manager 
	TelecommandManager tcm(3);

	NullRecoveryAction nra;
	tcm.setTelecommandListFullRecoveryAction(&nra);
	tcm.setTelecommandStackErrorRecoveryAction(&nra);

	TestTelecommandLoader ttl;
	tcm.setTelecommandLoader(&ttl);

	if (!tcm.isConfigured())
		return false;

	// Create and configure a mode manager and a configuration manager 
	// to act as dummy objects upon which telecommands can operate
	int defaultConfig=0;

	AocsMissionModeManager mm(3,0,0);
	NullModeChangeAction mca;
	mm.setModeChangeAction(&mca);

	BasicUnitReconfigurer bur(2,0,defaultConfig);
	short dummy_1,dummy_2,dummy_3;
	MacsTcController macsTcController(&dummy_1,&dummy_2,&dummy_3);
	macsTcController.setMaxPolling(2);
	GyrPrototype gyr_1(0,0), gyr_2(0,0);
	gyr_1.setAocsUnitHardware(&macsTcController);
	gyr_2.setAocsUnitHardware(&macsTcController);
	bur.setUnit(&gyr_1,0);
	bur.setUnit(&gyr_2,1);

	// Create test telecommands 
	int targetMode=1;
	TelemetryFormat targetTmFormat=longTm;
	ModeChangeTelecommand mcTC(0,&mm,targetMode);
	ReconfigureTelecommand rTC(0,&bur);
	TelemetryFormatTelecommand tfTC(0,&mm,targetTmFormat);

	// Load telecommands in the TC manager
	tcm.loadTelecommand(&mcTC);
	tcm.loadTelecommand(&rTC);
	tcm.loadTelecommand(&tfTC);

	// Run the TC manager and check execution of TC's
	tcm.run(0);

	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+6)
		return false;
	if (tcm.getTelecommandEventRepository()->latestTelecommandEvent()->
			getEventType()!=TC_SUCCESS)
		return false;

	if (mm.getMode()!=targetMode)
		return false;

	if (bur.getConfiguration()==defaultConfig)
		return false;

	if (mm.getTelemetryFormat()!=targetTmFormat)
		return false;

	if (tcm.getNumberOfTelecommands()!=0)
		return false;

	// Reload and reexecute the reconfigure TC. It should now fail since 
	// there are no more free configurations
	tcm.loadTelecommand(&rTC);

	tcm.run(0);

	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+8)
		return false;
	if (tcm.getTelecommandEventRepository()->latestTelecommandEvent()->
			getEventType()!=TC_FAILURE)
		return false;

	// Before exiting, check failure and configuration error event numbers
	// NB Two failure events are created as a result of the GYR reconfiguration
	// which causes MACS instructions to be sent to the GYR units. The MACS
	// instructions fail and failure events are created
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN+2)
		return false;
	if (AocsObject::getConfigurationEventRepository()->getCounter()!=configEvtN)
		return false;

	// If this point is reached, test was successful
	return true;
}

// -------------------------------------------------------------------
// Test telecommand transactions 

bool telecommandManagementTest_3()
{
	// Retrieve the number of failure, configuration and TC events
	int failureEvtN = AocsObject::getFailureEventRepository()->getCounter();
	int configEvtN = AocsObject::getConfigurationEventRepository()->getCounter();
	int tcEvtN = TelecommandManager::getTelecommandEventRepository()->getCounter();

	// Create and configure the TC manager 
	TelecommandManager tcm(3);

	NullRecoveryAction nra;
	tcm.setTelecommandListFullRecoveryAction(&nra);
	tcm.setTelecommandStackErrorRecoveryAction(&nra);

	TestTelecommandLoader ttl;
	tcm.setTelecommandLoader(&ttl);

	if (!tcm.isConfigured())
		return false;

	// Create and configure a mode manager and a configuration manager 
	// to act as dummy objects upon which telecommands can operate
	int defaultConfig=0;
	int defaultMode=1;

	AocsMissionModeManager mm(3,defaultMode,0);
	NullModeChangeAction mca;
	mm.setModeChangeAction(&mca);

	BasicUnitReconfigurer bur(2,0,defaultConfig);
	short dummy_1,dummy_2,dummy_3;
	MacsTcController macsTcController(&dummy_1,&dummy_2,&dummy_3);
	macsTcController.setMaxPolling(2);
	GyrPrototype gyr_1(0,0), gyr_2(0,0);
	gyr_1.setAocsUnitHardware(&macsTcController);
	gyr_2.setAocsUnitHardware(&macsTcController);
	bur.setUnit(&gyr_1,0);
	bur.setUnit(&gyr_2,1);

	// Create test transaction telecommands 
	int targetMode=2;
	TelemetryFormat targetTmFormat=longTm;

	ModeChangeTransactionTelecommand mcTC(0,&mm,targetMode);
	ReconfigureTransactionTelecommand rTC(0,&bur);
	TelemetryFormatTransactionTelecommand tfTC(0,&mm,targetTmFormat);

	// Configure static part of transaction telecommands
	TransactionTelecommand::setTelecommandManager(&tcm);

	// Link the transaction telecommands
	mcTC.setNextTransactionTelecommand(&tfTC);
	tfTC.setNextTransactionTelecommand(&rTC);

	// Load the transaction telecommand in the TC manager
	tcm.loadTelecommand(&mcTC);

	// Run the TC manager and check execution of TC's
	tcm.run(0);

	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+2)
		return false;
	if (tcm.getTelecommandEventRepository()->latestTelecommandEvent()->
			getEventType()!=TC_TRANS_SUCCESS)
		return false;

	if (mm.getMode()!=targetMode)
		return false;

	if (bur.getConfiguration()==defaultConfig)
		return false;

	if (mm.getTelemetryFormat()!=targetTmFormat)
		return false;

	if (tcm.getNumberOfTelecommands()!=0)
		return false;

	// Disable one of the transaction telecommands, reload the transaction
	// TC and check that it will not be executed
	tfTC.disable();
	
	tcm.loadTelecommand(&mcTC);
	tcm.run(0);

	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+4)
		return false;
	if (tcm.getTelecommandEventRepository()->latestTelecommandEvent()->
			getEventType()!=TC_UNLOADED)
		return false;

	// Reset the mode manager to its original state and renable telemetry TC
	mm.reset();

	tfTC.enable();

	// Reload and reexecute the transaction TC. It should now fail since 
	// the reconfigre TC fails (there are no more free configurations)
	tcm.loadTelecommand(&mcTC);

	tcm.run(0);

	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+6)
		return false;
	if (tcm.getTelecommandEventRepository()->latestTelecommandEvent()->
			getEventType()!=TC_TRANS_FAILURE)
		return false;

	// Check that the mode change and telemetry change telecommands did
	// not take effect
	if (mm.getMode()!=defaultMode)	
		return false;

	if (mm.getTelemetryFormat()!=targetTmFormat)	
		return false;

	// Before exiting, check failure and configuration error event numbers
	// NB Two failure events are created as a result of the GYR reconfiguration
	// which causes MACS instructions to be sent to the GYR units. The MACS
	// instructions fail and failure events are created
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN+2)
		return false;
	if (AocsObject::getConfigurationEventRepository()->getCounter()!=configEvtN)
		return false;

	// If this point is reached, test was successful
	return true;
}

// -------------------------------------------------------------------
// Test the telecommand loader with simple TC's.
// Two telecommand buffers are prepared and loaded onto the TC loader. 
// The first buffer consists of a single packet containing a single
// simple TC. The second buffer contains two buffers with each a single
// simple TC.

bool telecommandManagementTest_4(DummyAocsClock* aocsClock)
{
	// Retrieve the number of failure, configuration and TC events
	int failureEvtN = AocsObject::getFailureEventRepository()->getCounter();
	int configEvtN = AocsObject::getConfigurationEventRepository()->getCounter();
	int tcEvtN = TelecommandManager::getTelecommandEventRepository()->getCounter();

	// Create a null recovery action
	NullRecoveryAction nra;

	// Create a TC buffer (caution: not deallocated when procedure exits!)
	int tcBufferInt[TC_BUFFER_SIZE/4];
	char* tcBuffer = (char*)tcBufferInt;

	// Create and configure the DMA telecommand loader and TC manager
	// (the TC loader is not the same in the Visual Studio and ERC32 
	// environments)
#ifdef MS_HOST
	VsDmaTelecommandLoader tcl(tcBufferInt);
#else
	Erc32DmaTelecommandLoader tcl(tcBufferInt);
#endif

	TelecommandManager tcm(3);
	
	tcl.setTelecommandLoaderRecoveryAction(&nra);
	tcm.setTelecommandListFullRecoveryAction(&nra);
	tcm.setTelecommandStackErrorRecoveryAction(&nra);

	tcl.setTelecommandManager(&tcm);
	tcm.setTelecommandLoader(&tcl);

	// Check that TC loader is configured
	if (!tcl.isConfigured())
		return false;

	// Create and configure a mode manager to act as dummy object 
	// upon which a test TC can operate
	AocsMissionModeManager mm(3,0,0);
	NullModeChangeAction mca;
	mm.setModeChangeAction(&mca);

	// Load the mode manager address in a 4-byte array
	int mmAddressInt = (int)&mm;
	int* mmAddressPtr = &mmAddressInt;
	char* mmAddress = (char*)mmAddressPtr;

	// Create a dummy mode change TC. It is needed to recover the VTable 
	// pointer for class ModeChangeTelecommand.
	ModeChangeTelecommand mct(0,&mm,1);

	// Load the VTable pointer for class ModeChangeTelecommand into a 
	// 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int mctVTableAddress = *((int*)&mct);
#else
	int mctVTableAddress = *(((int*)&mct+3));
#endif
	int* mctVTablePtrInt = &mctVTableAddress;
	char* mctVTablePtr = (char*)mctVTablePtrInt;

	// Load a 4-byte array with the time-tag for the test TC
	AocsTime timeTag = aocsClock->getTime()-1;
	int* timeTagPtrInt = (int*)&timeTag;
	char* timeTagPtr = (char*)timeTagPtrInt;

	// Prepare the first TC buffer. The TC buffer consists of a single 
	// TC packet containing a simple TC.
	// The simple TC is a mode change TC.

	tcBuffer[0]=1;	// start of TC buffer -- 1 packet in TC buffer	
	tcBuffer[1]=1;	// start of TC packet -- 1 block in TC packet
	tcBuffer[2]=14;	// start of TC block -- 14 bytes in TC data section
	tcBuffer[3]=0;	// no VTable section 

	tcBuffer[4]=mctVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[5]=mctVTablePtr[1];
	tcBuffer[6]=mctVTablePtr[2];
	tcBuffer[7]=mctVTablePtr[3];

	tcBuffer[8]=timeTagPtr[0];		// time tag
	tcBuffer[9]=timeTagPtr[1];
	tcBuffer[10]=timeTagPtr[2];
	tcBuffer[11]=timeTagPtr[3];

	tcBuffer[12]=mct.getClassId();	// class identifier

	tcBuffer[13]=mmAddress[0];		// mode manager address
	tcBuffer[14]=mmAddress[1];
	tcBuffer[15]=mmAddress[2];
	tcBuffer[16]=mmAddress[3];

	tcBuffer[17]=1;					// new mode

	// Run the TC loader
	tcl.run(0);

	// Check that one TC was loaded
	if (tcm.getNumberOfTelecommands()!=1)
		return false;

	// Run the TC manager -- the mode change TC should be executed
	tcm.run(0);

	// Check that the mode change TC was executed
	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+2)
		return false;
	if (tcm.getTelecommandEventRepository()->latestTelecommandEvent()->
			getEventType()!=TC_SUCCESS)
		return false;

	if (mm.getMode()!=1)
		return false;

	// Prepare the second TC buffer. The TC buffer consists of two 
	// TC packets each containing a simple TC.
	// The simple TC is a mode change TC. 
	// Locations 1 to 17 of TcBuffer are the same as for the first
	// TC buffer.

	tcBuffer[0]=2;	// start of TC buffer -- 2 packets in TC buffer	
					
	tcBuffer[18]=1;	// start of second TC packet -- 1 block in TC packet
	tcBuffer[19]=14;// start of TC block -- 14 bytes in TC data section
	tcBuffer[20]=0;	// no VTable section 

	tcBuffer[21]=mctVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[22]=mctVTablePtr[1];
	tcBuffer[23]=mctVTablePtr[2];
	tcBuffer[24]=mctVTablePtr[3];

	tcBuffer[25]=timeTagPtr[0];		// time tag
	tcBuffer[26]=timeTagPtr[1];
	tcBuffer[27]=timeTagPtr[2];
	tcBuffer[28]=timeTagPtr[3];

	tcBuffer[29]=mct.getClassId();	// class identifier

	tcBuffer[30]=mmAddress[0];		// mode manager address
	tcBuffer[31]=mmAddress[1];
	tcBuffer[32]=mmAddress[2];
	tcBuffer[33]=mmAddress[3];

	tcBuffer[34]=2;					// new mode

	// Run the TC loader
	tcl.run(0);

	// Check that two TC's were loaded
	if (tcm.getNumberOfTelecommands()!=2)
		return false;

	// Run the TC manager -- the two mode change TC should be executed
	tcm.run(0);

	// Check that the mode change TC's were executed
	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+6)
		return false;
	if (tcm.getTelecommandEventRepository()->latestTelecommandEvent()->
			getEventType()!=TC_SUCCESS)
		return false;

	if (mm.getMode()!=2)
		return false;

	// Before exiting, check failure and configuration error event numbers
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN)
		return false;
	if (AocsObject::getConfigurationEventRepository()->getCounter()!=configEvtN)
		return false;

	// If this point is reached, test was successful
	return true;
}

// -------------------------------------------------------------------
// Test the telecommand loader with TC transaction.
// One telecommand buffer is prepared and loaded onto the TC loader. 
// The TC buffer consists of a single TC packet containing a single
// TC transaction. 
// The TC transaction contains two transaction TC: one to change the 
// mode of the AOCS mission mode manager and one to change its TM
// format.

bool telecommandManagementTest_5(DummyAocsClock* aocsClock)
{
	// Retrieve the number of failure, configuration and TC events
	int failureEvtN = AocsObject::getFailureEventRepository()->getCounter();
	int configEvtN = AocsObject::getConfigurationEventRepository()->getCounter();
	int tcEvtN = TelecommandManager::getTelecommandEventRepository()->getCounter();

	// Create a null recovery action
	NullRecoveryAction nra;
	
	// Create a TC buffer (caution: not deallocated when procedure exits!)
	int tcBufferInt[TC_BUFFER_SIZE/4];
	char* tcBuffer = (char*)tcBufferInt;

	// (the TC loader is not the same in the Visual Studio and ERC32 
	// environments)
#ifdef MS_HOST
	VsDmaTelecommandLoader tcl(tcBufferInt);
#else
	Erc32DmaTelecommandLoader tcl(tcBufferInt);
#endif
	
	TelecommandManager tcm(3);
	
	tcl.setTelecommandLoaderRecoveryAction(&nra);
	tcm.setTelecommandListFullRecoveryAction(&nra);
	tcm.setTelecommandStackErrorRecoveryAction(&nra);

	tcl.setTelecommandManager(&tcm);
	tcm.setTelecommandLoader(&tcl);

	// Check that TC loader is configured
	if (!tcl.isConfigured())
		return false;

	// Load the telecommand manager in the transaction telecommands
	TransactionTelecommand::setTelecommandManager(&tcm);

	// Create and configure a mode manager to act as dummy object 
	// upon which test TC's can operate
	AocsMissionModeManager mm(3,0,0);
	NullModeChangeAction mca;
	mm.setModeChangeAction(&mca);

	// Load the mode manager address in a 4-byte array
	int mmAddressInt = (int)&mm;
	int* mmAddressPtr = &mmAddressInt;
	char* mmAddress = (char*)mmAddressPtr;

	// The mode manager implements the Telemeterable interface. Load the address 
	// of the mode manager seen as an instance of type Telemeterable in a 4-byte array
	Telemeterable* telemeterableMm = &mm;
	int mmTelemeterableAddressInt = (int)telemeterableMm;
	int* mmTelemeterableAddressPtr = &mmTelemeterableAddressInt;
	char* mmTelemeterableAddress = (char*)mmTelemeterableAddressPtr;

	// Create a dummy mode change transaction TC. It is needed to recover the VTable 
	// pointer for class ModeChangeTransactionTelecommand.
	ModeChangeTransactionTelecommand mct(0,&mm,1);

	// Load the VTable pointer for class ModeChangeTelecommand into a 
	// 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int mctVTableAddress = *((int*)&mct);
#else
	int mctVTableAddress = *(((int*)&mct+3));
#endif
	int* mctVTablePtrInt = &mctVTableAddress;
	char* mctVTablePtr = (char*)mctVTablePtrInt;

	// Create a dummy telemetry format transaction TC. It is needed to recover the VTable 
	// pointer for class .TelemetryFormatTransactionTelecommand
	TelemetryFormatTransactionTelecommand tft(0,&mm,longTm);

	// Load the VTable pointer for class TelemetryFormatTransactionTelecommand into a 
	// 4-byte array
#ifdef MS_HOST
	int tftVTableAddress = *((int*)&tft);
#else
	int tftVTableAddress = *(((int*)&tft+3));
#endif
	int* tftVTablePtrInt = &tftVTableAddress;
	char* tftVTablePtr = (char*)tftVTablePtrInt;

	// Load a 4-byte array with the time-tag for the test TC's
	AocsTime timeTag = aocsClock->getTime()-1;
	int* timeTagPtrInt = (int*)&timeTag;
	char* timeTagPtr = (char*)timeTagPtrInt;

	// Prepare the TC buffer. 
	// The buffer consists of one packet.
	// The packet contains two blocks.
	// The first block is a mode change transaction TC.
	// The second block is a telemetry format transaction TC.

	tcBuffer[0]=1;	// start of TC buffer -- 1 packet in TC buffer	
	tcBuffer[1]=2;	// start of TC packet -- 2 blocks in TC packet

	tcBuffer[2]=14;	// start of first TC block -- 14 bytes in TC data section
	tcBuffer[3]=0;	// no VTable section 

	tcBuffer[4]=mctVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[5]=mctVTablePtr[1];
	tcBuffer[6]=mctVTablePtr[2];
	tcBuffer[7]=mctVTablePtr[3];

	tcBuffer[8]=timeTagPtr[0];		// time tag
	tcBuffer[9]=timeTagPtr[1];
	tcBuffer[10]=timeTagPtr[2];
	tcBuffer[11]=timeTagPtr[3];

	tcBuffer[12]=mct.getClassId();	// class identifier

	tcBuffer[13]=mmAddress[0];		// mode manager address
	tcBuffer[14]=mmAddress[1];
	tcBuffer[15]=mmAddress[2];
	tcBuffer[16]=mmAddress[3];

	tcBuffer[17]=1;					// new mode

#ifdef MS_HOST
	tcBuffer[18]=10;	// start of second TC block -- 10 bytes in TC data section
#else
	tcBuffer[18]=13;	// start of second TC block -- 13 bytes in TC data section
#endif
	tcBuffer[19]=0;		// no VTable section 

	tcBuffer[20]=tftVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[21]=tftVTablePtr[1];
	tcBuffer[22]=tftVTablePtr[2];
	tcBuffer[23]=tftVTablePtr[3];

	tcBuffer[24]=tft.getClassId();	// class identifier

	tcBuffer[25]=mmTelemeterableAddress[0];		// mode manager address
	tcBuffer[26]=mmTelemeterableAddress[1];
	tcBuffer[27]=mmTelemeterableAddress[2];
	tcBuffer[28]=mmTelemeterableAddress[3];

#ifdef MS_HOST
	tcBuffer[29]=2;					// new telemetry format
#else
	tcBuffer[29]=0;					// new telemetry format
	tcBuffer[30]=0;
	tcBuffer[31]=0;
	tcBuffer[32]=2;
#endif

	// Run the TC loader
	tcl.run(0);

	// Check that one TC was loaded
	if (tcm.getNumberOfTelecommands()!=1)
		return false;

	// Run the TC manager -- the TC transaction should be executed changing
	// both the mode and the TM format of the mode manager
	tcm.run(0);

	// Check that the TC was executed
	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+2)
		return false;
	if (tcm.getTelecommandEventRepository()->latestTelecommandEvent()->
			getEventType()!=TC_TRANS_SUCCESS)
		return false;

	if (mm.getMode()!=1)
		return false;

	if (mm.getTelemetryFormat()!=longTm)
		return false;

	// Before exiting, check failure and configuration error event numbers
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN)
		return false;
	if (AocsObject::getConfigurationEventRepository()->getCounter()!=configEvtN)
		return false;

	// If this point is reached, test was successful
	return true;
}

// -------------------------------------------------------------------
// Test the telecommand loader with manoeuvre TC.
// One telecommand buffer is prepared and loaded onto the TC loader. 
// The buffer consists of a single packet containing a single TC to
// load a manoeuvre in the manoeuvre manager.

bool telecommandManagementTest_6(DummyAocsClock* aocsClock)
{
	// Retrieve the number of failure, configuration and TC events
	int failureEvtN = AocsObject::getFailureEventRepository()->getCounter();
	int configEvtN = AocsObject::getConfigurationEventRepository()->getCounter();
	int tcEvtN = TelecommandManager::getTelecommandEventRepository()->getCounter();

	// Create a null recovery action
	NullRecoveryAction nra;

	// Create a TC buffer (caution: not deallocated when procedure exits!)
	int tcBufferInt[TC_BUFFER_SIZE/4];
	char* tcBuffer = (char*)tcBufferInt;

	// Create and configure the DMA telecommand loader and TC manager
	// (the TC loader is not the same in the Visual Studio and ERC32 
	// environments)
#ifdef MS_HOST
	VsDmaTelecommandLoader tcl(tcBufferInt);
#else
	Erc32DmaTelecommandLoader tcl(tcBufferInt);
#endif

	TelecommandManager tcm(3);
	
	tcl.setTelecommandLoaderRecoveryAction(&nra);
	tcm.setTelecommandListFullRecoveryAction(&nra);
	tcm.setTelecommandStackErrorRecoveryAction(&nra);

	tcl.setTelecommandManager(&tcm);
	tcm.setTelecommandLoader(&tcl);

	// Check that TC loader is configured
	if (!tcl.isConfigured())
		return false;

	// Create and configure a manoeuvre manager where the TC will load
	// the test manoeuvre
	ManoeuvreManager mnManager;
	mnManager.initializeManoeuvreList(20);

	// Create the dummy manoeuvre that the TC will load
	AocsTime dmStartTime = 0;	// manoeuvre to start immediately
	int dmStepCount = 25;
	DummyManoeuvre dm(dmStartTime, dmStepCount);
	dm.setMonitorListSize(3);

	// Configure the static part of the manoeuvre telecommand
	ManoeuvreTelecommand::setManoeuvreManager(&mnManager);

	// Load the dummy manoeuvre address in a 4-byte array
	int dmAddressInt = (int)&dm;
	int* dmAddressPtr = &dmAddressInt;
	char* dmAddress = (char*)dmAddressPtr;

	// Create a dummy manoeuvre TC. It is needed to recover the VTable 
	// pointer for class ManoeuvreTelecommand.
	ManoeuvreTelecommand mntc(0,&dm);

	// Load the VTable pointer for class ManoeuvreTelecommand into a 
	// 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int mntcVTableAddress = *((int*)&mntc);
#else
	int mntcVTableAddress = *(((int*)&mntc+3));
#endif
	int* mntcVTablePtrInt = &mntcVTableAddress;
	char* mntcVTablePtr = (char*)mntcVTablePtrInt;

	// Load a 4-byte array with the time-tag for the test TC
	AocsTime timeTag = 0;
	int* timeTagPtrInt = (int*)&timeTag;
	char* timeTagPtr = (char*)timeTagPtrInt;

	// Prepare the TC buffer. The TC buffer consists of a single 
	// TC packet containing the manoeuvre TC.

	tcBuffer[0]=1;	// start of TC buffer -- 1 packet in TC buffer	
	tcBuffer[1]=1;	// start of TC packet -- 1 block in TC packet
	tcBuffer[2]=13;	// start of TC block -- 13 bytes in TC data section
	tcBuffer[3]=0;	// no VTable section 

	tcBuffer[4]=mntcVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[5]=mntcVTablePtr[1];
	tcBuffer[6]=mntcVTablePtr[2];
	tcBuffer[7]=mntcVTablePtr[3];

	tcBuffer[8]=timeTagPtr[0];		// time tag
	tcBuffer[9]=timeTagPtr[1];
	tcBuffer[10]=timeTagPtr[2];
	tcBuffer[11]=timeTagPtr[3];

	tcBuffer[12]=mntc.getClassId();	// class identifier

	tcBuffer[13]=dmAddress[0];		// manoeuvre address
	tcBuffer[14]=dmAddress[1];
	tcBuffer[15]=dmAddress[2];
	tcBuffer[16]=dmAddress[3];

	// Run the TC loader
	tcl.run(0);

	// Check that one TC was loaded
	if (tcm.getNumberOfTelecommands()!=1)
		return false;

	// Run the TC manager -- the manoeuvre should be loaded in the 
	// manoeuvre manager
	tcm.run(0);

	// Run the manoeuvre manager -- the manoeuvre should be executed
	mnManager.run(0);

	// Check that two TC events were geenrated (TC was loaded and TC
	// was executed)
	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+2)
		return false;

	// Check that the manoeuvre was loaded in the manoeuvre manager
	if (mnManager.getManoeuvreEventRepository()->latestManoeuvreEvent()->
			getEventType()!=MANOEUVRE_STARTED)
		return false;

	// If this point is reached, test was successful
	return true;
}

// -------------------------------------------------------------------
// Test the telecommand loader with attitude slew TC.
// One telecommand buffer is prepared and loaded onto the TC loader. 
// The buffer consists of a single packet containing a single TC to
// configure and load an attitude slew in the manoeuvre manager.

bool telecommandManagementTest_7(DummyAocsClock* aocsClock)
{
	// Retrieve the number of failure, configuration and TC events
	int failureEvtN = AocsObject::getFailureEventRepository()->getCounter();
	int configEvtN = AocsObject::getConfigurationEventRepository()->getCounter();
	int tcEvtN = TelecommandManager::getTelecommandEventRepository()->getCounter();

	// Create a null recovery action
	NullRecoveryAction nra;

	// Create a TC buffer (caution: not deallocated when procedure exits!)
	int tcBufferInt[TC_BUFFER_SIZE/4];
	char* tcBuffer = (char*)tcBufferInt;

	// Create and configure the DMA telecommand loader and TC manager
	// (the TC loader is not the same in the Visual Studio and ERC32 
	// environments)
#ifdef MS_HOST
	VsDmaTelecommandLoader tcl(tcBufferInt);
#else
	Erc32DmaTelecommandLoader tcl(tcBufferInt);
#endif

	TelecommandManager tcm(3);
	
	tcl.setTelecommandLoaderRecoveryAction(&nra);
	tcm.setTelecommandListFullRecoveryAction(&nra);
	tcm.setTelecommandStackErrorRecoveryAction(&nra);

	tcl.setTelecommandManager(&tcm);
	tcm.setTelecommandLoader(&tcl);

	// Check that TC loader is configured
	if (!tcl.isConfigured())
		return false;

	// Create and configure a manoeuvre manager where the TC will load
	// the test manoeuvre
	ManoeuvreManager mnManager;
	mnManager.initializeManoeuvreList(20);

	// Create the attitude slew manoeuvre that the TC will load
	AocsTime startTime = 6;			// slew start time
	AocsTime endTime = 100;			// slew end time
	float errorThreshold = 100;
	float att_end = 10;				// slew target attitude
	Real itemToSet = 0.0;			// to be set by attitude slew
	AocsTime itemTimeTag = 0;		// time tag for itemToBeSet
	DataItemWrite attDataItem(&itemTimeTag, &itemToSet);

	AttitudeSlew as;
	as.configure(startTime,endTime,att_end,attDataItem,errorThreshold);
	as.setMonitorListSize(3);

	// Configure the static part of the attitude slew telecommand
	AttitudeSlewTelecommand::setManoeuvreManager(&mnManager);

	// Load the attitude slew manoeuvre address in a 4-byte array
	int asAddressInt = (int)&as;
	int* asAddressPtr = &asAddressInt;
	char* asAddress = (char*)asAddressPtr;

	// Create a dummy attitude slew TC. It is needed to recover the VTable 
	// pointer for class AttitudeSlewTelecommand.
	AttitudeSlewTelecommand astc(0,&as, errorThreshold, att_end, startTime,
									endTime, &itemToSet, &itemTimeTag);

	// Load the VTable pointer for class AttitudeSlewTelecommand into a 
	// 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int astcVTableAddress = *((int*)&astc);
#else
	int astcVTableAddress = *(((int*)&astc+3));
#endif
	int* astcVTablePtrInt = &astcVTableAddress;
	char* astcVTablePtr = (char*)astcVTablePtrInt;

	// Load a 4-byte array with the time-tag for the test TC
	AocsTime timeTag = 0;
	int* timeTagPtrInt = (int*)&timeTag;
	char* timeTagPtr = (char*)timeTagPtrInt;

	// Load a 4-byte array with the error threshold
	int* errorThresholdPtrInt = (int*)&errorThreshold;
	char* errorThresholdPtr = (char*)errorThresholdPtrInt;

	// Load a 4-byte array with the attitude end value
	int* att_endPtrInt = (int*)&att_end;
	char* att_endPtr = (char*)att_endPtrInt;

	// Load a 4-byte array with the start time for the attitude slew
	int* startTimePtrInt = (int*)&startTime;
	char* startTimePtr = (char*)startTimePtrInt;

	// Load a 4-byte array with the end time for the attitude slew
	int* endTimePtrInt = (int*)&endTime;
	char* endTimePtr = (char*)endTimePtrInt;

	// Load a 4-byte array with pointer to the item to set
	Real* itemToSetAddr = &itemToSet;
	int* itemToSetPtrInt = (int*)&itemToSetAddr;
	char* itemToSetPtr = (char*)itemToSetPtrInt;

	// Load a 4-byte array with pointer to the item to set
	AocsTime* itemTimeTagAddr = &itemTimeTag;
	int* itemTimeTagPtrInt = (int*)&itemTimeTagAddr;
	char* itemTimeTagPtr = (char*)itemTimeTagPtrInt;

	// Prepare the TC buffer. The TC buffer consists of a single 
	// TC packet containing the manoeuvre TC.

	tcBuffer[0]=1;	// start of TC buffer -- 1 packet in TC buffer	
	tcBuffer[1]=1;	// start of TC packet -- 1 block in TC packet
	tcBuffer[2]=37;	// start of TC block -- 33 bytes in TC data section
	tcBuffer[3]=0;	// no VTable section 

	tcBuffer[4]=astcVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[5]=astcVTablePtr[1];
	tcBuffer[6]=astcVTablePtr[2];
	tcBuffer[7]=astcVTablePtr[3];

	tcBuffer[8]=timeTagPtr[0];		// time tag
	tcBuffer[9]=timeTagPtr[1];
	tcBuffer[10]=timeTagPtr[2];
	tcBuffer[11]=timeTagPtr[3];

	tcBuffer[12]=astc.getClassId();	// class identifier

	tcBuffer[13]=asAddress[0];		// attitude slew address
	tcBuffer[14]=asAddress[1];
	tcBuffer[15]=asAddress[2];
	tcBuffer[16]=asAddress[3];

	tcBuffer[17]=errorThresholdPtr[0];		// error threshold
	tcBuffer[18]=errorThresholdPtr[1];
	tcBuffer[19]=errorThresholdPtr[2];
	tcBuffer[20]=errorThresholdPtr[3];

	tcBuffer[21]=att_endPtr[0];				// attitude end point
	tcBuffer[22]=att_endPtr[1];
	tcBuffer[23]=att_endPtr[2];
	tcBuffer[24]=att_endPtr[3];

	tcBuffer[25]=startTimePtr[0];			// slew starting time
	tcBuffer[26]=startTimePtr[1];
	tcBuffer[27]=startTimePtr[2];
	tcBuffer[28]=startTimePtr[3];

	tcBuffer[29]=endTimePtr[0];				// slew end time
	tcBuffer[30]=endTimePtr[1];
	tcBuffer[31]=endTimePtr[2];
	tcBuffer[32]=endTimePtr[3];

	tcBuffer[33]=itemToSetPtr[0];			// item to set
	tcBuffer[34]=itemToSetPtr[1];
	tcBuffer[35]=itemToSetPtr[2];
	tcBuffer[36]=itemToSetPtr[3];

	tcBuffer[37]=itemTimeTagPtr[0];			// time tag of item to set
	tcBuffer[38]=itemTimeTagPtr[1];
	tcBuffer[39]=itemTimeTagPtr[2];
	tcBuffer[40]=itemTimeTagPtr[3];

	// Run the TC loader
	tcl.run(0);

	// Check that one TC was loaded
	if (tcm.getNumberOfTelecommands()!=1)
		return false;

	// Run the TC manager -- the manoeuvre should be loaded in the 
	// manoeuvre manager
	tcm.run(0);

	// Run the manoeuvre manager -- the manoeuvre should be initialized
	mnManager.run(0);

	// Advance the time 
	aocsClock->run(0);

	// Run the manoeuvre manager -- manoeuvre execution should begin
	mnManager.run(0);

	// Check that two TC events were generated (TC was loaded and TC
	// was executed)
	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+2)
		return false;

	// Check that the manoeuvre was loaded in the manoeuvre manager
	if (mnManager.getManoeuvreEventRepository()->latestManoeuvreEvent()->
			getEventType()!=MANOEUVRE_STARTED)
		return false;

	// Check that the data item and its time tag were updated by the 
	// attitude slew
	if (itemToSet == 0)
		return false;

	if (itemTimeTag == 0)
		return false;

	// If this point is reached, test was successful
	return true;
}
