// TestCaseTelecommand_4.cpp
//
// Version	1.0
// Date		06.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseTelecommand_4.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../GeneralInclude/CompilerSwitches.h"
#include "../../AocsEvent/TelecommandEvent.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../TelecommandManagement/ReconfigureTelecommand.h"
#include "../../TelecommandManagement/ModeChangeTelecommand.h"
#include "../../TelecommandManagement/TelemetryFormatTelecommand.h"
#include "../../TelecommandManagement/VsDmaTelecommandLoader.h"
#include "../../TelecommandManagement/Erc32DmaTelecommandLoader.h"
#include "../../TelecommandManagement/TelecommandManager.h"
#include "../../TelecommandManagement/TestTelecommandLoader.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"

// -------------------------------------------------------------------
TestCaseTelecommand_4::TestCaseTelecommand_4():
	TestCaseGenericSetUp(ID_TELECOMMANDMANAGER*10+4,
									"TestCaseTelecommand_4") {
		return;
}

// -------------------------------------------------------------------
// Test the telecommand loader with simple TC's.
// Two telecommand buffers are prepared and loaded onto the TC loader. 
// The first buffer consists of a single packet containing a single
// simple TC. The second buffer contains two buffers with each a single
// simple TC.

void TestCaseTelecommand_4::runTestCase() {

	// Create a null recovery action
	NullRecoveryAction nra;

	// Create a TC buffer (caution: not deallocated when procedure exits!)
	int tcBufferInt[TC_BUFFER_SIZE/4];
	char* tcBuffer = (char*)tcBufferInt;

	// Create and configure the DMA telecommand loader and TC manager
	// (the TC loader is not the same in the Visual Studio and ERC32 
	// environments)
#ifdef MS_HOST
	VsDmaTelecommandLoader tcl(tcBufferInt);
#else
	Erc32DmaTelecommandLoader tcl(tcBufferInt);
#endif

	TelecommandManager tcm(3);
	
	tcl.setTelecommandLoaderRecoveryAction(&nra);
	tcm.setTelecommandListFullRecoveryAction(&nra);
	tcm.setTelecommandStackErrorRecoveryAction(&nra);

	tcl.setTelecommandManager(&tcm);
	tcm.setTelecommandLoader(&tcl);

	// Check that TC loader is configured
	if (!tcl.isConfigured())
	{	setTestResult(TEST_FAILURE, "Telecommand test 4 failed");
		return;
	}

	// Create and configure a mode manager to act as dummy object 
	// upon which a test TC can operate
	AocsMissionModeManager mm(3,0,0);
	NullModeChangeAction mca;
	mm.setModeChangeAction(&mca);

	// Load the mode manager address in a 4-byte array
	int mmAddressInt = (int)&mm;
	int* mmAddressPtr = &mmAddressInt;
	char* mmAddress = (char*)mmAddressPtr;

	// Create a dummy mode change TC. It is needed to recover the VTable 
	// pointer for class ModeChangeTelecommand.
	ModeChangeTelecommand mct(0,&mm,1);

	// Load the VTable pointer for class ModeChangeTelecommand into a 
	// 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int mctVTableAddress = *((int*)&mct);
#else
	int mctVTableAddress = *(((int*)&mct+3));
#endif
	int* mctVTablePtrInt = &mctVTableAddress;
	char* mctVTablePtr = (char*)mctVTablePtrInt;

	// Load a 4-byte array with the time-tag for the test TC
	AocsTime timeTag = aocsClock.getTime()-1;
	int* timeTagPtrInt = (int*)&timeTag;
	char* timeTagPtr = (char*)timeTagPtrInt;

	// Prepare the first TC buffer. The TC buffer consists of a single 
	// TC packet containing a simple TC.
	// The simple TC is a mode change TC.

	tcBuffer[0]=1;	// start of TC buffer -- 1 packet in TC buffer	
	tcBuffer[1]=1;	// start of TC packet -- 1 block in TC packet
	tcBuffer[2]=14;	// start of TC block -- 14 bytes in TC data section
	tcBuffer[3]=0;	// no VTable section 

	tcBuffer[4]=mctVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[5]=mctVTablePtr[1];
	tcBuffer[6]=mctVTablePtr[2];
	tcBuffer[7]=mctVTablePtr[3];

	tcBuffer[8]=timeTagPtr[0];		// time tag
	tcBuffer[9]=timeTagPtr[1];
	tcBuffer[10]=timeTagPtr[2];
	tcBuffer[11]=timeTagPtr[3];

	tcBuffer[12]=mct.getClassId();	// class identifier

	tcBuffer[13]=mmAddress[0];		// mode manager address
	tcBuffer[14]=mmAddress[1];
	tcBuffer[15]=mmAddress[2];
	tcBuffer[16]=mmAddress[3];

	tcBuffer[17]=1;					// new mode

	// Run the TC loader
	tcl.run(0);

	// Check that one TC was loaded
	if (tcm.getNumberOfTelecommands()!=1)
	{	setTestResult(TEST_FAILURE, "Telecommand test 4 failed");
		return;
	}

	// Run the TC manager -- the mode change TC should be executed
	tcm.run(0);

	// Check that the mode change TC was executed
	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+2)
	{	setTestResult(TEST_FAILURE, "Telecommand test 4 failed");
		return;
	}
	if (tcm.getTelecommandEventRepository()->latestTelecommandEvent()->
			getEventType()!=TC_SUCCESS)
	{	setTestResult(TEST_FAILURE, "Telecommand test 4 failed");
		return;
	}

	if (mm.getMode()!=1)
	{	setTestResult(TEST_FAILURE, "Telecommand test 4 failed");
		return;
	}

	// Prepare the second TC buffer. The TC buffer consists of two 
	// TC packets each containing a simple TC.
	// The simple TC is a mode change TC. 
	// Locations 1 to 17 of TcBuffer are the same as for the first
	// TC buffer.

	tcBuffer[0]=2;	// start of TC buffer -- 2 packets in TC buffer	
					
	tcBuffer[18]=1;	// start of second TC packet -- 1 block in TC packet
	tcBuffer[19]=14;// start of TC block -- 14 bytes in TC data section
	tcBuffer[20]=0;	// no VTable section 

	tcBuffer[21]=mctVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[22]=mctVTablePtr[1];
	tcBuffer[23]=mctVTablePtr[2];
	tcBuffer[24]=mctVTablePtr[3];

	tcBuffer[25]=timeTagPtr[0];		// time tag
	tcBuffer[26]=timeTagPtr[1];
	tcBuffer[27]=timeTagPtr[2];
	tcBuffer[28]=timeTagPtr[3];

	tcBuffer[29]=mct.getClassId();	// class identifier

	tcBuffer[30]=mmAddress[0];		// mode manager address
	tcBuffer[31]=mmAddress[1];
	tcBuffer[32]=mmAddress[2];
	tcBuffer[33]=mmAddress[3];

	tcBuffer[34]=2;					// new mode

	// Run the TC loader
	tcl.run(0);

	// Check that two TC's were loaded
	if (tcm.getNumberOfTelecommands()!=2)
	{	setTestResult(TEST_FAILURE, "Telecommand test 4 failed");
		return;
	}

	// Run the TC manager -- the two mode change TC should be executed
	tcm.run(0);

	// Check that the mode change TC's were executed
	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+6)
	{	setTestResult(TEST_FAILURE, "Telecommand test 4 failed");
		return;
	}
	if (tcm.getTelecommandEventRepository()->latestTelecommandEvent()->
			getEventType()!=TC_SUCCESS)
	{	setTestResult(TEST_FAILURE, "Telecommand test 4 failed");
		return;
	}

	if (mm.getMode()!=2)
	{	setTestResult(TEST_FAILURE, "Telecommand test 4 failed");
		return;
	}

	// Before exiting, check failure and configuration error event numbers
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN)
	{	setTestResult(TEST_FAILURE, "Telecommand test 4 failed");
		return;
	}
	if (AocsObject::getConfigurationEventRepository()->getCounter()!=configEvtN)
	{	setTestResult(TEST_FAILURE, "Telecommand test 4 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
