// TestCaseTelecommand_5.cpp
//
// Version	1.0
// Date		06.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseTelecommand_5.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../GeneralInclude/CompilerSwitches.h"
#include "../../AocsEvent/TelecommandEvent.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../TelecommandManagement/ReconfigureTelecommand.h"
#include "../../TelecommandManagement/ModeChangeTelecommand.h"
#include "../../TelecommandManagement/TelemetryFormatTelecommand.h"
#include "../../TelecommandManagement/ReconfigureTransactionTelecommand.h"
#include "../../TelecommandManagement/ModeChangeTransactionTelecommand.h"
#include "../../TelecommandManagement/TelemetryFormatTransactionTelecommand.h"
#include "../../TelecommandManagement/VsDmaTelecommandLoader.h"
#include "../../TelecommandManagement/Erc32DmaTelecommandLoader.h"
#include "../../TelecommandManagement/TelecommandManager.h"
#include "../../TelecommandManagement/TestTelecommandLoader.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"

// -------------------------------------------------------------------
TestCaseTelecommand_5::TestCaseTelecommand_5():
	TestCaseGenericSetUp(ID_TELECOMMANDMANAGER*10+5,
									"TestCaseTelecommand_5") {
		return;
}

// -------------------------------------------------------------------
// Test the telecommand loader with TC transaction.
// One telecommand buffer is prepared and loaded onto the TC loader. 
// The TC buffer consists of a single TC packet containing a single
// TC transaction. 
// The TC transaction contains two transaction TC: one to change the 
// mode of the AOCS mission mode manager and one to change its TM
// format.

void TestCaseTelecommand_5::runTestCase() {

	// Create a null recovery action
	NullRecoveryAction nra;
	
	// Create a TC buffer (caution: not deallocated when procedure exits!)
	int tcBufferInt[TC_BUFFER_SIZE/4];
	char* tcBuffer = (char*)tcBufferInt;

	// (the TC loader is not the same in the Visual Studio and ERC32 
	// environments)
#ifdef MS_HOST
	VsDmaTelecommandLoader tcl(tcBufferInt);
#else
	Erc32DmaTelecommandLoader tcl(tcBufferInt);
#endif
	
	TelecommandManager tcm(3);
	
	tcl.setTelecommandLoaderRecoveryAction(&nra);
	tcm.setTelecommandListFullRecoveryAction(&nra);
	tcm.setTelecommandStackErrorRecoveryAction(&nra);

	tcl.setTelecommandManager(&tcm);
	tcm.setTelecommandLoader(&tcl);

	// Check that TC loader is configured
	if (!tcl.isConfigured())
	{	setTestResult(TEST_FAILURE, "Telecommand test 5 failed");
		return;
	}

	// Load the telecommand manager in the transaction telecommands
	TransactionTelecommand::setTelecommandManager(&tcm);

	// Create and configure a mode manager to act as dummy object 
	// upon which test TC's can operate
	AocsMissionModeManager mm(3,0,0);
	NullModeChangeAction mca;
	mm.setModeChangeAction(&mca);

	// Load the mode manager address in a 4-byte array
	int mmAddressInt = (int)&mm;
	int* mmAddressPtr = &mmAddressInt;
	char* mmAddress = (char*)mmAddressPtr;

	// The mode manager implements the Telemeterable interface. Load the address 
	// of the mode manager seen as an instance of type Telemeterable in a 4-byte array
	Telemeterable* telemeterableMm = &mm;
	int mmTelemeterableAddressInt = (int)telemeterableMm;
	int* mmTelemeterableAddressPtr = &mmTelemeterableAddressInt;
	char* mmTelemeterableAddress = (char*)mmTelemeterableAddressPtr;

	// Create a dummy mode change transaction TC. It is needed to recover the VTable 
	// pointer for class ModeChangeTransactionTelecommand.
	ModeChangeTransactionTelecommand mct(0,&mm,1);

	// Load the VTable pointer for class ModeChangeTelecommand into a 
	// 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int mctVTableAddress = *((int*)&mct);
#else
	int mctVTableAddress = *(((int*)&mct+3));
#endif
	int* mctVTablePtrInt = &mctVTableAddress;
	char* mctVTablePtr = (char*)mctVTablePtrInt;

	// Create a dummy telemetry format transaction TC. It is needed to recover the VTable 
	// pointer for class .TelemetryFormatTransactionTelecommand
	TelemetryFormatTransactionTelecommand tft(0,&mm,longTm);

	// Load the VTable pointer for class TelemetryFormatTransactionTelecommand into a 
	// 4-byte array
#ifdef MS_HOST
	int tftVTableAddress = *((int*)&tft);
#else
	int tftVTableAddress = *(((int*)&tft+3));
#endif
	int* tftVTablePtrInt = &tftVTableAddress;
	char* tftVTablePtr = (char*)tftVTablePtrInt;

	// Load a 4-byte array with the time-tag for the test TC's
	AocsTime timeTag = aocsClock.getTime()-1;
	int* timeTagPtrInt = (int*)&timeTag;
	char* timeTagPtr = (char*)timeTagPtrInt;

	// Prepare the TC buffer. 
	// The buffer consists of one packet.
	// The packet contains two blocks.
	// The first block is a mode change transaction TC.
	// The second block is a telemetry format transaction TC.

	tcBuffer[0]=1;	// start of TC buffer -- 1 packet in TC buffer	
	tcBuffer[1]=2;	// start of TC packet -- 2 blocks in TC packet

	tcBuffer[2]=14;	// start of first TC block -- 14 bytes in TC data section
	tcBuffer[3]=0;	// no VTable section 

	tcBuffer[4]=mctVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[5]=mctVTablePtr[1];
	tcBuffer[6]=mctVTablePtr[2];
	tcBuffer[7]=mctVTablePtr[3];

	tcBuffer[8]=timeTagPtr[0];		// time tag
	tcBuffer[9]=timeTagPtr[1];
	tcBuffer[10]=timeTagPtr[2];
	tcBuffer[11]=timeTagPtr[3];

	tcBuffer[12]=mct.getClassId();	// class identifier

	tcBuffer[13]=mmAddress[0];		// mode manager address
	tcBuffer[14]=mmAddress[1];
	tcBuffer[15]=mmAddress[2];
	tcBuffer[16]=mmAddress[3];

	tcBuffer[17]=1;					// new mode

#ifdef MS_HOST
	tcBuffer[18]=10;	// start of second TC block -- 10 bytes in TC data section
#else
	tcBuffer[18]=13;	// start of second TC block -- 13 bytes in TC data section
#endif
	tcBuffer[19]=0;		// no VTable section 

	tcBuffer[20]=tftVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[21]=tftVTablePtr[1];
	tcBuffer[22]=tftVTablePtr[2];
	tcBuffer[23]=tftVTablePtr[3];

	tcBuffer[24]=tft.getClassId();	// class identifier

	tcBuffer[25]=mmTelemeterableAddress[0];		// mode manager address
	tcBuffer[26]=mmTelemeterableAddress[1];
	tcBuffer[27]=mmTelemeterableAddress[2];
	tcBuffer[28]=mmTelemeterableAddress[3];

#ifdef MS_HOST
	tcBuffer[29]=2;					// new telemetry format
#else
	tcBuffer[29]=0;					// new telemetry format
	tcBuffer[30]=0;
	tcBuffer[31]=0;
	tcBuffer[32]=2;
#endif

	// Run the TC loader
	tcl.run(0);

	// Check that one TC was loaded
	if (tcm.getNumberOfTelecommands()!=1)
	{	setTestResult(TEST_FAILURE, "Telecommand test 5 failed");
		return;
	}

	// Run the TC manager -- the TC transaction should be executed changing
	// both the mode and the TM format of the mode manager
	tcm.run(0);

	// Check that the TC was executed
	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+2)
	{	setTestResult(TEST_FAILURE, "Telecommand test 5 failed");
		return;
	}
	if (tcm.getTelecommandEventRepository()->latestTelecommandEvent()->
			getEventType()!=TC_TRANS_SUCCESS)
	{	setTestResult(TEST_FAILURE, "Telecommand test 5 failed");
		return;
	}

	if (mm.getMode()!=1)
	{	setTestResult(TEST_FAILURE, "Telecommand test 5 failed");
		return;
	}

	if (mm.getTelemetryFormat()!=longTm)
	{	setTestResult(TEST_FAILURE, "Telecommand test 5 failed");
		return;
	}

	// Before exiting, check failure and configuration error event numbers
	if (AocsObject::getFailureEventRepository()->getCounter()!=failureEvtN)
	{	setTestResult(TEST_FAILURE, "Telecommand test 5 failed");
		return;
	}
	if (AocsObject::getConfigurationEventRepository()->getCounter()!=configEvtN)
	{	setTestResult(TEST_FAILURE, "Telecommand test 5 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}
