// TestCaseTelecommand_7.cpp
//
// Version	1.0
// Date		06.02.01
// Author	A. Pasetti
//
// -------------------------------------------------------------------

#include "TestCaseTelecommand_7.h"
#include "../../GeneralInclude/ClassId.h"
#include "../../GeneralInclude/Constants.h"
#include "../../GeneralInclude/CompilerSwitches.h"
#include "../../AocsEvent/TelecommandEvent.h"
#include "../../AocsEvent/FailureEvent.h"
#include "../../AocsEvent/ManoeuvreEvent.h"
#include "../../FailureRecovery/NullRecoveryAction.h"
#include "../../TelecommandManagement/AttitudeSlewTelecommand.h"
#include "../../TelecommandManagement/ModeChangeTelecommand.h"
#include "../../TelecommandManagement/TelemetryFormatTelecommand.h"
#include "../../TelecommandManagement/ReconfigureTransactionTelecommand.h"
#include "../../TelecommandManagement/ModeChangeTransactionTelecommand.h"
#include "../../TelecommandManagement/TelemetryFormatTransactionTelecommand.h"
#include "../../TelecommandManagement/VsDmaTelecommandLoader.h"
#include "../../TelecommandManagement/Erc32DmaTelecommandLoader.h"
#include "../../TelecommandManagement/TelecommandManager.h"
#include "../../TelecommandManagement/TestTelecommandLoader.h"
#include "../../ModeManagement/AocsMissionModeManager.h"
#include "../../ModeManagement/NullModeChangeAction.h"
#include "../../ManoeuvreManagement/ManoeuvreManager.h"
#include "../../ManoeuvreManagement/AttitudeSlew.h"

// -------------------------------------------------------------------
TestCaseTelecommand_7::TestCaseTelecommand_7():
	TestCaseGenericSetUp(ID_TELECOMMANDMANAGER*10+7,
									"TestCaseTelecommand_7") {
		return;
}

// -------------------------------------------------------------------
// Test the telecommand loader with attitude slew TC.
// One telecommand buffer is prepared and loaded onto the TC loader. 
// The buffer consists of a single packet containing a single TC to
// configure and load an attitude slew in the manoeuvre manager.

void TestCaseTelecommand_7::runTestCase() {

	// Create a null recovery action
	NullRecoveryAction nra;

	// Create a TC buffer (caution: not deallocated when procedure exits!)
	int tcBufferInt[TC_BUFFER_SIZE/4];
	char* tcBuffer = (char*)tcBufferInt;

	// Create and configure the DMA telecommand loader and TC manager
	// (the TC loader is not the same in the Visual Studio and ERC32 
	// environments)
#ifdef MS_HOST
	VsDmaTelecommandLoader tcl(tcBufferInt);
#else
	Erc32DmaTelecommandLoader tcl(tcBufferInt);
#endif

	TelecommandManager tcm(3);
	
	tcl.setTelecommandLoaderRecoveryAction(&nra);
	tcm.setTelecommandListFullRecoveryAction(&nra);
	tcm.setTelecommandStackErrorRecoveryAction(&nra);

	tcl.setTelecommandManager(&tcm);
	tcm.setTelecommandLoader(&tcl);

	// Check that TC loader is configured
	if (!tcl.isConfigured())
	{	setTestResult(TEST_FAILURE, "Telecommand test 7 failed");
		return;
	}

	// Create and configure a manoeuvre manager where the TC will load
	// the test manoeuvre
	ManoeuvreManager mnManager;
	mnManager.initializeManoeuvreList(20);

	// Create the attitude slew manoeuvre that the TC will load
	AocsTime startTime = 6;			// slew start time
	AocsTime endTime = 100;			// slew end time
	float errorThreshold = 100;
	float att_end = 10;				// slew target attitude
	Real itemToSet = 0.0;			// to be set by attitude slew
	AocsTime itemTimeTag = 0;		// time tag for itemToBeSet
	DataItemWrite attDataItem(&itemTimeTag, &itemToSet);

	AttitudeSlew as;
	as.configure(startTime,endTime,att_end,attDataItem,errorThreshold);
	as.setMonitorListSize(3);

	// Configure the static part of the attitude slew telecommand
	AttitudeSlewTelecommand::setManoeuvreManager(&mnManager);

	// Load the attitude slew manoeuvre address in a 4-byte array
	int asAddressInt = (int)&as;
	int* asAddressPtr = &asAddressInt;
	char* asAddress = (char*)asAddressPtr;

	// Create a dummy attitude slew TC. It is needed to recover the VTable 
	// pointer for class AttitudeSlewTelecommand.
	AttitudeSlewTelecommand astc(0,&as, errorThreshold, att_end, startTime,
									endTime, &itemToSet, &itemTimeTag);

	// Load the VTable pointer for class AttitudeSlewTelecommand into a 
	// 4-byte array
	// NB the VS compiler puts the VTable pointer of a Telecommand object 
	// in the first longword. The GNU compiler in the ERC32 environment
	// puts in the third longword
#ifdef MS_HOST
	int astcVTableAddress = *((int*)&astc);
#else
	int astcVTableAddress = *(((int*)&astc+3));
#endif
	int* astcVTablePtrInt = &astcVTableAddress;
	char* astcVTablePtr = (char*)astcVTablePtrInt;

	// Load a 4-byte array with the time-tag for the test TC
	AocsTime timeTag = 0;
	int* timeTagPtrInt = (int*)&timeTag;
	char* timeTagPtr = (char*)timeTagPtrInt;

	// Load a 4-byte array with the error threshold
	int* errorThresholdPtrInt = (int*)&errorThreshold;
	char* errorThresholdPtr = (char*)errorThresholdPtrInt;

	// Load a 4-byte array with the attitude end value
	int* att_endPtrInt = (int*)&att_end;
	char* att_endPtr = (char*)att_endPtrInt;

	// Load a 4-byte array with the start time for the attitude slew
	int* startTimePtrInt = (int*)&startTime;
	char* startTimePtr = (char*)startTimePtrInt;

	// Load a 4-byte array with the end time for the attitude slew
	int* endTimePtrInt = (int*)&endTime;
	char* endTimePtr = (char*)endTimePtrInt;

	// Load a 4-byte array with pointer to the item to set
	Real* itemToSetAddr = &itemToSet;
	int* itemToSetPtrInt = (int*)&itemToSetAddr;
	char* itemToSetPtr = (char*)itemToSetPtrInt;

	// Load a 4-byte array with pointer to the item to set
	AocsTime* itemTimeTagAddr = &itemTimeTag;
	int* itemTimeTagPtrInt = (int*)&itemTimeTagAddr;
	char* itemTimeTagPtr = (char*)itemTimeTagPtrInt;

	// Prepare the TC buffer. The TC buffer consists of a single 
	// TC packet containing the manoeuvre TC.

	tcBuffer[0]=1;	// start of TC buffer -- 1 packet in TC buffer	
	tcBuffer[1]=1;	// start of TC packet -- 1 block in TC packet
	tcBuffer[2]=37;	// start of TC block -- 33 bytes in TC data section
	tcBuffer[3]=0;	// no VTable section 

	tcBuffer[4]=astcVTablePtr[0];	// beginning of data section -- VTable pointer
	tcBuffer[5]=astcVTablePtr[1];
	tcBuffer[6]=astcVTablePtr[2];
	tcBuffer[7]=astcVTablePtr[3];

	tcBuffer[8]=timeTagPtr[0];		// time tag
	tcBuffer[9]=timeTagPtr[1];
	tcBuffer[10]=timeTagPtr[2];
	tcBuffer[11]=timeTagPtr[3];

	tcBuffer[12]=astc.getClassId();	// class identifier

	tcBuffer[13]=asAddress[0];		// attitude slew address
	tcBuffer[14]=asAddress[1];
	tcBuffer[15]=asAddress[2];
	tcBuffer[16]=asAddress[3];

	tcBuffer[17]=errorThresholdPtr[0];		// error threshold
	tcBuffer[18]=errorThresholdPtr[1];
	tcBuffer[19]=errorThresholdPtr[2];
	tcBuffer[20]=errorThresholdPtr[3];

	tcBuffer[21]=att_endPtr[0];				// attitude end point
	tcBuffer[22]=att_endPtr[1];
	tcBuffer[23]=att_endPtr[2];
	tcBuffer[24]=att_endPtr[3];

	tcBuffer[25]=startTimePtr[0];			// slew starting time
	tcBuffer[26]=startTimePtr[1];
	tcBuffer[27]=startTimePtr[2];
	tcBuffer[28]=startTimePtr[3];

	tcBuffer[29]=endTimePtr[0];				// slew end time
	tcBuffer[30]=endTimePtr[1];
	tcBuffer[31]=endTimePtr[2];
	tcBuffer[32]=endTimePtr[3];

	tcBuffer[33]=itemToSetPtr[0];			// item to set
	tcBuffer[34]=itemToSetPtr[1];
	tcBuffer[35]=itemToSetPtr[2];
	tcBuffer[36]=itemToSetPtr[3];

	tcBuffer[37]=itemTimeTagPtr[0];			// time tag of item to set
	tcBuffer[38]=itemTimeTagPtr[1];
	tcBuffer[39]=itemTimeTagPtr[2];
	tcBuffer[40]=itemTimeTagPtr[3];

	// Run the TC loader
	tcl.run(0);

	// Check that one TC was loaded
	if (tcm.getNumberOfTelecommands()!=1)
	{	setTestResult(TEST_FAILURE, "Telecommand test 7 failed");
		return;
	}

	// Run the TC manager -- the manoeuvre should be loaded in the 
	// manoeuvre manager
	tcm.run(0);

	// Run the manoeuvre manager -- the manoeuvre should be initialized
	mnManager.run(0);

	// Advance the time 
	aocsClock.run(0);

	// Run the manoeuvre manager -- manoeuvre execution should begin
	mnManager.run(0);

	// Check that two TC events were generated (TC was loaded and TC
	// was executed)
	if (tcm.getTelecommandEventRepository()->getCounter()!=tcEvtN+2)
	{	setTestResult(TEST_FAILURE, "Telecommand test 7 failed");
		return;
	}

	// Check that the manoeuvre was loaded in the manoeuvre manager
	if (mnManager.getManoeuvreEventRepository()->latestManoeuvreEvent()->
			getEventType()!=MANOEUVRE_STARTED)
	{	setTestResult(TEST_FAILURE, "Telecommand test 7 failed");
		return;
	}

	// Check that the data item and its time tag were updated by the 
	// attitude slew
	if (itemToSet == 0)
	{	setTestResult(TEST_FAILURE, "Telecommand test 7 failed");
		return;
	}

	if (itemTimeTag == 0)
	{	setTestResult(TEST_FAILURE, "Telecommand test 7 failed");
		return;
	}

	// If this point is reached, test was successful
	setTestResult(TEST_SUCCESS,NULL);
	return;
}

